Package: TSAR
Type: Package
Title: Thermal Shift Analysis in R
Version: 1.11.0
Year: 2025
Authors@R: c(person("Xinlin", 
			"Gao", 
			email = "candygao2015@outlook.com", 
			role = c("aut", "cre"),
			comment = c(ORCID = "0009-0002-2518-235X")),
		person(c("William", "M."), 
			"McFadden", 
			email = "wmm27@pitt.edu", 
			role = c("aut","fnd"),
			comment = c(ORCID = "0000-0001-6911-2172")),
		person(c("Zhijiang", "Ye"), 
			"Ye", 
			email = "zhijiang.ye@emory.edu", 
			role = c("aut","fnd"),
			comment = c(ORCID = "0009-0005-5464-7306")),
		person(c("Stefan", "G."),
			"Sarafianos",
			email = "stefanos.sarafianos@emory.edu",
			role = c("fnd","aut","ths"),
			comment = c(ORCID = "0000-0002-5840-154X"))
		)
Description: This package automates analysis workflow for Thermal Shift 
    Analysis (TSA) data. Processing, analyzing, and visualizing data through 
    both shiny applications and command lines. Package aims to simplify 
	data analysis and offer front to end workflow, from raw data to 
	multiple trial analysis.
License: AGPL-3
Encoding: UTF-8
LazyData: false
Testthat: true
RoxygenNote: 7.3.2
Imports: dplyr (>= 1.0.7), ggplot2 (>= 3.3.5), ggpubr (>= 0.4.0),
        magrittr (>= 2.0.3), mgcv (>= 1.8.38), readxl (>= 1.4.0),
        stringr (>= 1.4.0), tidyr (>= 1.1.4), utils (>= 4.3.1), shiny
        (>= 1.7.4.1), plotly (>= 4.10.2), shinyjs (>= 2.1.0), jsonlite
        (>= 1.8.7), rhandsontable (>= 0.3.8), openxlsx (>= 4.2.5.2),
        shinyWidgets (>= 0.7.6), minpack.lm (>= 1.2.3)
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
biocViews: Software, ShinyApps, Visualization, qPCR
DataRaw: data/qPCR_data1.rda
DataRaw2: data/qPCR_data2.rda
DataRaw3: data/Well_Information.rda
DataRaw4: data/Well_Information_Template.rda
DataRawr: data/example_tsar_data.rda
Depends: R (>= 4.3.0)
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/TSAR
git_branch: devel
git_last_commit: 5ead14f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:41:00 UTC; biocbuild
Author: Xinlin Gao [aut, cre] (ORCID: <https://orcid.org/0009-0002-2518-235X>),
  William M. McFadden [aut, fnd] (ORCID:
    <https://orcid.org/0000-0001-6911-2172>),
  Zhijiang Ye Ye [aut, fnd] (ORCID:
    <https://orcid.org/0009-0005-5464-7306>),
  Stefan G. Sarafianos [fnd, aut, ths] (ORCID:
    <https://orcid.org/0000-0002-5840-154X>)
Maintainer: Xinlin Gao <candygao2015@outlook.com>

# Generated by roxygen2: do not edit by hand

export(TSA_Tms)
export(TSA_average)
export(TSA_boxplot)
export(TSA_compare_plot)
export(TSA_ligands)
export(TSA_proteins)
export(TSA_wells_plot)
export(Tm_difference)
export(Tm_est)
export(analyze_norm)
export(condition_IDs)
export(gam_analysis)
export(get_legend)
export(graph_tsar)
export(join_well_info)
export(merge_TSA)
export(merge_norm)
export(model_beta)
export(model_boltzmann)
export(model_fit)
export(model_gam)
export(normalize)
export(normalize_fluorescence)
export(read_analysis)
export(read_raw_data)
export(read_tsar)
export(remove_raw)
export(rescale)
export(run_boltzmann)
export(screen)
export(view_deriv)
export(view_model)
export(weed_raw)
export(well_IDs)
export(write_tsar)
import(ggplot2)
import(shiny)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n_distinct)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(ggpubr,ggarrange)
importFrom(jsonlite,toJSON)
importFrom(magrittr,"%>%")
importFrom(mgcv,gam)
importFrom(minpack.lm,nlsLM)
importFrom(openxlsx,read.xlsx)
importFrom(plotly,animation_opts)
importFrom(plotly,event_data)
importFrom(plotly,event_register)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(plotly,subplot)
importFrom(readxl,read_excel)
importFrom(rhandsontable,rHandsontableOutput)
importFrom(rhandsontable,rhandsontable)
importFrom(shinyWidgets,actionBttn)
importFrom(shinyWidgets,materialSwitch)
importFrom(shinyjs,hidden)
importFrom(shinyjs,runjs)
importFrom(shinyjs,toggle)
importFrom(shinyjs,useShinyjs)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_replace_all)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,unite)
importFrom(utils,write.csv)
importFrom(utils,write.table)

readme file
================

<img src="vignettes/images/TSAR_logo.png" width="400px" />


> **TSAR 1.9.3 update, December 2025**
> Introduction of cubic spline with beta-knots for smoother curve fitting
> see [`TSAR_Beta_Method_Introduction.Rmd`](vignettes/TSAR_Beta_Method_Introduction.Rmd)

# TSAR

    TSAR, short for Thermal Shift Analysis in R, presents the simple interface to TSA data analysis and visualization.
    TSAR Package is contains three separate shiny application regarding, data pre-processing, data analysis, and data visualization. All application can be opened in both interactive window or browsing engine by copy pasting server address into web browser. All functions can be run outside shiny applications, refer to vignette, "TSAR Workflow by Command" for instructions.
    Download TSAR package from Bioconductor branch using the below command lines.

``` r
library(BiocManager)
BiocManager::install("TSAR")
library(TSAR)
library(shiny)
```

## Data Pre-Processing

    Load data using RStudio import, read.csv, or read.delim. Select and preview raw curve using `weed_raw()`.

``` r
data("qPCR_data1")
```

```
runApp(weed_raw(qPCR_data1))
```

``` r
qPCR_data1 <- remove_raw(qPCR_data1, removerange = c("B", "H", "1", "12"))
```

    In the plot panel, user can interact with graph and select and un-select data by clicking on curves. Selection by grid is also possible. Click on grid to highlight wells of selection. Click again to un-highlight. User may also `View Selected` wells only and `Remove Selected`. To return to default page and current all change to data set, click `Refresh Screening`. To propagate the same changes to local data set, click `Copy Selected in Full Function Call` to copy a `remove_raw()` call containing all selected wells into clipboard. Paste the call back into console or script and run to propagate changes locally. 
    Use `View Selected` and `Remove Selected` to view and remove selection. To return to viewing rest of the data, click `Refresh Screening`. To propagate the same changes to local data set, click `Copy Selected in Full Function Call` to copy a `remove_raw()` call containing all selected wells into clipboard. Paste the call back into console or script and run to propagate changes locally.

## Data Analysis

    Analyze data by calling function `analyze_norm()` and follow the workflow from top to bottom. Preview data table for changes occurring at each step and refer to graph to view fit of model on each well. Remember to save analysis output locally by clicking `Save File`. Always preview data before saving to ensure data contains all necessary information.

``` r
runApp(analyze_norm(qPCR_data1))
```

    The top left panel output a preview of current data set. The right panel allows user to view the fit of model and Tm estimation by individual wells of selection. Once confirming correct data input and modeling effects,
    Click `Analyze all Wells` to propagate model and analysis onto the rest of data. A preview of analyzed data will also be modeling and analyzing all 96 Wells will take between 30 to 50 seconds.
    Upload well information by excel template and preview to confirm if information is correct. Use `Preview` to preview uploaded information and directly edit inside the window. User may also choose `Manual Input` to put in all condition information by Well. Make sure to hit `Save Changes` after editing and click `Set Conditions` to link all data to the conditions. A success message should be prompted.
    Lastly, to save all analysis locally, click `Save File` after previewing output. 

## Data Visualization

    Use function `graph_tsar()` to start a Shiny application for graphing options. Run `na.omit()` on data if error occurs. Three graphing options are allowed, boxplot of Tm, compare plots, and conditions plot.
    Function takes optional data parameter. If analysis file is already imported in the environment, call function as `graph_tsar(tsar_data)`. Else, user may use the merge data panel to upload and merge data of all test trials. Simply call `graph_tsar()` and click `Upload and Merge Data` button to reveal the panel to merge data.

``` r
runApp(graph_tsar())
```

    Upload is limited to 30MB in data size, but not count size. After upload, user will be prompted corresponding numbers of input boxes to specify date of each experiment. Click `Save Dates`, then `Merge and Save Data`. A short preview of tsar_data will be prompted. For a full list of well_ID and conditions_ID, refer to the helper buttons at the bottom of page.
    Top panel outputs all plots, selected desired graphing features below and click generate to output graphs. p.s. Graphing compare plots and selected curves are takes longer than boxplot, please give it few seconds to load.
    Generating compare plots will output all plausible comparisons by control. To any specific one, a drop list `View Only:` will be prompted below the botton `Generate Compare Plots`. Select by condition_ID to zoom in on graphs.

## Funding

    This research was supported in part by the National Institutes of Health (R01 AI120860 and U54 AI170855 to S.G.S.; W.M.M. was supported in part by T32 GM135060 and F31 AI174951). S.G.S. acknowledges funding from the Nahmias-Schinazi Distinguished Chair in Research. Any opinions, findings, conclusions, or recommendations expressed in this material are those of the authors and do not necessarily reflect the view of the funding agencies.  

#' Weed raw data for corrupt curves
#'
#' The weed_raw function allows users to interact with a screening graph
#'     and select curves to weed out before entering analysis. Function wraps
#'     together \code{\link[TSAR]{screen}} and \code{\link{remove_raw}}.
#'
#' @importFrom plotly ggplotly plotlyOutput renderPlotly event_data
#'     event_register
#' @import shiny
#' @import ggplot2
#' @importFrom shinyjs runjs useShinyjs
#' @importFrom jsonlite toJSON
#' @importFrom shinyWidgets materialSwitch
#'
#' @export
#'
#' @param raw_data The raw data for screening.
#' @param checkrange list type input identifying range of wells to select.
#'     For example, if viewing first 8 wells from row A to C is needed, one can
#'     specify the row letters and column numbers like this:
#'     \code{checkrange = c("A", "C", "1", "8")}
#' @param checklist use this parameter to view selected Wells with full
#'     Well names. For example, \code{checklist = c('A01', 'D11')}
#'
#' @return prompts separate app window for user interaction,
#'     does not return specific value
#'
#' @family data_preprocess
#'
#' @seealso \code{\link[TSAR]{screen}} and \code{\link{remove_raw}}
#'
#' @examples
#' data("qPCR_data1")
#' if (interactive()) {
#'     runApp(weed_raw(qPCR_data1, checkrange = c("A", "B", "1", "12")))
#' }
#'
weed_raw <- function(raw_data,
                     checkrange = NULL,
                     checklist = NULL) {
    ui <- mainpage(raw_data, data_name = deparse(substitute(raw_data)))

    server <- function(input, output) {
        clicked_points <- reactiveValues(legend_text = NULL)
        highlighted_cells <- reactiveVal(NULL)
        unhighlighted_cells <- reactiveVal(NULL)
        dataset <- reactiveVal(raw_data)
        gg1 <- reactiveVal(NULL)

        observe_grid(
            input, output, highlighted_cells,
            unhighlighted_cells
        )
        clear_selection(
            input, output, clicked_points,
            highlighted_cells, unhighlighted_cells
        )
        unhighlight_grid(input, output, highlighted_cells)
        highlight_grid(input, output, highlighted_cells)
        toggle_grid(input, output)
        print_click(input, output, clicked_points)
        copy_click(input, output, highlighted_cells, clicked_points)
        copy_click_full(
            input, output, highlighted_cells,
            clicked_points
        )
        stop_window(input, output, dataset)
        remove_selected(
            input, output, dataset, checkrange, checklist,
            gg1, highlighted_cells, clicked_points
        )
        refresh(input, output, dataset, checkrange, checklist, gg1)
        register_click(input, output, clicked_points, gg1)
        view_selected(
            input, output, dataset, checkrange, checklist,
            highlighted_cells, clicked_points, gg1
        )
        saved_weeded(input, output, dataset)
        print_plot(input, output, dataset, checkrange, checklist, gg1)
    }
    app <- shinyApp(ui = ui, server = server)
}

render_message <- function(input, output) {
    output$Plot_Message <- renderText({
        c(
            "Select one of the following graph options and click generate.",
            "\nGraphing takes few seconds to load, please wait :)"
        )
    })
}

tohide <- function(input, output) {
    output$hideBtn <- renderUI({
        NULL
    })
}

totoggle <- function(input, output) {
    output$hideBtn <- renderUI({
        shinyWidgets::actionBttn(
            inputId = "hideBtn", label = "hide",
            style = "minimal", color = "success", size = "xs"
        )
    })
}

build_dateboxes <- function(input, output) {
    output$date_boxes <- renderUI({
        date_boxes <- lapply(input$file$name, function(i) {
            shiny::dateInput(
                inputId = paste0("Date for file ", i),
                label = paste0("Date for file ", i)
            )
        })
        do.call(tagList, date_boxes)
    })
}

build_table <- function(input, output, graph_tsar_data) {
    output$table <- renderTable({
        data.frame(head(graph_tsar_data()))
    })
}

render_derivative <- function(input, output, graph_tsar_data) {
    shinyjs::hide("bplot")
    shinyjs::show("alternativeplot")
    output$altplot <- renderPlotly({
        df <- graph_tsar_data()
        # Compute derivative if missing, using RFU if available, else Normalized, else Fluorescence
        if (!"norm_deriv" %in% names(df)) {
            y_col <- if ("RFU" %in% names(df)) {
                "RFU"
            } else if ("Normalized" %in% names(df)) {
                "Normalized"
            } else if ("Fluorescence" %in% names(df)) {
                "Fluorescence"
            } else NA_character_
            if (is.na(y_col)) {
                stop("No intensity column found to compute derivative (expected 'RFU', 'Normalized', or 'Fluorescence').")
            }
            if (!"well_ID" %in% names(df) && "Well.Position" %in% names(df)) {
                df$well_ID <- df$Well.Position
            }
            by_well <- split(df, df$well_ID)
            parts <- lapply(by_well, function(w) {
                # ensure numeric Temperature
                w$Temperature <- suppressWarnings(as.numeric(w$Temperature))
                o <- order(w$Temperature)
                w <- w[o, ]
                dy <- diff(as.numeric(w[[y_col]]))
                dx <- diff(as.numeric(w$Temperature))
                deriv <- rep(NA_real_, nrow(w))
                if (length(dy) > 0 && length(dx) == length(dy)) {
                    dd <- dy / dx
                    dd[!is.finite(dd)] <- NA_real_
                    deriv[seq_along(dd)] <- dd
                }
                w$norm_deriv <- deriv
                w
            })
            df <- do.call(rbind, parts)
        }
        # Ensure Tm column exists for label mapping inside view_deriv
        if (!"Tm" %in% names(df)) {
            df$Tm <- NA_real_
        }
        view_deriv(df, frame_by = input$frame_by)
    })
}

render_box <- function(input, output, box) {
    if (input$Legend == FALSE) {
        if (input$makeplotly == TRUE) {
            shinyjs::hide("bplot")
            shinyjs::show("alternativeplot")
            output$altplot <- renderPlotly({
                boxp <- box + labs(y = "Tm (degree Celsius)")
                boxp <- plotly::ggplotly(boxp, originalData = TRUE)
                plotly::layout(boxp,
                    yaxis = list(title_font = list(size = 18)),
                    xaxis = list(title_font = list(size = 18))
                )
            })
        } else {
            shinyjs::show("bplot")
            shinyjs::hide("alternativeplot")
            output$Plot <- renderPlot({
                box + theme(text = element_text(size = 18))
            })
        }
    } else {
        shinyjs::hide("alternativeplot")
        shinyjs::show("bplot")
        output$Plot <- renderPlot({
            box[[1]] <- box[[1]] + theme(
                text = element_text(size = 18)
            )
            ggarrange(
                plotlist = box,
                font.label = list(size = 20)
            )
        })
    }
}

redner_compare <- function(input, output, compare) {
    shinyjs::show("bplot")
    shinyjs::hide("alternativeplot")
    output$Plot <- renderPlot({
        ggarrange(plotlist = compare(), common.legend = TRUE)
    })
    output$plot_select <- renderUI({
        selectInput("plot_selected",
            label = "View Only:",
            choices = c("Select Option..", names(compare()))
        )
    })
}

render_selected_compare <- function(input, output, compare) {
    shinyjs::show("bplot")
    shinyjs::hide("alternativeplot")
    output$Plot <- renderPlot({
        compare()[[input$plot_selected]]
    })
}

render_selected_condition <- function(input, output, curve_graph) {
    shinyjs::show("bplot")
    shinyjs::hide("alternativeplot")
    output$Plot <- renderPlot({
        sep_flag <- tolower(as.character(input$separate_legend)) == "true"
        if (!isTRUE(sep_flag)) {
            # curve_graph is a ggplot
            curve_graph + theme(text = element_text(size = 18))
        } else {
            # curve_graph is a list
            curve_graph[[1]] <- curve_graph[[1]] + theme(text = element_text(size = 18))
            ggpubr::ggarrange(plotlist = curve_graph)
        }
    })
}

print_condition <- function(input, output, graph_tsar_data) {
    output$Condition_ID <- renderPrint({
        condition_IDs(graph_tsar_data())
    })
}

print_well <- function(input, output, graph_tsar_data) {
    output$Well_ID <- renderPrint({
        well_IDs(graph_tsar_data())
    })
}

print_tm <- function(input, output, graph_tsar_data) {
    output$tmlist <- renderTable({
        TSA_Tms(graph_tsar_data())
    })
}

print_deltatm <- function(input, output, graph_tsar_data) {
    output$tmlist <- renderTable({
        Tm_difference(graph_tsar_data(),
            control_condition = input$control_tm
        )
    })
}

print_dates <- function(input, output, saved_dates) {
    output$output <- renderPrint({
        saved_dates
    })
}

dummy_plot <- function(input, output) {
    waiting_data <- data.frame(
        x = 0.5, y = 0.5,
        message = "Awaiting graph :)"
    )
    output$Plot <- renderPlot({
        ggplot(waiting_data, aes(x = x, y = y)) +
            geom_point(alpha = 0) +
            geom_text(aes(label = message), size = 18) +
            theme_void()
    })
}

render_data <- function(input, output, raw_data) {
    output$table <- renderDataTable(
        {
            data.frame(raw_data)
        },
        options = list(pageLength = 7)
    )
}

render_data_title <- function(input, output, title) {
    output$table_title <- renderText({
        title
    })
}

build_view <- function(input, output, gg) {
    gg[[1]] <- ggplotly(gg[[1]], width = 500)
    gg[[2]] <- ggplotly(gg[[2]], width = 500)
    gg[[1]] <- plotly::layout(gg[[1]],
        yaxis = list(tickfont = list(size = 8), showgrid = TRUE)
    )
    gg[[2]] <- plotly::layout(gg[[2]],
        yaxis = list(tickfont = list(size = 8), showgrid = TRUE)
    )
    output$Plot <- renderPlotly({
        sub <- plotly::subplot(gg[[1]], gg[[2]],
            nrows = 2, shareX = TRUE,
            shareY = TRUE
        )
        plotly::layout(sub,
            legend = list(
                orientation = "h", x = 0, y = 1.15,
                font = list(size = 8)
            ),
            xaxis = list(tickfont = list(size = 8), showgrid = TRUE)
        )
    })
}

build_well <- function(input, output, hide = FALSE) {
    output$excelTable <- rhandsontable::renderRHandsontable({
        if (hide == TRUE) {
            NULL
        } else {
            rhandsontable(data.frame(head(well_information_template, 9)))
        }
    })
}

#' TSAR Condition IDs
#'
#' This function is used to extract information of the condition IDs from a
#'     loaded TSA Analysis Data file. Condition IDs are automatically generated
#'     by the read_analysis function in the automated workflow. This returns
#'     either a character vector of unique IDs present or a numeric value of the
#'     number of unique IDs.
#'
#'
#' @param analysis_data a data frame that is unmerged and generated by
#'     TSAR::read_analysis() or a merged TSA data frame generated by
#'     TSAR::merge_TSA(). Data frames require a column named 'condition_ID'.
#' @param n logical value; \code{n = FALSE} by default. When TRUE, a numeric
#'     value of unique IDs is returned. When FALSE, a character vector of unique
#'     IDs are returned.
#'
#' @return Either a character vector of condition_IDs or a numeric value.
#'
#' @examples
#' data("example_tsar_data")
#' condition_IDs(example_tsar_data)
#'
#' @family TSA Summary Functions
#' @seealso \code{\link{merge_TSA}} and \code{\link{read_analysis}}
#' for preparing input.
#' @export
#'
condition_IDs <- function(
    analysis_data, # analysis data to determine number of unique conditions
    n = FALSE # c("T", "F") F = vector, T = number summary
    ) {
    if (n) {
        return(length(unique(analysis_data$condition_ID)))
    } else {
        return(unique(analysis_data$condition_ID))
    }
}




#' TSAR Well IDs
#'
#' This function is used to extract information of the well IDs from a
#'     merged TSA data frame. Well IDs are automatically generated
#'     by the read_analysis and read_raw_data functions in the automated
#'     workflow. This function returns
#'     either a character vector of unique IDs present or a numeric value of the
#'     number of unique IDs.
#'
#'
#' @param tsa_data a data frame that is merged and generated by
#'     TSAR::merge_TSA(), or an unmerged data frame read by
#'     TSAR::read_analysis() or TSAR::read_raw_data().
#'     Data frames require a column named 'well_ID'.
#' @param n logical value; \code{n = FALSE} by default. When TRUE, a numeric
#'     value of unique IDs is returned. When FALSE, a character vector of unique
#'     IDs are returned.
#'
#' @return Either a character vector of unique well IDs or a numeric value.
#'
#' @family TSA Summary Functions
#' @seealso \code{\link{merge_TSA}}, \code{\link{read_raw_data}},
#' and \code{\link{read_analysis}} for preparing input.
#'
#' @examples
#' data("example_tsar_data")
#' well_IDs(example_tsar_data)
#'
#' @export

well_IDs <- function(
    tsa_data, # tsa data to determine number of unique conditions
    n = FALSE # c("T", "F") F = List, T = number summary
    ) {
    if (n) {
        return(length(unique(tsa_data$well_ID)))
    } else {
        return(unique(tsa_data$well_ID))
    }
}

#' TSA Proteins
#'
#' This function is used to extract information from a data frame of TSA data.
#'     The Protein values should be assigned in the TSA software.
#'
#' @param tsa_data a data frame that is merged and generated by
#'     TSAR::merge_TSA(), or an unmerged data frame read by
#'     TSAR::read_analysis() or TSAR::read_raw_data().
#'     The data frame must have a column named 'Protein'.
#' @param n logical value; \code{n = FALSE} by default. When TRUE, a numeric
#'     value describing the number of unique protein names is returned.
#'     When FALSE, a character vector of unique IDs are returned.
#'
#' @return Either a character vector of unique well_IDs or a numeric value.
#'
#' @family TSA Summary Functions
#' @seealso \code{\link{merge_TSA}}, \code{\link{read_raw_data}},
#'     and \code{\link{read_analysis}} for preparing input.
#'
#' @examples
#' data("example_tsar_data")
#' TSA_proteins(example_tsar_data)
#'
#' @export

TSA_proteins <- function(
    tsa_data, # tsa data to determine number of unique conditions
    n = FALSE # c("T", "F") F = List, T = number summary
    ) {
    if (n) {
        return(length(levels(factor(tsa_data$Protein))))
    } else {
        return(levels(factor(tsa_data$Protein)))
    }
}

#' TSA Ligands
#'
#' This function is used to extract information from a data frame of TSA data.
#'     The Ligand values should be assigned in the TSA software.
#'
#' @param tsa_data a data frame that is merged and generated by
#'     TSAR::merge_TSA(), or an unmerged data frame read by
#'     TSAR::read_analysis() or TSAR::read_raw_data().
#'     The data frame must have a column named 'Ligand'.
#' @param n logical value; \code{n = FALSE} by default. When TRUE, a numeric
#'     value describing the number of unique ligand names is returned.
#'     When FALSE, a character vector of unique IDs are returned.
#' @return Either a character vector of unique well_IDs or a numeric value.
#' @family TSA Summary Functions
#' @seealso \code{\link{merge_TSA}}, \code{\link{read_raw_data}},
#'     and \code{\link{read_analysis}} for preparing input.
#'
#' @examples
#' data("example_tsar_data")
#' TSA_ligands(example_tsar_data)
#'
#' @export

TSA_ligands <- function(
    tsa_data, # tsa data to determine number of unique conditions
    n = FALSE # c("T", "F") F = List, T = number summary
    ) {
    if (n) {
        return(length(levels(factor(tsa_data$Ligand))))
    } else {
        return(levels(factor(tsa_data$Ligand)))
    }
}

test_that("Test remove_raw", {
    data("qPCR_data1")
    output <- remove_raw(qPCR_data1, removelist = c("A01", "D11"))
    result <- ("A01" %in% output$Well.Position)
    expect_equal(result, FALSE)
})

test_that("Test merge_norm for correct merging", {
    data("qPCR_data1")
    result1 <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
    data("well_information")
    norm_data <- join_well_info(
        file_path = NULL, file = well_information,
        read_tsar(result1, output_content = 2), type = "by_template"
    )
    norm_data <- na.omit(norm_data)
    data("qPCR_data2")
    result2 <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
    norm_data2 <- join_well_info(
        file_path = NULL, file = well_information,
        read_tsar(result2, output_content = 2), type = "by_template"
    )
    norm_data2 <- na.omit(norm_data2)
    tsar_data <- merge_norm(
        data = list(norm_data, norm_data2),
        name = c("Thermal Shift_162.eds.csv", "Thermal Shift_168.eds.csv"),
        date = c("20230203", "20230209")
    )
    result <- ("well_ID" %in% names(tsar_data)) &&
        ("condition_ID" %in% names(tsar_data))
    expect_equal(result, TRUE)
})

test_that("Test join_well_info", {
    data("qPCR_data1")
    analysis <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
    data("well_information")
    output <- join_well_info(
        file_path = NULL, file = well_information,
        read_tsar(analysis, output_content = 2), type = "by_template"
    )
    result <- ("Protein" %in% names(output)) && ("Ligand" %in% names(output))
    expect_equal(result, TRUE)
})

test_that("Test read_tsar", {
    data("qPCR_data1")
    analysis <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
    data("well_information")
    output <- read_tsar(analysis, output_content = 2)
    result <- ("tm" %in% names(output))
    expect_equal(result, TRUE)
})

test_that("Test gam_analysis", {
    data("qPCR_data1")
    analysis <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
    result <- ("tm" %in% names(analysis[[1]]))
    expect_equal(result, TRUE)
})

test_that("Test normalize", {
    data("qPCR_data1")
    test <- subset(qPCR_data1, Well.Position == "A01")
    output <- normalize(test)
    result <- ("Normalized" %in% names(output))
    expect_equal(result, TRUE)
})

test_that("Test model_fit", {
    data("qPCR_data1")
    test <- subset(qPCR_data1, Well.Position == "A01")
    test <- normalize(test)
    gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
    output <- model_fit(test, model = gammodel)
    result <- ("norm_deriv" %in% names(output))
    expect_equal(result, TRUE)
})

test_that("Test Tm_est", {
    data("qPCR_data1")
    test <- subset(qPCR_data1, Well.Position == "A01")
    test <- normalize(test)
    gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
    fit <- model_fit(test, model = gammodel)
    result <- Tm_est(fit)
    expect_equal(result, 53.736416)
})

test_that("Test condition_IDs", {
    data("example_tsar_data")
    output <- condition_IDs(example_tsar_data)
    result <- output[1]
    expect_equal(result, "CA FL_DMSO")
})

test_that("Test well_IDs", {
    data("example_tsar_data")
    output <- well_IDs(example_tsar_data)
    result <- output[1]
    expect_equal(result, "A01_CA FL_DMSO_20230203")
})

test_that("Test TSA_proteins", {
    data("example_tsar_data")
    output <- TSA_proteins(example_tsar_data)
    result <- output[1]
    expect_equal(result, "CA FL")
})

test_that("Test TSA_ligands", {
    data("example_tsar_data")
    output <- TSA_ligands(example_tsar_data)
    result <- ("4-ABA" %in% output)
    expect_equal(result, TRUE)
})

test_that("Test Tm_difference", {
    data("example_tsar_data")
    control <- condition_IDs(example_tsar_data)[1]
    output <- Tm_difference(example_tsar_data, control_condition = control)
    result <- output$delta_Tm[1]
    expect_equal(result, 0)
})

test_that("Test TSA_Tms", {
    data("example_tsar_data")
    output <- TSA_Tms(example_tsar_data)
    result <- ("Avg_Tm" %in% names(output))
    expect_equal(result, TRUE)
})

test_that("Test TSA_average", {
    data("example_tsar_data")
    output <- TSA_average(example_tsar_data,
        y = "Fluorescence", digits = 1,
        avg_smooth = TRUE, sd_smooth = TRUE
    )
    result <- ("sd" %in% names(output)) && ("sd_max" %in% names(output))
    expect_equal(result, TRUE)
})

test_that("Test ", {
    input <- c(0, 1, 3)
    result <- rescale(input)
    expect_equal(result, c(0.0000000, 0.33333333, 1.0000000))
})

hide_p <- function(input, output) {
    shiny::observeEvent(input$hideBtn, {
        shinyjs::hide(id = "myPanel")
        tohide(input, output)
    })
}
toggle_p <- function(input, output) {
    shiny::observeEvent(input$toggleButton, {
        shinyjs::show(id = "myPanel")
        totoggle(input, output)
    })
}
initialize <- function(input, output, graph_tsar_data) {
    shiny::observe({
        if (length(graph_tsar_data()) == 0) {
            if (!input$dialogueToggle) {
                shiny::showModal(shiny::modalDialog(
                    title = "No data input :(",
                    "No data input: Please upload analysis files to merge
                            or close window \nand call function with data
                            included as parameter. e.g. graph_tsar(tsar_data)"
                ))
            }
        }
    })
}
build_dates <- function(input, output) {
    shiny::observeEvent(input$file, {
        build_dateboxes(input, output)
    })
}
save_dates <- function(input, output, dated, datelist) {
    shiny::observeEvent(input$save_dates, {
        saved_dates <- vapply(input$file$name, function(i) {
            as.character(input[[paste0("Date for file ", i)]])
        }, FUN.VALUE = character(1))
        datelist(saved_dates)
        if (!is.null(datelist)) {
            if (!input$dialogueToggle) {
                shiny::showModal(shiny::modalDialog(
                    title = "Dates saved",
                    "Confirm in the textbox below if saved dates are
                            correct and proceede to merging data."
                ))
                print_dates(input, output, saved_dates)
                dated(TRUE)
            }
        } else {
            if (!input$dialogueToggle) {
                shiny::showModal(shiny::modalDialog(
                    title = ":(",
                    "Please upload analysis files first before
                            setting dates."
                ))
            }
        }
    })
}
saved_merged <- function(input, output, graph_tsar_data) {
    shiny::observeEvent(input$savetolocal, {
        assign("tsar_data", graph_tsar_data(), envir = .GlobalEnv)
        if (!input$dialogueToggle) {
            shiny::showModal(shiny::modalDialog(
                title = "Data Saved Successfully",
                "Data saved to global environment of r console.
                Check for variable 'tsar_data'"
            ))
        }
    })
}
merge_update <- function(input, output, dated, graph_tsar_data,
                         datelist, session) {
    shiny::observeEvent(input$generate, {
        if ((dated() == FALSE) && (!input$dialogueToggle)) {
            shiny::showModal(shiny::modalDialog(
                title = "Dates are not saved",
                "please review and save dates of experiment!"
            ))
        } else {
            tsar_data_p <- merge_norm(
                data = input$file$datapath,
                name = input$file$name,
                date = datelist()
            )
            tsar_data_p <- na.omit(tsar_data_p)
            graph_tsar_data(tsar_data_p)
            build_table(input, output, graph_tsar_data)
            render_message(input, output)

            updateSelectInput(session, "Control",
                label = "Control Condition",
                choices = c("NA", condition_IDs(graph_tsar_data()))
            )
            updateSelectInput(session, "Control_s",
                label = "Control Condition",
                choices = c(condition_IDs(graph_tsar_data()))
            )
            updateSelectInput(session, "Selected_condition",
                label = "Select condition: ",
                choices = c(condition_IDs(graph_tsar_data()))
            )
            updateSelectInput(session, "control_tm",
                label = "List Delta Tm:",
                choices = c(
                    "Select Control: ",
                    condition_IDs(graph_tsar_data())
                )
            )
            updateSelectInput(session, "remove_condition",
                label = "Select By Condition_ID:",
                choices = condition_IDs(graph_tsar_data())
            )
            updateSelectInput(session, "remove_well",
                label = "Select By Well_ID:",
                choices = well_IDs(graph_tsar_data())
            )
        }
    })
}
build_boxplot <- function(input, output, graph_tsar_data) {
    shiny::observeEvent(input$Boxplot, {
        req(input$Boxplot)
        if (length(graph_tsar_data()) == 0) {
            if (!input$dialogueToggle) {
                shiny::showModal(shiny::modalDialog(
                    title = "No data input",
                    "No data input: Please upload analysis files to merge
                            or close window \nand call function with data
                            included as parameter. e.g. graph_tsar(tsar_data)"
                ))
            }
        } else {
            req(input$Boxplot)
            # Validate required columns for boxplot
            required_cols <- c("Tm", "condition_ID", "Protein", "Ligand", "well_ID")
            if (!all(required_cols %in% names(graph_tsar_data()))) {
                if (!input$dialogueToggle) {
                    shiny::showModal(shiny::modalDialog(
                        title = "Missing required columns",
                        paste0(
                            "Boxplot requires analyzed tsar_data with columns: ",
                            paste(required_cols, collapse = ", "),
                            ".\nUse 'Upload and Merge Data' to load analysis outputs or call graph_tsar(tsar_data)."
                        )
                    ))
                }
                return(invisible(NULL))
            }
            if (input$Control == "NA") {
                control_option_b <- NA
            } else {
                control_option_b <- as.character(input$Control)
            }
            # coerce Legend to logical
            legend_flag <- tolower(as.character(input$Legend)) == "true"
            box <- TSA_boxplot(graph_tsar_data(),
                color_by = input$Color,
                label_by = input$Label,
                separate_legend = legend_flag,
                control_condition = control_option_b
            )
            render_box(input, output, box)
        }
    })
}
build_compare <- function(input, output, graph_tsar_data, compare) {
    shiny::observeEvent(input$Compareplot, {
        if (length(graph_tsar_data()) == 0) {
            if (!input$dialogueToggle) {
                shiny::showModal(shiny::modalDialog(
                    title = "No data input",
                    "No data input: Please upload analysis files to merge
                            or close window \nand call function with data
                            included as parameter. e.g. graph_tsar(tsar_data)"
                ))
            }
        } else {
            show_tm_flag <- tolower(as.character(input$show_tm)) == "true"
            # map UI 'GAM'/'beta' to TSA_compare_plot smoother arg
            smoother_val <- if (!is.null(input$compare_smoother) && tolower(input$compare_smoother) == "beta") "beta" else "gam"
            compare_p <- TSA_compare_plot(graph_tsar_data(),
                y = input$y_axis,
                title_by = input$title_by,
                show_Tm = show_tm_flag,
                control_condition = input$Control_s,
                smoother = smoother_val
            )
            compare(compare_p)
            redner_compare(input, output, compare)
        }
    })
}
view_compare <- function(input, output, compare) {
    shiny::observeEvent(input$plot_selected, {
        if (input$plot_selected != "Select Option..") {
            render_selected_compare(input, output, compare)
        }
    })
}
build_curves <- function(input, output, graph_tsar_data) {
    shiny::observeEvent(input$curves, {
        if (length(graph_tsar_data()) == 0) {
            if (!input$dialogueToggle) {
                shiny::showModal(shiny::modalDialog(
                    title = "No data input",
                    "No data input: Please upload analysis files to merge
                            or close window \nand call function with data
                            included as parameter. e.g. graph_tsar(tsar_data)"
                ))
            }
        } else {
            selected_curves <- subset(
                graph_tsar_data(),
                condition_ID ==
                    input$Selected_condition
            )
            # coerce flags from selectInput
            show_tm_c_flag <- tolower(as.character(input$show_tm_c)) == "true"
            average_flag   <- tolower(as.character(input$average)) == "true"
            smooth_flag    <- tolower(as.character(input$smooth)) == "true"
            sep_legend_flag<- tolower(as.character(input$separate_legend)) == "true"
            smoother_curves <- if (!is.null(input$curves_smoother) && tolower(input$curves_smoother) == "beta") "beta" else "gam"
            curve_graph <- TSA_wells_plot(selected_curves,
                show_Tm = show_tm_c_flag,
                y = input$y_axis_c,
                show_average = average_flag,
                smooth = smooth_flag,
                Tm_label_nudge = input$num,
                separate_legend = sep_legend_flag,
                smoother = smoother_curves
            )
            render_selected_condition(input, output, curve_graph)
        }
    })
}
build_derivatives <- function(input, output, graph_tsar_data) {
    shiny::observeEvent(input$build_deriv, {
        render_derivative(input, output, graph_tsar_data)
    })
}

render_condition <- function(input, output, graph_tsar_data) {
    shiny::observeEvent(input$condition, {
        print_condition(input, output, graph_tsar_data)
    })
}
render_well <- function(input, output, graph_tsar_data) {
    shiny::observeEvent(input$well, {
        print_well(input, output, graph_tsar_data)
    })
}
render_tm <- function(input, output, graph_tsar_data) {
    shiny::observeEvent(input$tm, {
        if (input$control_tm == "N/A") {
            print_tm(input, output, graph_tsar_data)
        } else {
            print_deltatm(input, output, graph_tsar_data)
        }
    })
}
remove_selected_graph <- function(input, output, graph_tsar_data) {
    shiny::observeEvent(input$Remove, {
        cur <- graph_tsar_data()
        cur <- subset(cur, !condition_ID %in% input$remove_condition)
        cur <- subset(cur, !well_ID %in% input$remove_well)
        graph_tsar_data(cur)
        if (!input$dialogueToggle) {
            shiny::showModal(shiny::modalDialog(
                title = "Selection Removed",
                "All selected condition_ID and well_ID are removed.
                To revert to original data, click 'Restore Removal'."
            ))
        }
    })
}
restore_removal <- function(input, output, graph_tsar_data, stock_tsar_data) {
    shiny::observeEvent(input$Restore, {
        graph_tsar_data(stock_tsar_data())
        if (!input$dialogueToggle) {
            shiny::showModal(shiny::modalDialog(
                title = "All removals restored",
                "Data restored to original status."
            ))
        }
    })
}
closegraph <- function(input, output) {
    shiny::observeEvent(input$stopButton, {
        shiny::stopApp()
    })
}

#' Reformat TSA data into TSA Tms
#'
#' This function is used to output calculated Tm data from TSA analysis. The
#'     input data frame will be transformed into
#'     a new format that is helpful for user reading and automated analysis.
#'     The Tm values can be listed as a data frame of individual wells or the
#'     Tms from identical conditions can be averaged. When condition_average
#'     is TRUE (the default), samples with identical condition IDs will be
#'     aggregated and the average / standard deviation will be calculated
#'     where appropriate. To analyze multiple TSA experiments, use merge_TSA()
#'     to make a single data frame for analysis.
#'
#' @importFrom  dplyr distinct bind_rows
#'
#' @param analysis_data a data frame that is unmerged and generated by
#'     TSAR::read_analysis() or a merged TSA data frame generated by
#'     TSAR::merge_TSA(). Data frames require a column named 'condition_ID' for
#'     averaging.
#' @param condition_average logical value; \code{n = TRUE} by default.
#'     When TRUE, the average Tm is calculated by matched condition IDs within
#'     the data frame. When FALSE, each well is reported as a unique value with
#'     the corresponding Tm.
#' @return A data frame of Tm values.
#' @family TSAR Formatting
#' @seealso \code{\link{merge_TSA}}, \code{\link{read_raw_data}},
#'     and \code{\link{read_analysis}} for preparing input.
#' @examples
#' data("example_tsar_data")
#' TSA_Tms(example_tsar_data)
#' @export
#'

TSA_Tms <- function(
    analysis_data,
    condition_average = TRUE) {
    if (condition_average) { #--- Code to average by condition
        n_conditions <- condition_IDs(analysis_data, n = TRUE)
        tm_df_list <- as.data.frame(matrix(ncol = 0, nrow = 1))
        condition_data_list <- lapply(1:n_conditions, function(i) {
            condition_i <- condition_IDs(analysis_data)[i]
            analysis_data_i <- analysis_data[analysis_data$condition_ID == condition_i, ]
            tm_df_i <- distinct(analysis_data_i, well_ID, Tm)
            n_tms_i <- nrow(tm_df_i)
            avg_tm_i <- mean(tm_df_i$Tm, na.rm = TRUE)
            sd_tm_i <- sd(tm_df_i$Tm, na.rm = TRUE)

            tm_df_list[i, seq_len((n_tms_i + 3))] <-
                c(condition_i, avg_tm_i, sd_tm_i, tm_df_i$Tm)
            tm_df_list[i,]
        })
        tm_df <- bind_rows(condition_data_list)


        #--- Format col names and data type
        names(tm_df) <- c(
            "condition_ID", "Avg_Tm", "SD_Tm",
            paste("Tm_", (4:ncol(tm_df) - 3))
        )
        tm_df[, seq.int(2, ncol(tm_df))] <-
            lapply(tm_df[, seq.int(2, ncol(tm_df))], as.numeric)
        #--- Matching values to get Protein and Ligand into table
        tm_df$Protein <-
            analysis_data$Protein[match(
                tm_df$condition_ID,
                analysis_data$condition_ID
            )]
        tm_df$Ligand <-
            analysis_data$Ligand[match(
                tm_df$condition_ID,
                analysis_data$condition_ID
            )]
    } else { # When only reporting wells, not conditions
        tm_df <- dplyr::distinct(
            analysis_data, well_ID, Tm,
            condition_ID, Protein, Ligand
        )
    }
    return(tm_df)
}

#' Well information input function
#'
#' Reads in the ligand and protein information and joins them accordingly
#'     to the big data frame for graphing purposes.
#'
#' @importFrom magrittr %>%
#' @importFrom readxl read_excel
#' @importFrom tidyr pivot_longer
#' @importFrom dplyr mutate select left_join
#'
#' @param file_path string; file path to read in the file
#' @param file object; use file to override the need of file_path if
#'     information is already read in
#' @param analysis_file data frame; data frame containing smoothed fluorescence
#'     data and tm values
#' @param skips integer; number indicating the number of headers present in
#'     input file, default set to 0 when file input is "by_well"
#'     If the input follows the excel template, this parameter does not apply.
#' @param nrows integer; number indicating the number of rows the data is.
#'     Default set to 96 assuming analysis on 96 well plate. Parameter is only
#'     applicable when file input is "by_well". If inputting by excel
#'     template, this parameter does not apply, please ignore.
#' @param type string; variable specifies the type of input read in.
#'     type = "by_well" requires input of csv or txt files of three variables:
#'     Well, Protein, Ligand.
#'     type = "by_template" requires input of excel file following the template
#'     format provided
#' @return outputs data frame joining data information with well information
#'
#' @family read_write_analysis
#' @examples
#' data("qPCR_data1")
#' result <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
#' data("well_information")
#' join_well_info(
#'     file_path = NULL, file = well_information,
#'     read_tsar(result, output_content = 2), type = "by_template"
#' )
#'
#' @export
join_well_info <- function(
    file_path,
    file = NULL,
    analysis_file,
    skips = 0,
    nrows = 96,
    type) {
    # input by well
    type <- match.arg(type, choices = c("by_well", "by_template"))

    if (type == "by_well") {
        well_info <- utils::read.delim(
            header = TRUE,
            skip = skips,
            nrow = nrows,
            file_path
        )
        well_info <- well_info %>%
            dplyr::select(c("Well", "Protein", "Ligand"))
        # input by excel template
    } else if (type == "by_template") {
        # read file, specify column as texts
        if (!is.null(file) && length(file) != 0) {
            well_info_table <- file
        } else {
            well_info_table <- readxl::read_excel(file_path,
                col_types =
                    c(
                        "text", "text", "text", "text", "text", "text",
                        "text", "text", "text", "text", "text", "text",
                        "text", "text", "text", "text", "text", "text",
                        "text", "text", "text", "text", "text", "text",
                        "text"
                    )
            )
        }

        # remove header row
        well_info_table <- well_info_table[-c(1), ]
        # seperate protein and ligand information into two data frames
        protein <- well_info_table[, c(
            1, 2, 4, 6, 8, 10, 12,
            14, 16, 18, 20, 22, 24
        )]
        ligand <- well_info_table[, -c(
            2, 4, 6, 8, 10, 12, 14,
            16, 18, 20, 22, 24
        )]
        # rename information by well
        names(protein) <-
            c(
                "Col", "01", "02", "03", "04", "05",
                "06", "07", "08", "09", "10", "11", "12"
            )
        names(ligand) <-
            c(
                "Col", "01", "02", "03", "04", "05",
                "06", "07", "08", "09", "10", "11", "12"
            )

        # pivot table to place values into correct positions
        protein <- protein %>%
            tidyr::pivot_longer(
                cols = !Col,
                names_to = "pos",
                values_to = "Protein"
            ) %>%
            mutate(Well = paste(Col, pos, sep = "")) %>%
            dplyr::select(Well, Protein)

        ligand <- ligand %>%
            tidyr::pivot_longer(
                cols = !Col,
                names_to = "pos",
                values_to = "Ligand"
            ) %>%
            mutate(Well = paste(Col, pos, sep = "")) %>%
            dplyr::select(Well, Ligand)

        # join protein and ligand data frames by well
        well_info <- left_join(protein,
            ligand,
            by = "Well"
        )
    }

    # join well information to big data frame
    combined <- left_join(analysis_file,
        well_info,
        by = c("Well.Position" = "Well")
    )

    combined <- na.omit(combined)
    return(combined)
}

observe_grid <- function(input, output,
                         highlighted_cells, unhighlighted_cells) {
    shiny::observe({
        for (i in seq_len(96)) {
            shinyjs::hide("hidden")
            shinyjs::runjs(
                sprintf("$('#cell-%d').click(function() {
                        if ($(this).css('background-color') ===
                        'rgb(255, 255, 255)') {
                        $(this).css('background-color', 'lightgrey');
                        var cellId = $(this).attr('id');
                        var label = $(this).text().trim();
                        Shiny.onInputChange('highlighted_cells',
                        {id: cellId, label: label});
                        } else {
                        $(this).css('background-color', 'white');
                        var cellId = $(this).attr('id');
                        var label = $(this).text().trim();
                        Shiny.onInputChange('highlighted_cells', null);
                        Shiny.onInputChange('unhighlighted_cells',
                        {id: cellId, label: label});
                        }
                        });", i)
            )
        }
    })
}
clear_selection <- function(input, output, clicked_points,
                            highlighted_cells, unhighlighted_cells) {
    shiny::observeEvent(input$clear, {
        clicked_points$legend_text <- NULL
        highlighted_cells(NULL)
        unhighlighted_cells(NULL)
        print_grid(input, output, highlighted_cells)
        print_click(input, output, clicked_points)
    })
}
unhighlight_grid <- function(input, output, highlighted_cells) {
    shiny::observeEvent(input$unhighlighted_cells, {
        cell_id <- input$unhighlighted_cells$id
        cell_label <- input$unhighlighted_cells$label
        if (cell_label %in% unlist(highlighted_cells())) {
            current_values <- highlighted_cells()
            updated_values <- current_values[current_values
            != cell_label]
            highlighted_cells(updated_values)
            print_grid(input, output, highlighted_cells)
        }
    })
}
highlight_grid <- function(input, output, highlighted_cells) {
    shiny::observeEvent(input$highlighted_cells, {
        if (!is.null(input$highlighted_cells)) {
            cell_id <- input$highlighted_cells$id
            cell_label <- input$highlighted_cells$label
            if (!is.null(cell_id)) {
                highlighted_cells(c(
                    highlighted_cells(),
                    cell_label
                ))
                print_grid(input, output, highlighted_cells)
            }
        } else {
            if (cell_label %in% unlist(highlighted_cells())) {
                current_values <- highlighted_cells()
                updated_values <- current_values[current_values
                != cell_label]
                highlighted_cells(updated_values)
                print_grid(input, output, highlighted_cells)
            }
        }
    })
}
toggle_grid <- function(input, output) {
    shiny::observeEvent(input$toggle_button, {
        shinyjs::toggle("grid-panel")
    })
}
register_click <- function(input, output, clicked_points, gg1) {
    shiny::observeEvent(event_data("plotly_click", source = "plotdata"), {
        d <- event_data("plotly_click", source = "plotdata")
        pd <- gg1()
        legend_text <- pd[["x"]][["data"]][[d$curveNumber + 1]][["name"]]


        if (
            (legend_text %in% clicked_points$legend_text)) {
            clicked_points$legend_text <- clicked_points$legend_text[
                clicked_points$legend_text != legend_text
            ]
        } else {
            clicked_points$legend_text <- c(
                clicked_points$legend_text,
                legend_text
            )
        }
    })
}
copy_click <- function(input, output, highlighted_cells, clicked_points) {
    shiny::observeEvent(input$myButton, {
        clicked_points <- gsub(
            ", ", "','",
            toString(unique(
                c(
                    clicked_points$legend_text,
                    highlighted_cells()
                )
            ))
        )
        tobecopied <- paste("'", clicked_points, "'",
            sep = ""
        )
        jscode <- sprintf(
            "var message = %s;
                         var tempInput = $('<input>');
                         $('body').append(tempInput);
                         tempInput.val(message).select();
                         document.execCommand('copy');
                         tempInput.remove();
                         Shinyjs.showAlert('Copied!', type = 'success');",
            jsonlite::toJSON(tobecopied)
        )
        shinyjs::runjs(jscode)
        if (!input$dialogueToggle) {
            showModal(modalDialog(
                title = "Successfully Copied",
                "Review using screen() and remove data with caution."
            ))
        }
    })
}
copy_click_full <- function(input, output, highlighted_cells, clicked_points) {
    shiny::observeEvent(input$removeCall, {
        clicked_points <- gsub(
            ", ", "','",
            toString(unique(
                c(
                    clicked_points$legend_text,
                    highlighted_cells()
                )
            ))
        )
        tobecopied <- paste("remove_raw(",
            input$extractedname,
            ", removelist = c('", clicked_points, "'))",
            sep = ""
        )
        jscode <- sprintf(
            "var message = %s;
                         var tempInput = $('<input>');
                         $('body').append(tempInput);
                         tempInput.val(message).select();
                         document.execCommand('copy');
                         tempInput.remove();
                         Shinyjs.showAlert('Copied!', type = 'success');",
            jsonlite::toJSON(tobecopied)
        )
        shinyjs::runjs(jscode)
        if (!input$dialogueToggle) {
            showModal(modalDialog(
                title = "Successfully Copied",
                "Paste function call to script and run directly to
                            remove selected wells."
            ))
        }
    })
}
saved_weeded <- function(input, output, dataset) {
    shiny::observeEvent(input$savetoR, {
        assign("new_raw_data", dataset(), envir = .GlobalEnv)
        if (!input$dialogueToggle) {
            showModal(modalDialog(
                title = "Data Saved Successfully",
                "Data saved to global environment of r console.
                Check for variable 'tsar_data'"
            ))
        }
    })
}
stop_window <- function(input, output, dataset) {
    shiny::observeEvent(input$stopButton, {
        stopApp()
    })
}
remove_selected <- function(input, output, dataset, checkrange, checklist, gg1,
                            highlighted_cells, clicked_points) {
    shiny::observeEvent(input$removeButton, {
        new_data <- remove_raw(dataset(),
            removelist = c(
                clicked_points$legend_text,
                highlighted_cells()
            )
        )
        dataset(new_data)
        if (!input$dialogueToggle) {
            showModal(modalDialog(
                title = "Successfully Removed",
                "Review new data within window."
            ))
        }
        print_plot(input, output, dataset, checkrange, checklist, gg1)
    })
}
refresh <- function(input, output, dataset, checkrange, checklist, gg1) {
    shiny::observeEvent(input$refreshButton, {
        print_plot(input, output, dataset, checkrange, checklist, gg1)
        if (!input$dialogueToggle) {
            showModal(modalDialog(
                title = "Successfully Refreshed",
                "All edits to dataframe are temporary. Close window using the
                close buttom will store new dataset under new_raw_data
                in the global environment. User may also copy wells and
                run remove_raw( ) in console to store change permanently."
            ))
        }
    })
}
view_selected <- function(input, output, dataset, checkrange, checklist,
                          highlighted_cells, clicked_points, gg1) {
    shiny::observeEvent(input$viewRemovedButton, {
        output$distPlot <- plotly::renderPlotly({
            gg1_p <- TSAR::screen(dataset(),
                checklist =
                    unique(c(
                        clicked_points$legend_text,
                        highlighted_cells()
                    ))
            )
            plotdata <- plotly::ggplotly(gg1_p, source = "plotdata")
            gg1(plotdata)
            plotly::event_register(gg1(), "plotly_click")
        })
        if (!input$dialogueToggle) {
            if ((is.null(clicked_points$legend_text)) &&
                (is.null(highlighted_cells()))) {
                showModal(modalDialog(
                    "Error: No valid Well variable was found.
                        Make sure it is named 'Well.Position' or 'Well'"
                ))
            } else {
                showModal(modalDialog(
                    title = "Viewing Selected Curves Only",
                    "Click remove if selections are correct. Interact with
                    graph or grid to select more or unselect curve. Else use
                    `Refrsh Screen` to return to home graph."
                ))
            }
        }
        print_grid(input, output, highlighted_cells)
        print_click(input, output, clicked_points)
    })
}

#' Merge and format norm_data into tsar_data
#'
#' This function merges data of experiment replicates across different dates.
#'     It merges and produces information variables used to group wells of same
#'     set up.
#'
#' @importFrom dplyr rename mutate bind_rows
#' @importFrom magrittr %>%
#'
#' @param data list, a character vector specifying the file paths of the data
#'     files or data frame objects of analysis data set. For example, given data
#'     frames named "data1" and "data2", specify parameter as
#'     \code{data = list(data1, data2)}.
#' @param name list, character vector specifying the experiment names.
#' @param date list, character vector specifying the dates. Does not require
#'     any date format restrictions.
#'
#' @details This function merges and normalizes test data from multiple files.
#'     The lengths of the \code{data}, \code{name}, and \code{date} vectors
#'     must match, otherwise an error is thrown.
#'
#' @examples
#' data("qPCR_data1")
#' result <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
#' data("well_information")
#' norm_data <- join_well_info(
#'     file_path = NULL, file = well_information,
#'     read_tsar(result, output_content = 2), type = "by_template"
#' )
#' norm_data <- na.omit(norm_data)
#' data("qPCR_data2")
#' result2 <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
#' norm_data2 <- join_well_info(
#'     file_path = NULL, file = well_information,
#'     read_tsar(result2, output_content = 2), type = "by_template"
#' )
#' norm_data2 <- na.omit(norm_data2)
#' tsar_data <- merge_norm(
#'     data = list(norm_data, norm_data2),
#'     name = c("Thermal Shift_162.eds.csv", "Thermal Shift_168.eds.csv"),
#'     date = c("20230203", "20230209")
#' )
#'
#' @return data frame in the format of tsar_data
#'
#' @family TSAR Formatting
#'
#' @export
#'
merge_norm <- function(data,
                       name,
                       date) {
    #check for matching lengths of parameters
    if (length(data) != length(name) || length(date) != length(name)) {
        stop("Data, name, and date counts do not match.")
    } else if (length(data) == 0) {
        stop("No data input, please check parameter input.")
    }

    tsar_data <- c()
    dataset <- c()

    #pivot data input
    dataset <- lapply(data, function(x) {
        if (!is.data.frame(x)) {
            return(as.data.frame(read.csv(file = toString(x), header = TRUE)))
        } else {
            return(as.data.frame(x))
        }
    })

    #create well ID and combine datasets
    tsar_data <- lapply(seq_along(dataset), function(i) {
        cur <- dataset[[i]]
        cur <- cur %>%
            mutate(ExperimentFileName = name[i])
        cur <- cur %>%
            mutate(well_ID = paste(Well.Position, Protein, Ligand,
                            date[i], sep = "_"))
        return(cur)
    }) %>% bind_rows()

    #create condition ID
    tsar_data <- data.frame(tsar_data)
    tsar_data <- tsar_data %>%
        mutate(condition_ID = paste(tsar_data$Protein,
            tsar_data$Ligand,
            sep = "_"
        )) %>%
        dplyr::rename(Tm = tm, Well = Well.Position)

    return(tsar_data)
}

analyzepage <- function(raw_data, data_name) {
    fluidPage(
        tags$head(tags$style(HTML("
            .h3-style {
              font-family: sans-serif;
              font-size: 26px;
              font-weight: 500;
            }
          "))),
        tags$style(
            HTML(
                "
              .sticky-panel {
                position: sticky;
                top: 0;
                z-index: 100;
                background-color: white;
              }
              "
            )
        ),
        shiny::checkboxInput("dialogueToggle",
            "Hide All Hints and Messages",
            value = FALSE
        ),
        fluidRow(
            class = "sticky-panel",
            column(
                width = 6,
                br(),
                div(
                    class = "h3-style",
                    shiny::textOutput("table_title")
                ),
                div(
                    style = "font-size: 10px;",
                    shiny::dataTableOutput("table")
                )
            ),
            column(
                width = 6,
                h3("Fit Model"),
                plotly::plotlyOutput("Plot")
            )
        ),
        h3("Test fit"),
        fluidRow(
            column(
                width = 2,
                shiny::selectInput("well",
                    label = "Select Well:",
                    choices = c(unique(raw_data$Well.Position))
                )
            ),
            column(
                width = 3,
                shiny::selectInput(
                    "model_used",
                    label = "Model used",
                    choices = c("GAM", "beta"),
                    selected = "GAM"
                )
            ),
            column(
                width = 3,
                shiny::numericInput("num",
                    label = "Enter y column number:",
                    value = 5
                )
            ),
            column(
                br(),
                width = 2,
                shiny::actionButton("model_fit", "View Model Fit")
            )
        ),
        h3("Derivative Analysis"),
        fluidRow(
            column(
                width = 3,
                shiny::selectInput("keep",
                    label = "Keep all data",
                    choices = c(TRUE, FALSE), selected = TRUE
                )
            ),
            column(
                width = 3,
                shiny::selectInput("fit",
                    label = "Return fit summary: ",
                    choices = c(TRUE, FALSE), selected = FALSE
                )
            ),
            column(
                width = 3,
                shiny::selectInput("smooth",
                    label = "Raw data smoothed?",
                    choices = c(TRUE, FALSE), selected = TRUE
                )
            )
        ),
        fluidRow(
            column(
                width = 12,
                shiny::checkboxGroupInput("checkGroup",
                    label = "Keep variables: (keep x and y)",
                    choices = c(names(raw_data)),
                    inline = TRUE,
                    selected = c(
                        "Well.Position",
                        "Temperature",
                        "Fluorescence",
                        "Normalized"
                    )
                )
            ),
            column(
                width = 3,
                shiny::numericInput("num_a",
                    label = "Enter y column number:", value = 5
                )
            ),
            column(
                br(),
                width = 4,
                shiny::actionButton("analyze", "Analyze all Wells")
            )
        ),
        h3("Input Well Conditions"),
        fluidRow(
            column(
                width = 3,
                shiny::fileInput("excelFile", "Upload Well Information.xlsx")
            ),
            column(
                width = 2, br(),
                shiny::actionButton("previewBtn", "Preview")
            ),
            column(
                width = 1, br(),
                shiny::actionButton("hideBtn", "Hide")
            ),
            column(
                width = 2, br(),
                shiny::actionButton("blank", "Manual Input")
            ),
            column(
                width = 2, br(),
                shiny::actionButton("saveBtn", "Save Changes")
            ),
            column(
                width = 1, br(),
                shiny::actionButton("join", "Set Conditions")
            ),
        ),
        div(
            style = "font-size: 11px;",
            rhandsontable::rHandsontableOutput("excelTable")
        ),
        h3("Save Data Locally"),
        fluidRow(
            column(
                width = 5,
                shiny::radioButtons("radio",
                    label = "Choose dataset",
                    choices = list(
                        "Only Tm" = 0,
                        "Fluorescence and Model" = 1,
                        "Both" = 2
                    ),
                    inline = TRUE,
                    selected = 2
                )
            ),
            column(
                width = 2,
                shiny::radioButtons("file_type",
                    label = "file type: ",
                    choices = list("txt", "csv"),
                    inline = TRUE,
                    selected = "csv"
                )
            ),
            column(
                width = 3,
                shiny::selectInput("withCondition",
                    label = "With Conditions: ",
                    choices = c(TRUE, FALSE), selected = FALSE
                )
            )
        ),
        fluidRow(
            column(
                width = 5,
                shiny::textInput("name",
                    label = "Enter Name: ",
                    value = paste(data_name,
                        Sys.Date(),
                        sep = "_"
                    )
                )
            ),
            column(
                br(),
                width = 2,
                shiny::actionButton("preview", "Preview Output")
            ),
            column(
                br(),
                width = 2,
                shiny::actionButton("write", "Save to Directory")
            ),
            column(
                br(),
                width = 2,
                shiny::actionButton("savelocal", "Save to R")
            )
        ),
        br(),
        shiny::actionButton("stopButton", "Close Window")
    )
}

#' Read TSA Analysis Data
#'
#' Open TSA Analysis files. This function is used to load data output from
#'     the thermal shift software
#'     analysis tab. Can be either .txt or .csv file with a path / file name
#'     including the string "AnalysisResults" due to its automatic naming from
#'     the software. The values assigned to wells within the TSA software are
#'     automatically extracted from the loaded file; values must be assigned
#'     within the TSA software for the automated workflow (See IDs Section
#'     Below).
#'     \strong{Note:} Wells that do not have an Analysis Group assigned are
#'     removed.
#'     The TSA software automatically assigns all wells to Analysis Group 1 by
#'     default, and can be changed but not removed by the software.
#'
#' @importFrom stringr str_detect str_replace_all
#' @importFrom tidyr unite
#'
#' @param path a character string; the path or the name of the file which the
#'    'AnalysisResults' data are to be read from. Either a .txt or .csv file.
#'     The path must contain the term \emph{AnalysisResults} as the TSA software
#'     automatically assigns this when exporting data.
#' @param type either c("boltzmann", "derivative"); \code{type = "derivative")}
#'     by default. Determines what model of
#'     Tm estimation to load from the TSA software. Loads Tms as 'Tm B'  when
#'     \code{type = "boltzmann")}; loads Tms as 'Tm D' when
#'     \code{type = "derivative")}.
#' @param conditions A character vector of condition types assigned within the
#'     TSA software to load.
#'     \code{conditions = c("Protein", "Ligand")} by default.
#'     These conditions are used to generate the IDs discussed.
#' @param manual_file NA by default. User can specify .eds for merging if needed
#'     for Well IDs if needed with a character string.
#' @param manual_conditions,manual_wells NA by default,
#'     enabling automated analysis.
#'     A character vector of Condition IDs and Well IDs to manually assign
#'     each row of the read data.
#' @param skip_flags logical value; \code{type = FALSE} by default.
#'     When \code{type = TRUE}, wells that have flags reported by TSA software
#'     are removed.
#' @return A data frame of TSA analysis data.
#' @section IDs:
#'     The TSAR package relies on matching conditions and file names for each
#'     well and for each set of conditions between multiple files output by
#'     the TSA software. Conditions are
#'     assigned to individual wells within the TSA software; these assigned
#'     values are detected by
#'     \code{\link{read_analysis}} and
#'     \code{\link{read_raw_data}} then are converted into IDs.
#'     Ensure your labeling of values within the TSA software is consistent
#'     so that similar values can be merged - typos or varying terms will be
#'     treated as distinct values within TSAR unless the values
#'     are manually specified by the user.
#'     Automatically generated well IDs within a TSA file can be found using the
#'     \code{\link{well_IDs}} function; condition IDs can be found using the
#'     \code{\link{condition_IDs}} function. \cr\cr
#'
#'
#'     \strong{Condition IDs} are generated from columns in TSA output
#'     specified by the 'conditions' argument.
#'     Protein and Ligand values, the default conditions within the TSA
#'     software, are the values used to create these IDs. You can manually
#'     specify the
#'     condition categories from the TSA software, including user-made
#'     conditions. Condition IDs are used to match equivalent observations
#'     between technical and biological replicates. Wells with identical
#'     condition IDs, specified by the 'conditions' argument, will be
#'     aggregated in down-stream analysis; user-specified conditions must
#'     remain consistent in use and order to create compatible IDs between TSA
#'     files from the same experiment and between replicates. \cr\cr
#'
#'     \strong{Well IDs} are similar to Condition IDs, as they are
#'     generated from columns in TSA output that are
#'     specified by the 'conditions' argument. Well IDs are used to match
#'     the analysis and raw data files for the same experiment, as both files
#'     contain unique, useful information for each well. In addition to the
#'     condition ID, the well ID includes the .eds file name saved from the
#'     PCR machine to match equivalent wells between files of the same
#'     experiment. Each well on all plates should have a unique well ID.
#'     If you wish to change or specify the file name used for the well ID,
#'     a new name can be manually assigned with the "manual_file" argument.
#'     \cr\cr
#'
#'     The user may manually assign condition IDs using the
#'     'manual_conditions' argument rather than using the automatically
#'     generated IDs. The same is true for well IDs, which can be manually
#'     assigned with 'manual_wells'. This is not suggested, as there may be
#'     issues with matching if well/conditions are not properly matching. This
#'     gives the potential for errors in downstream applications as well.
#'
#' @family Read TSA Data
#' @seealso \code{\link{read_raw_data}} for loading accompanying data.
#'     \code{\link{merge_TSA}} for joining Analysis Results and Raw Data files
#'     from the TSA software.
#' @examples
#' path <- "~/Desktop/analysis_data"
#' # note: example does not contain example data to run
#' # read_analysis(path)
#' @export
#'
#'
read_analysis <- function(
    path,
    type = "derivative",
    conditions = c("Protein", "Ligand"),
    manual_conditions = NA,
    manual_wells = NA,
    skip_flags = FALSE,
    manual_file = NA) {

    type <- match.arg(type, choices = c("fluorescence",
                                        "derivative", "boltzmann"))

    # Note: Wells that do not have an Analysis Group assigned are removed
    #--- Loading and formatting


    if (!stringr::str_detect(path, "AnalysisResults")) {
        warning('Check the input file,
                    The path name does not include "AnalysisResults"')
    }
    if (stringr::str_detect(path, pattern = "(.*\\.txt$)|(.*\\.csv$)")) {
        if (stringr::str_detect(path, pattern = "(.*\\.txt$)")) {
            analysis <- read.delim(path,
                skip = 2,
                na.strings = c("", "NA", " ")
            )
        }
        if (stringr::str_detect(path, pattern = "(.*\\.csv$)")) {
            analysis <- read.csv(path,
                skip = 2,
                na.strings = c("", "NA", " ")
            )
        }
        names(analysis) <- stringr::str_replace_all(names(analysis), "\\.", " ")
        names(analysis)[names(analysis) == "Flag Indicator"] <- "Flags"
        analysis <- analysis[seq_len(96), ] # Trim to 96 wells
        analysis$`Tm D` <- as.numeric(analysis$`Tm D`)
        analysis$`Tm B` <- as.numeric(analysis$`Tm B`)
    } else {
        stop("File type not .csv or .txt")
    }
    if (type == "boltzmann") {
        col_names <-
            c(
                "Well", conditions, "Tm B", "dTm B",
                "Analysis Group", "Flags", "Experiment File Name"
            )
    }
    if (type == "derivative") {
        col_names <-
            c(
                "Well", conditions, "Tm D", "dTm D",
                "Analysis Group", "Flags", "Experiment File Name"
            )
    }
    # Keep only rows with an analysis group and required+condition cols
    analysis <- analysis[(!is.na(analysis$`Analysis Group`)), col_names]
    names(analysis)[names(analysis) %in% c("Tm D", "Tm B")] <- "Tm"
    #--- Generating ID codes for downstream functions
    #--- Making condition_ID-same for all equivalent wells (for well matching)
    if (is.na(manual_conditions)) { # Default, auto generate condition IDs
        analysis$condition_ID <-
            tidyr::unite(analysis, "condition_ID", conditions)$condition_ID
    } else {
        analysis$condition_ID <- manual_conditions # manual well assignments
    }
    #--- Making well_codes-each well is independent (for raw&analysis matching)
    if (!is.na(manual_file)) {
        analysis$`Experiment File Name` <- manual_file
    }
    if (is.na(manual_wells)) { # Default, auto generate condition IDs
        analysis$well_ID <-
            tidyr::unite(
                analysis, "well_ID",
                c("Well", "Experiment File Name")
            )$well_ID
    } else {
        analysis$well_ID <- manual_wells # manual well assignments
    }
    if (skip_flags) { # remove flagged rows if user specifies
        analysis <- analysis[analysis$Flags == 0, ]
    }

    return(analysis)
}

#' qPCR_data1 Dataset
#'
#' Dataset Description:
#'     This dataset contains qPCR data for the CA121 protein and common
#'     vitamins. It provides fluorescence measurements obtained
#'     using QuantStudio3.
#'     Dataset is experimentally obtained by author of this package.
#'
#' @name qPCR_data1
#'
#' @format A data frame with the following columns:
#'     \describe{
#'       \item{Well}{Well Count, not required for user}
#'       \item{Well.Position}{Well Label, i.e. A01; required input}
#'       \item{Reading}{reading count in time series, not required for user}
#'       \item{Temperature}{temperature reading, required input}
#'       \item{Fluorescence}{fluorescence reading, required input}
#'     }
#'
#' @usage data(qPCR_data1)
#' @return qPCR_data1 data frame
#' @source experimentally obtained
#' @keywords dataset
data(qPCR_data1, envir = environment())

## ----echo=FALSE, fig.width=4, out.width="400px"-------------------------------
knitr::include_graphics("images/TSAR_logo.png")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(warning = FALSE, message = FALSE, comment = "#>")

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(TSAR)
library(dplyr)
library(ggplot2)

## -----------------------------------------------------------------------------
data("qPCR_data1")
raw_data <- qPCR_data1

## -----------------------------------------------------------------------------
test <- raw_data %>% filter(Well.Position == "A01")

## -----------------------------------------------------------------------------
# normalize fluorescence reading into scale between 0 and 1
test <- normalize(test, fluo = 5, selected = c(
    "Well.Position", "Temperature",
    "Fluorescence", "Normalized"
))
head(test)
gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
gammaresult <- model_fit(test, model = gammodel)

## -----------------------------------------------------------------------------
Tm_est(gammaresult)
view <- view_model(gammaresult)
view[[1]] + theme(aspect.ratio = 0.7, legend.position = "bottom")
view[[2]] + theme(aspect.ratio = 0.7, legend.position = "bottom")

## -----------------------------------------------------------------------------
betamodel <- model_beta(test, x ="Temperature", y = "Normalized")
view <- view_model(betamodel)
view[[1]] + theme(aspect.ratio = 0.7, legend.position = "bottom")
view[[2]] + theme(aspect.ratio = 0.7, legend.position = "bottom")

## -----------------------------------------------------------------------------
myApp <- weed_raw(raw_data, checkrange = c("A", "C", "1", "12"))

## ----eval = FALSE-------------------------------------------------------------
# shiny::runApp(myApp)

## -----------------------------------------------------------------------------
raw_data <- remove_raw(raw_data, removerange = c("B", "H", "1", "12"))
screen(raw_data) + theme(
    aspect.ratio = 0.7,
    legend.position = "bottom",
    legend.text = element_text(size = 6),
    legend.key.size = unit(0.4, "cm"),
    legend.title = element_text(size = 8)
) +
    guides(color = guide_legend(nrow = 2, byrow = TRUE))

## ----echo = FALSE-------------------------------------------------------------
x <- gam_analysis(raw_data,
    smoothed = TRUE, fluo_col = 5,
    selections = c(
        "Well.Position", "Temperature",
        "Fluorescence", "Normalized"
    )
)
x <- na.omit(x)

## ----echo = FALSE-------------------------------------------------------------
# look at only Tm result by well
output <- read_tsar(x, output_content = 0)
head(output)
tail(output)

## ----echo = FALSE-------------------------------------------------------------
# join protein and ligand information
data("well_information")
norm_data <- join_well_info(
    file_path = NULL,
    file = well_information,
    read_tsar(x, output_content = 2), type = "by_template"
)
norm_data <- na.omit(norm_data)
head(norm_data)
tail(norm_data)

## -----------------------------------------------------------------------------
data("qPCR_data2")
raw_data_rep <- qPCR_data2

raw_data_rep <- remove_raw(raw_data_rep, removerange = c("B", "H", "1", "12"))

## ----eval = FALSE-------------------------------------------------------------
# myApp <- weed_raw(raw_data_rep)
# shiny::runApp(myApp)

## -----------------------------------------------------------------------------
raw_data_rep <- remove_raw(raw_data_rep, removelist = "A12")
screen(raw_data_rep) + theme(
    aspect.ratio = 0.7,
    legend.position = "bottom",
    legend.text = element_text(size = 6),
    legend.key.size = unit(0.4, "cm"),
    legend.title = element_text(size = 8)
) +
    guides(color = guide_legend(nrow = 2, byrow = TRUE))

analysis_rep <- gam_analysis(raw_data_rep, smoothed = TRUE)
output_rep <- read_tsar(analysis_rep, output_content = 2)
norm_data_rep <- join_well_info(
    file_path = NULL,
    file = well_information,
    output_rep, type = "by_template"
)
norm_data_rep <- na.omit(norm_data_rep)

## -----------------------------------------------------------------------------
norm_data <- na.omit(norm_data)
norm_data_rep <- na.omit(norm_data_rep)
tsar_data <- merge_norm(
    data = list(norm_data, norm_data_rep),
    name = c(
        "Vitamin_RawData_Thermal Shift_02_162.eds.csv",
        "Vitamin_RawData_Thermal Shift_02_168.eds.csv"
    ),
    date = c("20230203", "20230209")
)

## -----------------------------------------------------------------------------
condition_IDs(tsar_data)
well_IDs(tsar_data)
conclusion <- tsar_data %>%
    filter(condition_ID != "NA_NA") %>%
    filter(condition_ID != "CA FL_Riboflavin")
TSA_boxplot(conclusion,
    color_by = "Protein", label_by = "Ligand",
    separate_legend = FALSE
)

## -----------------------------------------------------------------------------
control_ID <- "CA FL_DMSO"

TSA_compare_plot(conclusion,
    y = "Fluorescence",
    control_condition = control_ID,
    smoother = "beta",
    beta_shape = 4, 
    beta_knots_frac = 0.008, 
    show_Tm = TRUE
)

## -----------------------------------------------------------------------------
error <- conclusion %>% filter(condition_ID == "CA FL_PyxINE HCl")
TSA_wells_plot(error, y='Fluorescence', separate_legend = TRUE, smoother = "beta", beta_shape = 4, beta_knots_frac = 0.008, show_Tm = TRUE)

## -----------------------------------------------------------------------------
citation("TSAR")
citation()
citation("dplyr")
citation("ggplot2")

## -----------------------------------------------------------------------------
sessionInfo()

preview_model <- function(input, output, raw_data) {
    shiny::observeEvent(input$model_fit, {
        test <- subset(raw_data, Well.Position == as.character(input$well))
        test <- normalize(test, fluo = input$num)
        # Choose modeling method based on UI selection
        if (!is.null(input$model_used) && tolower(input$model_used) == "beta") {
            # Fit beta-spline model centered at Tm and augment for view_model()
            test <- model_beta(
                norm_data = test,
                x = "Temperature",
                y = "Normalized"
            )
            # model_beta already returns augmented df with fitted and norm_deriv
            # Ensure required columns exist for plotting
            if (!"fitted" %in% names(test)) {
                # fallback: if attribute stores model, predict
                fit_attr <- attr(test, "beta_model")
                if (!is.null(fit_attr)) {
                    test$fitted <- stats::predict(fit_attr, newdata = test)
                }
            }
        } else {
            model <- model_gam(test, x = test$Temperature, y = test$Normalized)
            test <- model_fit(test, model = model)
        }
        gg <- view_model(test)
        build_view(input, output, gg)
    })
}
perform_analysis <- function(input, output, raw_data, analysis,
                             output_data, analyzed_done) {
    shiny::observeEvent(input$analyze, {
        # convert selectInput values ("TRUE"/"FALSE") to logical
        .to_bool <- function(x) {
            if (is.logical(x)) return(x)
            if (is.null(x)) return(FALSE)
            tolower(as.character(x)) == "true"
        }
        keep_flag    <- .to_bool(input$keep)
        fit_flag     <- .to_bool(input$fit)
        smoothed_flag<- .to_bool(input$smooth)

        analysis_p <- gam_analysis(raw_data,
            fluo_col = input$num_a,
            keep = keep_flag,
            fit = fit_flag,
            smoothed = smoothed_flag,
            selections = c(
                input$checkGroup,
                "Normalized"
            )
        )
        d <- read_tsar(analysis_p, output_content = 2)
        analysis(analysis_p)
        render_data(input, output, d)
        render_data_title(input, output, "Analyzed Data: norm_data")
        analyzed_done(TRUE)
        if (!input$dialogueToggle) {
            shiny::showModal(shiny::modalDialog(
                title = "Analysis is complete",
                "Proceede to inputting conditions and saving data
                        before closing window."
            ))
        }
        output_data_p <- read_tsar(analysis(), output_content = input$radio)
        output_data(output_data_p)
    })
}
join_condition <- function(input, output, analysis, output_data, imported,
                           analyzed_done, well_info, inFile) {
    shiny::observeEvent(input$withCondition, {
    if (imported() == TRUE && tolower(as.character(input$withCondition)) == "true") {
            output_data_p <- join_well_info(
                file_path = inFile(),
                file = well_info(),
                analysis_file =
                    read_tsar(analysis(),
                        output_content = input$radio
                    ),
                type = "by_template"
            )
            output_data(output_data_p)
        } else {
            if (analyzed_done() == FALSE) {
                output_data(NULL)
            } else {
                output_data_p <- read_tsar(analysis(),
                    output_content = input$radio
                )
                output_data(output_data_p)
            }
        }
    })
}
write_file <- function(input, output, analyzed_done, output_data) {
    shiny::observeEvent(input$write, {
        if (analyzed_done() == FALSE) {
            if (!input$dialogueToggle) {
                shiny::showModal(shiny::modalDialog(
                    title = "Analysis is Incomplete!",
                    "Please analyze all data before saving."
                ))
            }
        } else {
            write_tsar(output_data(),
                name = input$name,
                file = input$file_type
            )
            if (!input$dialogueToggle) {
                shiny::showModal(shiny::modalDialog(
                    title = "Successfully Saved",
                    "Check your working directory for data file."
                ))
            }
        }
    })
}
preview_output <- function(input, output, analyzed_done, imported, output_data,
                           analysis, inFile, well_info) {
    shiny::observeEvent(input$preview, {
        if (analyzed_done() == FALSE) {
            if (!input$dialogueToggle) {
                shiny::showModal(shiny::modalDialog(
                    title = "Analysis is Incomplete!",
                    "Please analyze all data before saving."
                ))
            }
        } else {
            if (imported() == FALSE && tolower(as.character(input$withCondition)) == "true") {
                if (!input$dialogueToggle) {
                    shiny::showModal(shiny::modalDialog(
                        title = "Conditions are not specified!",
                        "Please input condition information first
                                or set 'With Conditions' as fasle."
                    ))
                }
            } else {
                if (!input$dialogueToggle) {
                    shiny::showModal(shiny::modalDialog(
                        title = "Hint :)",
                        "If you are saving data for tsar_graphing
                                functions, make sure to save all fluorescence
                                data (i.e. 'both') to output compare plots and
                                conditions plot."
                    ))
                }
                if (imported() == TRUE && tolower(as.character(input$withCondition)) == "true") {
                    output_data_p <- join_well_info(
                        file_path = inFile(),
                        file = well_info(),
                        analysis_file = read_tsar(analysis(),
                            output_content = input$radio
                        ),
                        type = "by_template"
                    )
                    output_data(output_data_p)
                } else {
                    if (analyzed_done() == FALSE) {
                        output_data(NULL)
                    } else {
                        output_data_p <- read_tsar(analysis(),
                            output_content = input$radio
                        )
                        output_data(output_data_p)
                    }
                }
                render_data(input, output, output_data())
                render_data_title(input, output, "Output Data: tsar_data")
            }
        }
    })
}
manual_well <- function(input, output) {
    shiny::observeEvent(input$blank, {
        build_well(input, output)
    })
}
set_condition <- function(input, output, imported, analyzed_done, inFile,
                          well_info, analysis, output_data) {
    shiny::observeEvent(input$join, {
        if ((imported() == FALSE) || (analyzed_done() == FALSE)) {
            if ((analyzed_done() == FALSE) && (!input$dialogueToggle)) {
                shiny::showModal(shiny::modalDialog(
                    title = "Analysis is Incomplete!",
                    "Please analyze all data before saving."
                ))
            } else {
                if (!input$dialogueToggle) {
                    shiny::showModal(shiny::modalDialog(
                        title = "Conditions are not specified!",
                        "Please input condition information first
                                or set 'With Conditions' as fasle."
                    ))
                }
            }
        } else {
            output_data_p <- join_well_info(
                file_path = inFile(),
                file = well_info(),
                analysis_file =
                    read_tsar(analysis(),
                        output_content = input$radio
                    ),
                type = "by_template"
            )
            output_data(output_data_p)
            if (!input$dialogueToggle) {
                shiny::showModal(shiny::modalDialog(
                    title = "Conditions Saved",
                    "Proceede to preview output and save data."
                ))
            }
            render_data(input, output, output_data())
            render_data_title(input, output, "Complete Data: tsar_data")
        }
    })
}

preview_condition <- function(input, output, well_info, data) {
    shiny::observeEvent(input$previewBtn, {
        output$excelTable <- rhandsontable::renderRHandsontable({
            if (!is.null(well_info())) {
                rhandsontable::rhandsontable(data.frame(head(well_info(), 9)))
            } else if (!is.null(data())) {
                rhandsontable::rhandsontable(data.frame(head(data(), 9)))
            }
        })
    })
}
hide <- function(input, output) {
    shiny::observeEvent(input$hideBtn, {
        build_well(input, output, TRUE)
    })
}
save_change <- function(input, output, imported, well_info) {
    shiny::observeEvent(input$saveBtn, {
        imported(TRUE)
        well_info(rhandsontable::hot_to_r(input$excelTable))
        if (!input$dialogueToggle) {
            shiny::showModal(shiny::modalDialog(
                title = "Success",
                "Changes saved successfully."
            ))
        }
    })
}
save_analysis <- function(input, output, output_data) {
    shiny::observeEvent(input$savelocal, {
        assign(input$name, output_data(), envir = .GlobalEnv)
        if (!input$dialogueToggle) {
            shiny::showModal(shiny::modalDialog(
                title = "Successfully Saved",
                "Check your R environment for data file."
            ))
        }
    })
}
stop_analyze <- function(input, output) {
    shiny::observeEvent(input$stopButton, {
    shiny::stopApp()
    })
}

#' Screen raw curves
#'
#' screens multiple wells of data and prepares to assist identification of
#'     corrupted wells and odd out behaviors
#'
#' @import ggplot2
#' @importFrom dplyr rename
#'
#' @param raw_data input raw_data
#' @param checkrange list type input identifying range of wells to select.
#'     For example, if viewing first 8 wells from row A to C is needed, one can
#'     specify the row letters and column numbers like this:
#'     \code{checkrange = c("A", "C", "1", "8")}
#' @param checklist use this parameter to view selected Wells with full
#'     Well names. For example, \code{checklist = c('A01', 'D11')}
#'
#' @examples
#' data("qPCR_data1")
#' screen(qPCR_data1, checkrange = c("A", "C", "1", "12"))
#'
#' @return returns a ggplot graph colors by well IDs
#'
#' @family data_preprocess
#'
#' @export
screen <- function(raw_data,
                   checkrange = NULL,
                   checklist = NULL) {
    if (is.null(checkrange) && (is.null(checklist))) {
        section <- raw_data
    } else {
        if (!(is.null(checklist))) {
            checkrange <- checklist
        } else {
            #translate checkrange into well position labels
            grid <- expand.grid(
                LETTERS[match(checkrange[1], LETTERS):
                match(checkrange[2], LETTERS)],
                sprintf("%02d", checkrange[3]:
                checkrange[4])
            )
            checkrange <- paste0(grid$Var1, grid$Var2)
        }
        if ("Well.Position" %in% names(raw_data)) {
            section <- subset(raw_data, Well.Position %in% checkrange)
        } else if ("Well" %in% names(raw_data)) {
            section <- subset(raw_data, Well %in% checkrange)
            section <- dplyr::rename(section, Well.Position = Well)
        } else {
            stop("Error: No valid Well variable was found.
                  Make sure it is named 'Well.Position' or 'Well'")
        }
    }
    #perform normalization
    unique_well_positions <- unique(section$Well.Position)

    normalize_by_well <- function(well_position, data) {
        by_well <- subset(data, Well.Position == well_position)
        by_well <- normalize(by_well, selected =
            c("Well.Position", "Temperature", "Fluorescence", "Normalized"))
        return(by_well)
    }
    screened_list <- lapply(unique_well_positions,
                        normalize_by_well, data = section)
    screened <- do.call(rbind, screened_list)

    if (nrow(screened) == 0) {
        stop("Error: Select of Wells do not exist or are already removed.")
        return()
    }
    ggplot(data = screened, aes(
        x = Temperature,
        y = Fluorescence,
        color = Well.Position
    )) +
        geom_line(size = 0.2) +
        theme_bw() +
        theme(panel.grid.major = element_blank())
}

#' Remove selected raw curves
#'
#' Removes selected curves with specified wells and range.
#'
#' @param raw_data dataframe; to be processed data
#' @param removerange list type input identifying range of wells to select.
#'     For example, if removing all 12 wells from row D to H is needed, one can
#'     specify the row letters and column numbers like this:
#'     \code{removerange = c("D", "H", "1", "12")}
#' @param removelist use this parameter to remove selected Wells with full
#'     Well names. For example, \code{removelist = c('A01', 'D11')}
#'
#' @examples
#' data("qPCR_data1")
#' remove_raw(qPCR_data1, removelist = c("A01", "D11"))
#'
#' @return dataframe; data frame with specified well removed
#' @family data_preprocess
#' @export
#'
remove_raw <- function(raw_data,
                       removerange = NULL,
                       removelist = NULL) {
    if (is.null(removerange) && is.null(removelist)) {
        return(raw_data)
    } else if (!is.null(removerange)) {
        #translate variable removerange into labels
        grid <- expand.grid(
            LETTERS[match(removerange[1], LETTERS):match(
                removerange[2], LETTERS
            )],
            sprintf("%02d", removerange[3]:removerange[4])
        )
        #remove
        removerange <- paste0(grid$Var1, grid$Var2)
    }
    removerange <- c(removerange, removelist)

    if ("Well.Position" %in% names(raw_data)) {
        return(subset(raw_data, !Well.Position %in% removerange))
    } else if ("Well" %in% names(raw_data)) {
        return(subset(raw_data, !Well %in% removerange))
    } else {
        stop("Error: No valid Well variable was found.
         Make sure it is named 'Well.Position' or 'Well'")
    }
}



#' View Model
#'
#' Function reviews data by well and output a graph of the fit and a graph of
#'     derivative. Function called within analyze_norm function.
#'
#' @import ggplot2
#' @param raw_data dataset input, not processing needed
#' @examples
#' data("qPCR_data1")
#' test <- subset(qPCR_data1, Well.Position == "A01")
#' test <- normalize(test)
#' gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
#' test <- model_fit(test, model = gammodel)
#' view_model(test)
#'
#' @return list of two ggplot graphs
#' @family data_preprocess
#' @export
#'
view_model <- function(raw_data) {
    #fit model
    fit <- ggplot(data = raw_data, aes(x = Temperature, y = Normalized)) +
        geom_point(
            shape = 1, alpha = 0.5,
            aes(color = "Normalized Fluorescence")
        ) +
        geom_line(aes(y = fitted, color = "Fitted Model"), size = 0.5) +
        geom_vline(xintercept = Tm_est(raw_data), color = "red") +
        labs(color = "Curves", y = "RFU") +
        theme_bw() +
        theme(panel.grid.major = element_blank())

    raw_data <- na.omit(raw_data)
    #create derivative graph aligned with model
    deriv <- ggplot(data = raw_data, aes(
        x = Temperature, y = norm_deriv,
        color = Well.Position
    )) +
        geom_point(
            shape = 1, alpha = 0.5, size = 0.3,
            aes(color = "derivative Norm-Fluo")
        ) +
        geom_vline(xintercept = Tm_est(raw_data), color = "red") +
        scale_color_manual(values = "blue") +
        labs(color = "Curves", y = "dRFU") +
        theme_bw() +
        theme(panel.grid.major = element_blank())

    return(list(fit, deriv))
}


#' View Derivative Curves
#'
#' Function reviews data by well and output a graph of the all derivatives
#'     wanted. Function called within graph_tsar function but also runnable
#'     outside.
#'
#' @import ggplot2
#' @importFrom plotly ggplotly layout animation_opts
#' @param tsar_data dataset input, analyzed must have norm_deriv as a variable;
#'     dataset qualifying norm_data or tsar_data both fulfills this parameter,
#'     although tsar_data is more recommended given more data options.
#' @param frame_by builds plotly by specified frame variable. To graph by a
#'     concentration gradient, well position, or other specified variable,
#'     simple specify \code{frame_by = "condition_ID"}. To view all derivative
#'     curves without frames, set to \code{frame_by = "NA"}, else it is
#'     defaulted to frame by well labels.
#' @examples
#' data("example_tsar_data")
#' view_deriv(example_tsar_data, frame_by = "condition_ID")
#'
#' @return plotly object of derivative curves
#' @family TSA Plots
#' @export
#'
view_deriv <- function(tsar_data, frame_by = "Well") {

    #when no framing variable is specified, graph all data in one panel
    if (frame_by == "NA") {
        graphed <- ggplot(
            data = tsar_data,
            aes(
                x = Temperature, y = norm_deriv,
                color = well_ID
            )
        )
    #if framing variable is specified, graph by frames
    } else if (!missing(frame_by)) {
        graphed <- ggplot(
            data = tsar_data,
            aes(
                x = Temperature, y = norm_deriv,
                color = well_ID, frame = !!sym(frame_by),
                label = Tm
            )
        )
    } else {
        stop("No frames specified; data does not contain Well or Well.Postion.")
    }

    graphed <- graphed +
        geom_line() +
        labs(y = "dRFU") +
        theme_bw() +
        theme(panel.grid.major = element_blank())
    graphed <- ggplotly(graphed)
    graphed <- animation_opts(graphed, mode = "next")
    #graph features sevring rShiny module details
    if (frame_by == "NA") {
        plotly::layout(graphed,
            yaxis = list(tickfont = list(size = 10), showgrid = TRUE),
            xaxis = list(tickfont = list(size = 10), showgrid = TRUE),
            showlegend = TRUE
        )
    } else {
        plotly::layout(graphed,
            yaxis = list(tickfont = list(size = 10), showgrid = TRUE),
            xaxis = list(tickfont = list(size = 10), showgrid = TRUE),
            showlegend = FALSE
        )
    }
}

#' Read analysis result
#'
#' reads previous pipeline output lists from \code{\link{gam_analysis}}()
#'     and organizes them into separate data frames.
#'
#' @param gam_result list; input uses resulting output of
#'     \code{\link{gam_analysis}}() function
#' @param output_content integer;
#'     \code{output_content = 0} returns only the tm value by wells
#'     \code{output_content = 1} returns data table with fitted values
#'     \code{output_content = 2} returns the combination of 0 and 1
#' @return output files with select dataset
#'
#' @family read_write_analysis
#'
#' @examples
#' data("qPCR_data1")
#' result <- gam_analysis(qPCR_data1,
#'     smoothed = TRUE, fluo_col = 5,
#'     selections = c(
#'         "Well.Position", "Temperature", "Fluorescence", "Normalized"
#'     )
#' )
#' read_tsar(result, output_content = 0)
#' output_data <- read_tsar(result, output_content = 2)
#'
#' @export
read_tsar <- function(gam_result, output_content) {
    # returns only the tm value by wells
    wellpos <- gam_result[[2]]$Well.Position
    wellpos <- paste(substr(wellpos, 1, 1),
        sprintf("%02s", substr(wellpos, 2, nchar(wellpos))),
        sep = ""
    )
    if (output_content == 0) {
        tmval <- gam_result[[1]]
        tmval <- data.frame(
            Well.Position =
                unique(wellpos[grepl("[A-Z]+[0-9]+[0-9]$", wellpos)]),
            TM = tmval
        )
        return(tmval)
    } else if (output_content == 1) {
        # returns the data table with the fitted values
        dat <- gam_result[[2]]
        dat <- data.frame(dat)
        return(dat)
    } else if (output_content == 2) {
        # returns the all data, combination of 0 and 1
        tmval <- gam_result[[1]]
        dat <- gam_result[[2]]
        dat <- data.frame(dat)
        tmval <- data.frame(
            Well.Position =
                unique(wellpos[grepl(
                    "[A-Z]+[0-9]+[0-9]$",
                    wellpos
                )]), TM = tmval
        )
        # left join seperate data frames into one
        all <- left_join(dat, tmval, by = "Well.Position")
        return(all)
    }
}


#' write output files
#'
#' writes output into csv or txt files
#'
#' @importFrom utils write.csv write.table
#'
#' @param data input data frame
#' @param name string, name file to be saved as. Final name will be appended
#'     "tsar_output"
#' @param file \code{file = "txt"} writes txt output files;
#'     \code{file = "csv"} writes csv output files;
#'     default set to \code{file = "txt"}
#' @return file output on the working directory where data was read in
#'
#' @family read_write_analysis
#' @examples
#' data("qPCR_data1")
#' result <- gam_analysis(qPCR_data1,
#'     smoothed = TRUE, fluo_col = 5,
#'     selections = c(
#'         "Well.Position", "Temperature", "Fluorescence", "Normalized"
#'     )
#' )
#' output_data <- read_tsar(result, output_content = 2)
#' # example does not run, will build excessive file in package
#' # write_tsar(output_data, name = "2022_03_18_test", file = "txt")
#'
#' @export
#'
write_tsar <- function(data, name, file = "txt") {
    file <- match.arg(file, choices = c("txt", "csv"))
    # write csv
    if (file == "csv") {
        rename <- paste(name, "tsar_output.csv", sep = "_")
        utils::write.csv(data,
            file = rename,
            row.names = FALSE,
            quote = FALSE
        )
    #write txt file
    } else if (file == "txt") {
        rename <- paste(name, "tsar_output.txt", sep = "_")
        utils::write.table(data,
            file = rename,
            row.names = FALSE,
            sep = "\t",
            quote = FALSE
        )
    }
}

print_plot <- function(input, output, dataset, checkrange, checklist, gg1) {
    output$distPlot <- plotly::renderPlotly({
        gg1_p <- TSAR::screen(dataset(),
            checkrange = checkrange,
            checklist = checklist
        )
        plotdata <- plotly::ggplotly(gg1_p, source = "plotdata")
        gg1(plotdata)
        plotly::event_register(gg1(), "plotly_click")
    })
}
print_click <- function(input, output, clicked_points) {
    output$info <- renderText({
        c("Selected Curve: ", clicked_points$legend_text)
    })
}
print_grid <- function(input, output, highlighted_cells) {
    output$highlighted_list <- renderText({
        c("Selected by Grid: ", highlighted_cells())
    })
}

#' Extract ggplot2 legend
#'
#' Function enables separation of legends from plots within the TSAR package.
#'
#' @import ggplot2
#' @param input_plot a ggplot2 object
#' @return two ggplots, one containing the legend and another containing
#'   all else.
#' @family TSA Plots
#' @export
#'
#' @examples
#' data("example_tsar_data")
#' boxplot <- TSA_boxplot(example_tsar_data,
#'     color_by = "Protein",
#'     label_by = "Ligand", separate_legend = FALSE
#' )
#' get_legend(boxplot)
#'
get_legend <- function(input_plot) {
    #obtain plot
    legend_plot <- input_plot
    legend_plot <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(legend_plot))
    legend <- which(vapply(legend_plot$grobs, function(x) x$name,
        FUN.VALUE = character(1)
    )
    == "guide-box")
    #obtain legend only if it exists
    if (length(legend) == 0) { # Prevents error
        warning("No Legend to Return")
    } else {
        legend <- legend_plot$grobs[[legend]]
        legend_plot <- ggpubr::as_ggplot(legend)
        legend_plot <- legend_plot + ggplot2::theme_void()
        return(legend_plot)
    }
}

graphpage <- function(tsar_data) {
    fluidPage(
        useShinyjs(),
        tags$style(
            HTML(
                ".sticky-panel {
                    position: sticky;
                    top: 0;
                    z-index: 100;
                    background-color: white;
                  }
                  "
            )
        ),
        shiny::checkboxInput("dialogueToggle",
            "Hide All Hints and Messages",
            value = FALSE
        ),
        fluidRow(
            column(
                width = 3,
                actionButton(
                    "toggleButton",
                    "Upload and Merge Data"
                )
            ),
            column(width = 2, shiny::uiOutput("hideBtn"))
        ),
        shinyjs::hidden(
            div(
                id = "myPanel",
                h3("Merge Replicate Trials"),
                fluidRow(
                    column(
                        width = 6,
                        shiny::fileInput("file",
                            label = "Upload All Analysis Files",
                            multiple = TRUE
                        )
                    ),
                    column(
                        width = 3,
                        shiny::actionButton("generate", "Merge and Save Data")
                    ),
                    column(
                        width = 3,
                        actionButton(
                            "savetolocal",
                            "Save to R"
                        )
                    ),
                ),
                div(
                    style = "font-size: 11px;",
                    shiny::tableOutput("table")
                ),
                shiny::uiOutput("date_boxes"),
                shiny::actionButton("save_dates", "Save Dates"),
                shiny::verbatimTextOutput("output")
            )
        ),
        br(),
        div(
            id = "bplot",
            class = "sticky-panel",
            shiny::plotOutput("Plot")
        ),
        shinyjs::hidden(
            div(
                class = "sticky-panel",
                id = "alternativeplot",
                plotly::plotlyOutput("altplot")
            )
        ),
        verbatimTextOutput("Plot_Message"),
        h3("Boxplot"),
        shiny::checkboxInput(
            "makeplotly", "Graph Interactive (legend cannot be separate)",
            value = FALSE, width = "600px"
        ),
        fluidRow(
            column(
                width = 2,
                shiny::selectInput("Color",
                    label = "Color:",
                    choices = c("Protein", "Ligand")
                )
            ),
            column(
                width = 2,
                shiny::selectInput("Label",
                    label = "Label:",
                    choices = c("Protein", "Ligand"),
                    selected = "Ligand"
                )
            ),
            column(
                width = 2,
                shiny::selectInput("Legend",
                    label = "Separate legend:",
                    choices = c(TRUE, FALSE), selected = FALSE
                )
            ),
            column(
                width = 3,
                shiny::selectInput("Control",
                    label = "Control Condition",
                    choices = c("NA", condition_IDs(tsar_data))
                )
            ),
            column(
                br(),
                width = 2,
                shiny::actionButton("Boxplot", "Generate Boxplot")
            )
        ),
        h3("Compare Plot"),
        fluidRow(
            column(
                width = 2,
                shiny::selectInput("y_axis",
                    label = "Graph y as:",
                    choices = c("Fluorescence", "RFU"),
                    selected = "RFU"
                )
            ),
            column(
                width = 2,
                shiny::selectInput("show_tm",
                    label = "Show Tm:",
                    choices = c(TRUE, FALSE)
                )
            ),
            column(
                width = 2,
                shiny::selectInput("title_by",
                    label = "Title by:",
                    choices = c("ligand", "protein", "both"),
                    selected = "both"
                )
            ),
            column(
                width = 2,
                shiny::selectInput(
                    "compare_smoother",
                    label = "Model used:",
                    choices = c("GAM", "beta"),
                    selected = "GAM"
                )
            ),
            column(
                width = 3,
                shiny::selectInput("Control_s",
                    label = "Control Condition",
                    choices = c(condition_IDs(tsar_data))
                )
            ),
            column(
                br(),
                width = 2,
                shiny::actionButton("Compareplot", "Generate Compare Plots")
            ),
            column(width = 2, uiOutput("plot_select"))
        ),
        h3("Condition Plot"),
        fluidRow(
            column(
                width = 2,
                shiny::selectInput("y_axis_c",
                    label = "Graph y as: ",
                    choices = c("Fluorescence", "RFU"),
                    selected = "RFU"
                )
            ),
            column(
                width = 2,
                shiny::selectInput("show_tm_c",
                    label = "Show Tm: ",
                    choices = c(TRUE, FALSE)
                )
            ),
            column(
                width = 2,
                shiny::selectInput(
                    "curves_smoother",
                    label = "Model used:",
                    choices = c("GAM", "beta"),
                    selected = "GAM"
                )
            ),
            column(
                width = 2,
                shiny::selectInput("separate_legend",
                    label = "Separate legend: ",
                    choices = c(TRUE, FALSE), selected = FALSE
                )
            ),
            column(
                width = 3,
                shiny::selectInput("Selected_condition",
                    label = "Select condition: ",
                    choices = c(condition_IDs(tsar_data))
                )
            ),
            column(
                br(),
                width = 2,
                shiny::actionButton("curves", "Graph Selected Curves")
            )
        ),
        fluidRow(
            column(
                width = 2,
                shiny::selectInput("smooth",
                    label = "Smooth curve: ",
                    choices = c(TRUE, FALSE)
                )
            ),
            column(
                width = 2,
                shiny::selectInput("average",
                    label = "Show average: ",
                    choices = c(TRUE, FALSE)
                )
            ),
            column(
                width = 2,
                shiny::numericInput("num",
                    label = "Shift Tm Label: ",
                    value = 7.5
                )
            )
        ),
        h3("Derivative Comparisons"),
        fluidRow(
            column(
                width = 2,
                shiny::selectInput("frame_by",
                    label = "Compare by: ",
                    choices = c("NA", subset(
                        names(tsar_data),
                        !names(tsar_data) %in% c(
                            "Temperature",
                            "Fluorescence",
                            "norm_deriv"
                        )
                    )),
                    selected = "NA"
                )
            ),
            column(
                br(),
                width = 2,
                shiny::actionButton("build_deriv", "Compare Derivatives")
            )
        ),
        hr(),
        h4("helper functions: "),
        fluidRow(
            column(
                width = 4,
                shiny::selectInput("remove_condition",
                    label = "Remove Condition_ID:",
                    multiple = TRUE,
                    choices = condition_IDs(tsar_data)
                )
            ),
            column(
                width = 4,
                shiny::selectInput("remove_well",
                    label = "Remove Well_ID:",
                    multiple = TRUE,
                    choices = well_IDs(tsar_data)
                )
            ),
            column(
                width = 2,
                br(),
                shiny::actionButton("Remove", "Remove Selected")
            ),
            column(
                width = 2,
                br(),
                shiny::actionButton("Restore", "Restore Data")
            )
        ),
        fluidRow(
            column(
                width = 2,
                shiny::actionButton("condition", "List Conditions IDs")
            ),
            column(
                width = 2,
                shiny::actionButton("well", "List Well IDs")
            )
        ),
        br(),
        shiny::verbatimTextOutput("Condition_ID"),
        shiny::verbatimTextOutput("Well_ID"),
        br(),
        fluidRow(
            column(
                width = 4,
                shiny::selectInput("control_tm",
                    label = "List Delta Tm:",
                    choices = c(
                        "N/A",
                        condition_IDs(tsar_data)
                    )
                )
            ),
            column(
                br(),
                width = 2,
                shiny::actionButton("tm", "List Tm")
            ),
        ),
        shiny::tableOutput("tmlist"),
        br(),
        shiny::actionButton("stopButton", "Close Window")
    )
}

#' TSA Well Curves Plot
#'
#' Generates the individual curves for each well in the merged tsa data input.
#'     Options to create an average and standard deviation sd of the plot
#'     in addition to the individual curves. The average and sd will be
#'     smoothened by linear regression; see \code{\link{TSA_average}} for
#'     details.
#'
#' @importFrom magrittr %>%
#' @importFrom dplyr group_by
#'
#' @inheritParams TSA_average
#' @param show_Tm logical; \code{show_Tm = TRUE} by default. When TRUE, the Tm
#'     is displayed on the plot. When FALSE, the Tm is not added to the plot.
#' @param Tm_label_nudge numeric; \code{Tm_label_nudge = 7.5} the direction
#'     in the x direction to move the Tm label. This is used prevent the label
#'     from covering data. Ignored if \code{show_Tm = FALSE}.
#' @param show_average logical; \code{show_average = TRUE} by default.
#'     When TRUE, the average is and sd is plotted as
#'     generated by \code{\link{merge_TSA}}.
#' @param smooth logical; \code{smooth = TRUE} by default. When TRUE, linear
#'     regression by \code{\link[mgcv]{gam}} is used to make clean lines on
#'     the plot. See \code{\link{TSA_average}} for more details. When FALSE,
#'     individual points are plotted (slows down rendering).
#' @param plot_title,plot_subtitle characer string, NA by default.
#'     User-specified plots to overright automatic naming.
#' @param separate_legend logical; \code{separate_legend = TRUE} by default.
#'     When TRUE, the ggplot2 legend is separated from the TSA curve.
#'     This is to help with readability. One ggplot is returned when FALSE.
#' @param smoother character; one of \code{c("gam","beta","none")}.
#'     Passed to \code{\link{TSA_average}} to select the aggregate smoother:
#'     \code{"gam"} uses \pkg{mgcv}, \code{"beta"} uses a natural cubic spline
#'     with Beta(a,a) interior knots centered at Tm, and \code{"none"} uses the
#'     unsmoothed average. Default follows \code{TSA_average}.
#' @param beta_shape numeric; shape parameter \eqn{a} for the Beta(a,a) knot
#'     placement when \code{smoother = "beta"}. \code{beta_shape = 3} by default. Passed to \code{TSA_average}.
#' @param beta_n_knots integer or \code{NULL}; number of interior knots when
#'     \code{smoother = "beta"}. If \code{NULL}, uses \code{beta_knots_frac}.
#'     Passed to \code{TSA_average}.
#' @param beta_knots_frac numeric in (0,1); fraction of unique temperatures used
#'     as interior knots when \code{smoother = "beta"} and \code{beta_n_knots}
#'     is \code{NULL}. \code{beta_knots_frac = 0.008} by default. Passed to \code{TSA_average}.
#' @param use_natural logical; if TRUE (default) uses natural cubic spline
#'     basis for the beta method. Passed to \code{TSA_average}.
#'
#' @return By default, two ggplots are returned: one TSA curve and one legend
#'     key (when \code{separate_legend = TRUE}). When
#'     \code{separate_legend = FALSE}, a single ggplot is returned.
#'
#' @family TSA Plots
#' @seealso \code{\link{merge_TSA}} and \code{\link{normalize_fluorescence}}
#'     for preparing data. See \code{\link{TSA_average}} and
#'     \code{\link{get_legend}} for details on function parameters.
#' @examples
#' data("example_tsar_data")
#' check <- subset(example_tsar_data, condition_ID == "CA FL_PyxINE HCl")
#' TSA_wells_plot(check, y = "Fluorescence", smooth = TRUE, separate_legend = TRUE,
#'                smoother = "beta", beta_shape = 4, beta_knots_frac = 0.008)
#'
#' @export

TSA_wells_plot <- function(
    tsa_data,
    y = "RFU",
  show_Tm = TRUE,
    Tm_label_nudge = 7.5,
  show_average = TRUE,
    plot_title = NA,
    plot_subtitle = NA,
  smooth = TRUE,                 # lines for wells + smooth aggregate if TRUE
  separate_legend = TRUE,
    # NEW: second-stage smoother for the aggregate curve/ribbon
    smoother = c("gam", "beta", "none"),
    beta_shape = 3,
    beta_n_knots = NULL,
    beta_knots_frac = 0.12,
    use_natural = TRUE
) {
  smoother <- match.arg(smoother)
  y <- match.arg(y, choices = c("Fluorescence", "RFU"))
  # sanitize logical flags (accept logical or "TRUE"/"FALSE" strings)
  .to_bool <- function(x) { if (is.logical(x)) x else (tolower(as.character(x)) == "true") }
  show_Tm_flag      <- .to_bool(show_Tm)
  show_average_flag <- .to_bool(show_average)
  smooth_flag       <- .to_bool(smooth)
  sep_flag          <- .to_bool(separate_legend)
  Tm_label_nudge_num <- suppressWarnings(as.numeric(Tm_label_nudge))
  if (!is.finite(Tm_label_nudge_num)) Tm_label_nudge_num <- 0
  
  if (!"well_ID" %in% names(tsa_data) || !"condition_ID" %in% names(tsa_data)) {
    stop("tsa_data must be a data frame merged by merge_TSA() or merge_norm()")
  }
  
  # Base plot per-well: lines if smooth, otherwise points
  if (y == "Fluorescence") {
    TSA_curve <- ggplot(tsa_data, aes(x = Temperature, y = Fluorescence))
    tm_height <- max(tsa_data$Fluorescence, na.rm = TRUE) / 4
  } else { # RFU
    if (!"RFU" %in% names(tsa_data)) {
      stop("RFU column not found. Run normalize_fluorescence() or choose y='Fluorescence'.")
    }
    TSA_curve <- ggplot(tsa_data, aes(x = Temperature, y = RFU))
    tm_height <- 0.4
  }
  
  if (isTRUE(smooth_flag)) {
    TSA_curve <- TSA_curve + geom_line(aes(color = well_ID), alpha = 0.95)
  } else {
    TSA_curve <- TSA_curve + geom_point(aes(color = well_ID), alpha = 0.5)
  }
  
  TSA_curve <- TSA_curve + theme_bw()
  
  # Optional Tm line/label
  if (isTRUE(show_Tm_flag)) {
    # protect against vector of Tm; pick the first (single condition expected)
    avg_tm <- suppressWarnings(as.numeric(TSA_Tms(tsa_data)$Avg_Tm[1]))
    if (is.finite(avg_tm)) {
      TSA_curve <- TSA_curve +
        geom_vline(xintercept = avg_tm, linetype = "dashed", color = "#BC9595") +
        annotate(
          "label",
          x = avg_tm + Tm_label_nudge_num,
          y = tm_height,
          label = paste0("Tm=", round(avg_tm, 2), "C")
        )
    }
  }
  
  # Titles
  if (is.na(plot_title)) {
    if (TSA_proteins(tsa_data = tsa_data, n = TRUE) == 1) {
      title <- paste0("Thermal Profile of ", TSA_proteins(tsa_data = tsa_data))
    } else {
      title <- "Thermal Profile"
    }
  } else {
    title <- plot_title
  }
  
  subtitle <- if (is.na(plot_subtitle)) {
    paste(c("With:", TSA_ligands(tsa_data = tsa_data)), collapse = " ")
  } else {
    plot_subtitle
  }
  
  TSA_curve <- TSA_curve + labs(title = title, subtitle = subtitle)
  
  # Legend handling
  if (isTRUE(sep_flag)) {
    legend_plot <- get_legend(TSA_curve)
    TSA_curve <- TSA_curve + theme(legend.position = "none")
    TSA_return <- list(TSA_curve, legend_plot)
  } else {
    TSA_return <- TSA_curve
  }
  
  # Aggregate average curve + ribbon
  if (isTRUE(show_average_flag)) {
    # We use 'smooth' to decide whether to create smoothed columns in TSA_average,
    # and 'smoother' to choose the method (none/gam/beta) for those columns.
    tsa_average_df <- TSA_average(
      tsa_data = tsa_data,
      y = y,
      avg_smooth = smooth,
      sd_smooth  = smooth,
      smoother   = smoother,
      beta_shape = beta_shape,
      beta_n_knots = beta_n_knots,
      beta_knots_frac = beta_knots_frac,
      use_natural = use_natural
    )
    
    # Safe columns for plotting (works for any smoother selection)
  tsa_average_df$ymin_plot <- if (smooth_flag && "sd_min_smooth" %in% names(tsa_average_df))
      tsa_average_df$sd_min_smooth else tsa_average_df$sd_min
  tsa_average_df$ymax_plot <- if (smooth_flag && "sd_max_smooth" %in% names(tsa_average_df))
      tsa_average_df$sd_max_smooth else tsa_average_df$sd_max
  tsa_average_df$y_avg_plot <- if (smooth_flag && "avg_smooth" %in% names(tsa_average_df))
      tsa_average_df$avg_smooth else tsa_average_df$average

  # choose correct base plot depending on class (ggplot is also a list)
  base_plot <- if (inherits(TSA_return, "ggplot")) TSA_return else TSA_return[[1]]

    base_plot <- base_plot +
      geom_ribbon(
        inherit.aes = FALSE,
        data = tsa_average_df,
        aes(x = Temperature, ymin = ymin_plot, ymax = ymax_plot),
        alpha = 0.4
      ) +
      geom_line(
        inherit.aes = FALSE,
        linetype = "dotdash",
        data = tsa_average_df,
        aes(x = Temperature, y = y_avg_plot)
      )
    
    # Put back into return container
    if (isTRUE(separate_legend)) {
      TSA_return[[1]] <- base_plot
    } else {
      TSA_return <- base_plot
    }
  }
  
  return(TSA_return)
}

# small helper for null-coalescing (if not already defined somewhere in your codebase)
`%||%` <- function(a, b) if (!is.null(a)) a else b


#' TSA Box Plot
#'
#' Generates a box and whiskers plot for each condition specified. This
#'     is used to compare Tm values between the data set.
#'     See \code{\link{Tm_difference}} for details.
#'
#' @inheritParams TSA_average
#' @inheritParams TSA_wells_plot
#' @param control_condition Either a condition_ID or NA; NA by default.
#'     When a valid Condition ID is provided, a vertical line appears at the
#'     average Tm for the specified condition. When NA, this is skipped.
#' @param color_by character string, either c("Ligand", "Protein").
#'     The condition category to color the boxes within the box
#'     plot for comparison. This is represented in the legend.
#'     Set to NA to skip.
#' @param label_by character string, either c("Ligand", "Protein").
#'     The condition category to group the boxes within the box
#'     plot. This is represented in the axis. Set to NA to skip.
#' @return by default, two ggplots are returned: one TSA curve and one key.
#'     When \code{separate_legend = FALSE} one ggplot is returned.
#' @family TSA Plots
#' @seealso \code{\link{merge_TSA}}
#'     for preparing data. See \code{\link{Tm_difference}} and
#'     \code{\link{get_legend}} for details on function parameters.
#' @examples
#' data("example_tsar_data")
#' TSA_boxplot(example_tsar_data,
#'     color_by = "Protein",
#'     label_by = "Ligand", separate_legend = FALSE
#' )
#' @export

TSA_boxplot <- function(
    tsa_data,
    control_condition = NA, # Either a condition_ID or NA
    color_by = "Protein", # To skip, set as NA.
    label_by = "Ligand", # If not "Ligand" or "Protein", default order is used
    separate_legend = TRUE # Logical
    ) {

    color_by <- match.arg(color_by, choices = c("Protein", "Ligand", "NA"))
    label_by <- match.arg(label_by, choices = c("Protein", "Ligand", "NA"))

    plot_data <- TSA_Tms(
        analysis_data = tsa_data,
        condition_average = FALSE
    )
    plot_data <- plot_data[!is.na(plot_data$Tm), ]

    # If changing condition is not specified, calculate which varies more
    if (!color_by %in% c("Ligand", "Protein")) {
        n_proteins <- TSA_proteins(plot_data, n = TRUE)
        n_ligands <- TSA_ligands(plot_data, n = TRUE)
        if (n_ligands > n_proteins) {
            color_by <- "Ligand"
        } else {
            color_by <- "Protein"
        }
    }

    if (color_by == "Ligand") {
        tsa_plot <- ggplot(
            plot_data,
            aes(
                x = condition_ID,
                y = Tm,
                color = Ligand,
                label = well_ID
            )
        ) +
            geom_boxplot(alpha = 0.25) +
            geom_point(shape = 1) +
            scale_color_discrete(unique(tsa_data$Ligand),
                name = "Ligand"
            )
    }
    if (color_by == "Protein") {
        tsa_plot <- ggplot(
            plot_data,
            aes(
                x = condition_ID,
                y = Tm,
                color = Protein,
                label = well_ID
            )
        ) +
            geom_boxplot(alpha = 0.25) +
            geom_point(shape = 1) +
            scale_color_discrete(unique(tsa_data$Protein),
                name = "Protein"
            )
    }

    if (label_by == "Ligand") {
        tsa_plot <- tsa_plot +
            scale_x_discrete(
                breaks = plot_data$condition_ID,
                labels = plot_data$Ligand,
                name = label_by
            )
    }
    if (label_by == "Protein") {
        tsa_plot <- tsa_plot +
            scale_x_discrete(
                breaks = plot_data$condition_ID,
                labels = plot_data$Protein,
                name = label_by
            )
    }


    if (!is.na(control_condition)) {
        if (control_condition %in% condition_IDs(tsa_data)) {
            ctrl_Tm <-
                TSA_Tms(analysis_data = tsa_data[tsa_data$condition_ID ==
                    control_condition, ])
            ctrl_Tm <- ctrl_Tm$Avg_Tm
            tsa_plot <- tsa_plot +
                geom_hline(
                    yintercept = ctrl_Tm,
                    color = "grey",
                    linetype = 2,
                    alpha = 0.9
                )
        } else {
            stop("condition_ID assigned to control_condition is not
                found in the TSA data. Use condition_IDs(tsa_data) to get
                 the list of acceptable IDs")
        }
    }

    tsa_plot <- tsa_plot +
        coord_flip() +
        theme_bw() +
        labs(y = expression("T"["m"] ~ "(" * degree * "C)"))

    if (separate_legend) {
        legend_plot <- get_legend(tsa_plot)
        tsa_plot <- tsa_plot + theme(legend.position = "none")
        plot_list <- list(tsa_plot, legend_plot)
        return(plot_list)
    } else {
        tsa_plot <- tsa_plot
        return(tsa_plot)
    }
}

#' Compare TSA curves to control
#'
#'
#' Generate a number of plots based on the input data to compare the average
#' and standard deviation (sd) of each unique condition to a specified
#' control condition. To see all conditions use \code{condition_IDs(tsa_data)}.
#'
#' @inheritParams TSA_average
#' @inheritParams Tm_difference
#' @param show_Tm logical; \code{show_Tm = TRUE} by default. When TRUE, the Tm
#'     is displayed on the plot. When FALSE, the Tm is not added to the plot.
#' @param title_by character string; c("ligand", "protein", "both").
#'     Automatically names the plots by the specified condition category.
#' @param digits integer; the number of decimal places to round for change in Tm
#'     calculations displayed in the subtitle of each plot.
#' @param smoother character; one of \code{c("gam","beta","none")}.
#'     Passed to \code{\link{TSA_average}} to select the aggregate smoother:
#'     \code{"gam"} uses \pkg{mgcv}, \code{"beta"} uses a natural cubic spline
#'     with Beta(a,a) interior knots centered at Tm, and \code{"none"} uses the
#'     unsmoothed average. Default follows \code{TSA_average}.
#' @param smooth_conditions logical; \code{smooth_conditions = TRUE} by default.
#'   Controls whether each non-control condition curve is drawn using smoothed
#'   aggregates from \code{\link{TSA_average}}. When \code{TRUE}, the condition
#'   overlay uses \code{avg_smooth} and the smoothed standard deviation bounds
#'   (\code{sd_min_smooth}, \code{sd_max_smooth}) when available. When \code{FALSE},
#'   the overlay uses the unsmoothed mean (\code{average}) and raw bounds
#'   (\code{sd_min}, \code{sd_max}). The control curve remains smoothed.
#'   (Passed as \code{avg_smooth} and \code{sd_smooth} to \code{TSA_average}.)
#' @param beta_shape numeric; shape parameter \eqn{a} for the Beta(a,a) knot
#'     placement when \code{smoother = "beta"}. \code{beta_shape = 3} by default. Passed to \code{TSA_average}.
#' @param beta_n_knots integer or \code{NULL}; number of interior knots when
#'     \code{smoother = "beta"}. If \code{NULL}, uses \code{beta_knots_frac}.
#'     Passed to \code{TSA_average}.
#' @param beta_knots_frac numeric in (0,1); fraction of unique temperatures used
#'     as interior knots when \code{smoother = "beta"} and \code{beta_n_knots}
#'     is \code{NULL}. \code{beta_knots_frac = 0.008} by default. Passed to \code{TSA_average}.
#' @param use_natural logical; if TRUE (default) uses natural cubic spline
#'     basis for the beta method. Passed to \code{TSA_average}.
#'     
#' @return A \emph{named list} of ggplot objects. One plot per non-control
#'     condition is included, each overlaid against the control. A final plot for
#'     the control alone is appended and named \code{"Control: <control_condition>"}. 
#'     
#' @family TSA Plots
#' @seealso \code{\link{merge_TSA}} and \code{\link{normalize_fluorescence}}
#'     for preparing data. See \code{\link{TSA_average}} and
#'     \code{\link{get_legend}} for details on function parameters.
#'     See \code{\link{TSA_wells_plot}} for individual curves of the averaged
#'     conditions shown.
#' @examples
#' data("example_tsar_data")
#' TSA_compare_plot(example_tsar_data,
#'     y = "Fluorescence",
#'     control_condition = "CA FL_DMSO",
#'     smoother = "beta",
#'     beta_shape = 4, 
#'     beta_knots_frac = 0.008
#' )
#' @export
#' 
TSA_compare_plot <- function(
    tsa_data,
    control_condition,
    y = "Fluorescence",
    show_Tm = TRUE,
    title_by = "both",
    digits = 1,
    # new options:
    smoother   = c("gam", "beta", "none"),
    smooth_conditions = TRUE,
    beta_shape      = 3,
    beta_n_knots    = NULL,
    beta_knots_frac = 0.12,
    use_natural     = TRUE
) {

  smoother <- match.arg(smoother)
  y <- match.arg(y, choices = c("Fluorescence", "RFU"))

  if (!"well_ID" %in% names(tsa_data) || !"condition_ID" %in% names(tsa_data)) {
    stop("tsa_data must be a data frame merged by merge_TSA()")
  } else if (!control_condition %in% condition_IDs(tsa_data)) {
    stop("control_condition must be a value from tsa_data$condition_ID")
  }

  Tms_df <- TSA_Tms(tsa_data)
  control_avg <- Tms_df$Avg_Tm[Tms_df$condition_ID == control_condition]

  # --- control curve ---
  control_df <- tsa_data[tsa_data$condition_ID == control_condition, ]
  control_df <- TSA_average(
    tsa_data = control_df, y = y,
    avg_smooth = TRUE, sd_smooth = TRUE,
    smoother = smoother,
    beta_shape = beta_shape, beta_n_knots = beta_n_knots,
    beta_knots_frac = beta_knots_frac, use_natural = use_natural
  )

  # pick safe columns for control plotting
  control_df$y_plot   <- if ("avg_smooth"    %in% names(control_df)) control_df$avg_smooth    else control_df$average
  control_df$ymin_rib <- if ("sd_min_smooth" %in% names(control_df)) control_df$sd_min_smooth else control_df$sd_min
  control_df$ymax_rib <- if ("sd_max_smooth" %in% names(control_df)) control_df$sd_max_smooth else control_df$sd_max

  control_curve <- ggplot(
    control_df,
    aes(x = Temperature, y = y_plot)
  ) +
    geom_ribbon(
      aes(ymin = ymin_rib, ymax = ymax_rib),
      alpha = 0.4, fill = "black"
    ) +
    geom_line(linetype = "dotdash", color = "black")

  # palette
  colfunc <- grDevices::colorRampPalette(c("red", "blue"))
  col_vect <- colfunc(condition_IDs(tsa_data, n = TRUE))

  Tm_difference_DF <- Tm_difference(
    tsa_data = tsa_data,
    control_condition = control_condition
  )

  curve_list <- as.list(rep(NA, condition_IDs(tsa_data, n = TRUE)))

  for (i in seq_len(condition_IDs(tsa_data, n = TRUE))) {
    condition_i <- condition_IDs(tsa_data)[i]
    tm_avg_i <- Tms_df$Avg_Tm[Tms_df$condition_ID == condition_i]

    Tm_diff_i <- Tm_difference_DF$delta_Tm[
      Tm_difference_DF$condition_ID == condition_i
    ]
    Tm_diff_i <- round(Tm_diff_i, digits = digits)

    title_i <- condition_i
    subtitle_i <- paste("Tm = ", Tm_diff_i, "C", sep = "")
    ctrl_subtitle <- control_condition

    if (title_by == "ligand") {
      title_i <- Tm_difference_DF$Ligand[Tm_difference_DF$condition_ID == condition_i]
      ctrl_subtitle <- Tm_difference_DF$Ligand[Tm_difference_DF$condition_ID == control_condition]
    }
    if (title_by == "protein") {
      title_i <- Tm_difference_DF$Protein[Tm_difference_DF$condition_ID == condition_i]
      ctrl_subtitle <- Tm_difference_DF$Protein[Tm_difference_DF$condition_ID == control_condition]
    }
    if (title_by == "both") {
      title_i <- paste(
        Tm_difference_DF$Protein[Tm_difference_DF$condition_ID == condition_i],
        " + ",
        Tm_difference_DF$Ligand[Tm_difference_DF$condition_ID == condition_i],
        sep = ""
      )
      ctrl_subtitle <- paste(
        Tm_difference_DF$Protein[Tm_difference_DF$condition_ID == control_condition],
        " + ",
        Tm_difference_DF$Ligand[Tm_difference_DF$condition_ID == control_condition],
        sep = ""
      )
    }

    cond_df_i <- tsa_data[tsa_data$condition_ID == condition_i, ]
    cond_df_i <- TSA_average(
      tsa_data = cond_df_i, y = y,
      avg_smooth = smooth_conditions, sd_smooth = smooth_conditions,
      smoother = smoother,
      beta_shape = beta_shape, beta_n_knots = beta_n_knots,
      beta_knots_frac = beta_knots_frac, use_natural = use_natural
    )

    # pick safe columns for condition plotting
    cond_df_i$y_line   <- if (smooth_conditions && "avg_smooth"    %in% names(cond_df_i)) cond_df_i$avg_smooth    else cond_df_i$average
    cond_df_i$ymin_rib <- if (smooth_conditions && "sd_min_smooth" %in% names(cond_df_i)) cond_df_i$sd_min_smooth else cond_df_i$sd_min
    cond_df_i$ymax_rib <- if (smooth_conditions && "sd_max_smooth" %in% names(cond_df_i)) cond_df_i$sd_max_smooth else cond_df_i$sd_max

    diff_curve_i <- control_curve +
      geom_ribbon(
        data = cond_df_i,
        aes(x = Temperature, ymin = ymin_rib, ymax = ymax_rib),
        alpha = 0.4, fill = col_vect[i],
        inherit.aes = FALSE
      ) +
      geom_line(
        data = cond_df_i,
        aes(x = Temperature, y = y_line),
        color = col_vect[i],
        inherit.aes = FALSE
      )

    # --- FIX: restore BOTH vertical Tm lines in comparison plots (control + condition) ---
    if (isTRUE(show_Tm)) {
      ctrl_tm <- suppressWarnings(as.numeric(control_avg[1]))
      cond_tm <- suppressWarnings(as.numeric(tm_avg_i[1]))

      if (is.finite(ctrl_tm)) {
        diff_curve_i <- diff_curve_i +
          geom_vline(xintercept = ctrl_tm, linetype = "dashed", color = "black")
      }
      if (is.finite(cond_tm)) {
        diff_curve_i <- diff_curve_i +
          geom_vline(xintercept = cond_tm, linetype = "dashed", color = col_vect[i])
      }
    }

    diff_curve_i <- diff_curve_i +
      theme_bw() +
      labs(title = title_i, subtitle = subtitle_i)

    curve_list[[i]] <- diff_curve_i
    names(curve_list)[i] <- condition_i
  }

  # -- add control plot to list
  control_curve <- control_curve +
    labs(title = "Control", subtitle = ctrl_subtitle) +
    theme_bw()

  if (isTRUE(show_Tm)) {
    ctrl_tm <- suppressWarnings(as.numeric(control_avg[1]))
    if (is.finite(ctrl_tm)) {
      control_curve <- control_curve +
        geom_vline(xintercept = ctrl_tm, linetype = "dashed", color = "#BC9595")
    }
  }

  curve_list[names(curve_list) == control_condition] <- NULL
  curve_list[[length(curve_list) + 1]] <- control_curve
  names(curve_list)[length(curve_list)] <- paste("Control: ", control_condition, sep = "")

  return(curve_list)
}

#' Rescale values between minimum and maximum.
#'
#' For a vector of numeric values, the minimum and maximum values are determined
#'     and each value of the vector is rescaled between 0 and 1.
#'     Values near 0 are close to the minimum, values near 1 are close to the
#'     max. This function is utilized by other TSAR functions.
#' @param x a numeric vector to be rescaled
#' @return A numeric vector of rescaled values.
#' @family TSAR Formatting
#' @examples
#' x <- c(0, 1, 3)
#' rescale(x)
#'
#' @export
#'

rescale <- function(x) { # Rescale values between two values.
    rescaled_v <- (x - min(x)) / (max(x) - min(x))
    return(rescaled_v)
}


#' Normalize Fluorescence Curve
#'
#' This function will take the TSA data and normalize the arbitrary fluorescence
#'     measurements based on the specified method. Each well, determined by a
#'     unique well ID, is normalized independently.
#'     All measurements can be normalized to the minimum or maximum value.
#'     Alternatively, setting  \code{by = "rescale"} (the default) will cause
#'     all values to be normalized between the minimum and
#'     maximum values, with the maximum  = 1 and the minimum = 0 and all other
#'     values normalized in-between. Finally, the user can supply a single
#'     value or vector of values to normalize the data to. The returned data
#'     frame will be the input tsa data frame with a new column named "RFU"
#'     containing the normalized TSA data.
#' @param tsa_data a data frame that is unmerged and generated by
#'     TSAR::read_raw_data() or a merged TSA data frame generated by
#'     TSAR::merge_TSA(). Data frames require a column named "Fluorescence"
#'    containing numeric values for normalizing.
#' @param by character string; either c("rescale", "min", "max", "control").
#'     \code{by = "rescale"} by default, scaling Fluorescence values in-between
#'     the minimum and maximum observation. Each well can be
#'     normalized to either the minimum or maximum value with
#'     \code{by = "min"} or \code{by = "max"}, respectively. To normalize
#'     all values to a numeric value or vector, set \code{by = "control"}.
#' @param control_vect numeric vector to normalize the column "Fluorescence"
#'     to. An individual number will normalize all measurements to be normalized
#'     to it. The vector will need to align with tsa_data$Fluorescence. Ensure
#'     \code{by = "control"}, else the supplied vector will be ignored.
#'
#' @return a data frame identical to the tsa_data input with a new column
#'     named "RFU" containing the normalized values
#' @family TSAR Formatting
#' @seealso \code{\link{read_raw_data}} and  \code{\link{merge_TSA}}
#'     for loading data.
#' @examples
#' # examples not ran without example dataset
#' # raw_data <- read_raw_data(raw_data_path)
#' # normalize_fluorescence(raw_data, by == "control)
#'
#' @export
#'

normalize_fluorescence <- function(
    tsa_data = tsa_data,
    by = "rescale",
    control_vect = NA) {

    by <- match.arg(by, choices = c("rescale", "max", "min", "control"))

    if ("well_ID" %in% names(tsa_data)) {
        well_ids <- unique(tsa_data$well_ID)
        n_well_ids <- length(well_ids)
    } else {
        stop("A data frame loaded by read_raw_data() must be supplied")
    }
    tsa_data$RFU <- rep(NA, nrow(tsa_data))

    if (by == "rescale") { # Rescale between the min and max
        tsa_data$RFU <- unlist(lapply(well_ids, function(well_id) {
            tsa_data$Fluorescence[tsa_data$well_ID == well_id] <-
                rescale(tsa_data$Fluorescence[tsa_data$well_ID == well_id])
            tsa_data$Fluorescence[tsa_data$well_ID == well_id]
        }))

    }
    if (by == "max") { # Rescale by max
        tsa_data$RFU <- unlist(lapply(well_ids, function(well_id) {
            tsa_data$Fluorescence[tsa_data$well_ID == well_id] <-
                tsa_data$Fluorescence[tsa_data$well_ID == well_id] /
                max(tsa_data$Fluorescence[tsa_data$well_ID == well_id])
            tsa_data$Fluorescence[tsa_data$well_ID == well_id]
        }))

    }
    if (by == "min") { # Rescale by min
        tsa_data$RFU <- unlist(lapply(well_ids, function(well_id) {
            tsa_data$RFU[tsa_data$well_ID == well_id] <-
                tsa_data$Fluorescence[tsa_data$well_ID == well_id] /
                    min(tsa_data$Fluorescence[tsa_data$well_ID == well_id])
            tsa_data$Fluorescence[tsa_data$well_ID == well_id]
        }))
    }
    if (by == "control") { # Rescale by user supplied vector
        tsa_data$RFU <- unlist(lapply(well_ids, function(well_id) {
            tsa_data$RFU[tsa_data$well_ID == well_id] <-
                tsa_data$Fluorescence[tsa_data$well_ID == well_id] /
                control_vect
            tsa_data$Fluorescence[tsa_data$well_ID == well_id]
        }))
    }
    return(tsa_data)
}

#' Find inflection point function
#'
#' Looks for Tm temperature values by finding the inflection point in the
#'     fluorescence data. The inflection point is approximated by
#'     locating the maximum first derivative stored in "norm_deriv" column.
#'
#' @param norm_data data frame; data frame input containing derivative values
#'     can only be data frames for one well; finding inflections points across
#'     multiple wells require iteration through individual wells
#' @param min restricts finding to be above the given minimum temperature
#' @param max restricts finding to be below the given maximum temperature
#'     parameter min and max can be used to remove messy or undesired data
#'     for better accuracy in tm estimation; removing data is  before fitting
#'     the model is more recommended than removing here
#'
#' @return integer; tm estimation
#'
#' @family tsa_analysis
#'
#' @examples
#' data("qPCR_data1")
#' test <- subset(qPCR_data1, Well.Position == "A01")
#' test <- normalize(test, fluo = 5, selected = c(
#'     "Well.Position", "Temperature",
#'     "Fluorescence", "Normalized"
#' ))
#' gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
#' fit <- model_fit(test, model = gammodel)
#' Tm_est(fit)
#'
#' @export
#'
Tm_est <- function(norm_data, min, max) {
    # if min and max are not specified, default to check across the graph
    if (missing(min) && missing(max)) {
        return(norm_data$Temperature[which.max(norm_data$norm_deriv)])
    } else { # allow input to restrict domain of search for inflection points
        x <- norm_data[norm_data$Temperature >= min &
            norm_data$Temperature <= max, ]
        return(x$Temperature[which.max(norm_data$norm_deriv)])
    }
}


#' Analysis of all 96 wells through gam modeling
#'
#' Function pipeline that combines separated functions and iterate through
#'     each well to estimate the Tm.
#' @importFrom mgcv gam
#' @importFrom dplyr select mutate
#'
#' @param raw_data data frame; raw data frame
#' @param keep Boolean; set to \code{keep = TRUE}
#'     by default to return normalized data and fitted data
#' @param smoothed Boolean; set to \code{smoothed = FALSE} by default,
#'     if data is already smoothed, set smoothed to true
#' @param boltzmann Boolean; set to \code{boltzmann = FALSE} by default. Set
#'     to \code{boltzmann = TRUE} if a botlzmann fit is preferred.
#' @param fit Boolean; set to \code{fit = FALSE} by default, \code{fit = TRUE}
#'     returns access to information of each model fit. Not accessible in shiny.
#' @param selections list of characters; the variables in raw data user intends
#'     to keep. It is set, by default, to \code{c("Well.Position",
#'     "Temperature", "Fluorescence", "Normalized")}.
#' @param fluo_col integer; the Fluorescence variable column id
#'     (e.g. fluo = 5 when 5th column of data frame is the Fluorescence value)
#'     if fluorescence variable is named exactly as "Fluorescence", fluo does
#'     not need to be specified.
#'
#' @return List of data frames, list of three data frame outputs,
#'     Tm estimation by well, data set, fit of model by well.
#'
#' @family tsa_analysis
#'
#' @examples
#' data("qPCR_data1")
#' gam_analysis(qPCR_data1,
#'     smoothed = TRUE, boltzmann = FALSE, fluo_col = 5,
#'     selections = c(
#'         "Well.Position", "Temperature", "Fluorescence",
#'         "Normalized"
#'     )
#' )
#' model <- gam_analysis(qPCR_data1, smoothed = FALSE, boltzmann = TRUE)
#'
#' @export
gam_analysis <- function(
        raw_data,
        keep = TRUE,
        fit = FALSE,
        smoothed = FALSE,
        boltzmann = FALSE,
        fluo_col = NA,
        selections = c(
            "Well.Position",
            "Temperature",
            "Fluorescence",
            "Normalized"
        )) {

    # Function to process each well
    process_well <- function(i) {
        by_well <- subset(raw_data, Well.Position == i)
        by_well <- normalize(
            raw_data = by_well,
            fluo = fluo_col,
            selected = selections
        )

        # Initialize variables for each well
        tm <- c()
        kept <- data.frame()
        fitsummaries <- list()

        # fit model if not smoothed
        if (!smoothed) {
            if (boltzmann) {
                model <- run_boltzmann(norm_data = by_well)
            } else {
                model <- model_gam(
                    norm_data = by_well,
                    x = by_well$Temperature,
                    y = by_well$Normalized
                )
            }

            # check if user wishes to keep summaries of each well's model fit
            if (fit) {
                newsum <- list(summary(model))
                fitsummaries <- append(fitsummaries, newsum)
            }

            # calculate derivatives and append derivated using fitted values
            by_well <- model_fit(norm_data = by_well, model = model)
        } else {
            # if data smoothing is already present, not modeling required
            # calculate derivatives using smoothed data, append derivatives
            by_well <- mutate(by_well,
                        norm_deriv = c(diff(by_well$Normalized) /
                                        diff(by_well$Temperature), NA))
        }

        # estimate tm values and concat them into one list
        tm <- c(tm, Tm_est(by_well))

        # if user wishes to keep all fitted data
        if (keep) {
            if (!smoothed) {
                # keep selected variables, fitted values, and derivatives
                by_well <- dplyr::select(
                    by_well,
                    all_of(c(unlist(selections), "fitted", "norm_deriv"))
                )
            } else {
                # keep selected variables and derivatives
                by_well <- dplyr::select(
                    by_well,
                    all_of(c(unlist(selections), "norm_deriv"))
                )
            }
            # concat data frame of each well into one big data frame
            kept <- rbind(kept, by_well)
        }

        return(list(tm = tm, kept = kept, fitsummaries = fitsummaries))
    }

    # Apply process_well to each unique well
    unique_well_positions <- unique(raw_data$Well.Position)
    result_list <- lapply(unique_well_positions, process_well)

    # Combine results
    tm_list <- lapply(result_list, function(result) result$tm)
    kept_list <- lapply(result_list, function(result) result$kept)
    fitsummaries <- unlist(lapply(result_list,
                                    function(result) result$fitsummaries),
                                    recursive = FALSE)

    # Combine the lists into single data frames
    tm <- data.frame(do.call(rbind, tm_list))
    colnames(tm) <- "tm"
    kept <- data.frame(do.call(rbind, kept_list))

    # Prepare output list
    tobereturned <- list(tm)

    if (keep) {
        tobereturned <- append(tobereturned, list(kept))
    }

    if (fit) {
        tobereturned <- append(tobereturned, list(fitsummaries))
    }

    return(tobereturned)
}

#' Calculate Tm difference for all conditions
#'
#' From a specified control condition, the change in Tm is calculated for
#'     each condition in the tsa_data. Specifically,
#'     \eqn{Tm = condition - control}. Individual Tm values are averaged by
#'     condition, see \code{\link{TSA_average}} for details.
#'     To see all conditions use \code{condition_IDs(tsa_data)}.
#' @inheritParams TSA_average
#' @param control_condition character string matching a Condition ID. Must be
#'     equal to a value within tsa_data$condition_ID. See unique condition IDs
#'     with \code{\link{condition_IDs}}.
#' @return a data frame of reformatted data with the \code{\link{TSA_average}}
#'     data and the Tm.
#' @family TSAR Formatting
#' @seealso \code{\link{merge_TSA}} for preparing data.
#'     \code{\link{TSA_average}} for more information on the output data.
#'     \code{\link{condition_IDs}} to get unique Condition IDs within the
#'     input.
#'     \code{\link{TSA_boxplot}} for application.
#' @examples
#' data("example_tsar_data")
#' control <- condition_IDs(example_tsar_data)[1]
#' Tm_difference(example_tsar_data, control_condition = control)
#'
#' @export
Tm_difference <-
    function(tsa_data,
             control_condition) {
        TM_DF <- TSA_Tms(tsa_data)

        if (!is.na(control_condition)) {
            if (!control_condition %in% condition_IDs(tsa_data)) {
                stop("control_condition is not found in tsa_data$condition_ID")
            }
            ctrl_avg <- TM_DF$Avg_Tm[TM_DF$condition_ID == control_condition]
        }
        TM_DF$delta_Tm <- TM_DF$Avg_Tm - ctrl_avg

        return(TM_DF)
    }

#' Analyze to Normalize
#'
#' The analyze_norm function allows users to process analysis through an UI
#'     interface. Function wraps together all functions with in TSA_analysis
#'     family and read_write_analysis family.
#'
#' @importFrom openxlsx read.xlsx
#' @import shiny
#' @importFrom rhandsontable rHandsontableOutput rhandsontable
#' @importFrom plotly ggplotly subplot layout plotlyOutput renderPlotly
#' @import ggplot2
#'
#' @export
#' @return shiny application
#'
#' @param raw_data The raw data for analysis.
#'
#' @seealso \code{\link[TSAR]{gam_analysis}}, \code{\link{read_tsar}},
#'   \code{\link{write_tsar}}, \code{\link{join_well_info}}
#'
#' @examples
#' if (interactive()) {
#'     data("qPCR_data1")
#'     shiny::runApp(analyze_norm(qPCR_data1))
#' }
#'
analyze_norm <- function(raw_data) {
    ui <- fluidPage(
        analyzepage(raw_data, deparse(substitute(raw_data)))
    )

    server <- function(input, output) {
        analysis <- reactiveVal(NULL)
        analyzed_done <- reactiveVal(FALSE)
        imported <- reactiveVal(FALSE)
        output_data <- reactiveVal(NULL)
        well_info <- reactiveVal(NULL)
        inFile <- reactiveVal(NULL)
        data <- reactive({
            req(input$excelFile)
            inFile(input$excelFile$datapath)
            imported(TRUE)
            if (!is.null(inFile())) {
                read.xlsx(inFile(), sheet = 1)
            }
        })

        render_data(input, output, raw_data)
        render_data_title(input, output, "Current Data: raw_data")
        preview_model(input, output, raw_data)
        perform_analysis(
            input, output, raw_data, analysis,
            output_data, analyzed_done
        )
        join_condition(
            input, output, analysis, output_data,
            imported, analyzed_done, well_info, inFile
        )
        write_file(input, output, analyzed_done, output_data)
        preview_output(
            input, output, analyzed_done, imported,
            output_data, analysis, inFile, well_info
        )
        manual_well(input, output)
        set_condition(
            input, output, imported, analyzed_done,
            inFile, well_info, analysis, output_data
        )
        preview_condition(input, output, well_info, data)
        hide(input, output)
        save_change(input, output, imported, well_info)
        save_analysis(input, output, output_data)
        stop_analyze(input, output)
    }
    app <- shinyApp(ui = ui, server = server)
}

#' Normalize Fluorescence
#'
#' normalize() reads in raw_data. This function normalizes data by standardizing
#'     them according to maximum and minimum fluorescence per well, with maximum
#'     set to 1 and minimum set to 0. It also reformats data types by checking
#'     for potential error. i.e. a string specifying 100,000 will be read in
#'     as number, 100000, without issue.
#'     Function is applicable only to data of a single well, do not call on
#'     an entire data frame of all 96 well data. It is intended for single well
#'     screening purposes.
#'
#' @importFrom magrittr %>%
#' @importFrom dplyr mutate select
#'
#' @param raw_data data frame; raw dataset input, should be of only one well.
#'     If multiple wells need to be normalized, use \code{\link{gam_analysis}}()
#'     for 96 well application. If only preliminary screening is needed, use
#'     \code{\link[TSAR]{screen}}().
#' @param fluo integer; the Fluorescence variable column id (e.g. fluo = 5
#'     when 5th column of the data frame is the Fluorescence value)
#'     if fluorescence variable is named exactly as "Fluorescence", fluo does
#'     not need to be specified. i.e. fluo is set to NA by default,
#'     suggesting the variable is named "Fluorescence".
#' @param selected list of character strings;
#'     variables from the original data set users intend to keep.
#'     Variable default set to c("Well.Position", "Temperature", "Fluorescence",
#'     "Normalized") if not otherwise specified. If data frame variables are
#'     named differently, user needs to specify what column variables to keep.
#' @return cleaned up data framed with selected columns
#' @examples
#' data("qPCR_data1")
#' test <- subset(qPCR_data1, Well.Position == "A01")
#' normalize(test)
#'
#' @family data_preprocess
#' @export
normalize <- function(
    raw_data,
    fluo = NA,
    selected = c(
        "Well.Position",
        "Temperature",
        "Fluorescence",
        "Normalized"
    )) {
    #format inputs and cast them into desired types
    norm_data <- raw_data %>%
        transform(
            Fluorescence =
                as.double(gsub(",", "", raw_data$Fluorescence))
        ) %>%
        transform(
            Temperature =
                as.double(gsub(",", "", raw_data$Temperature))
        ) %>%
        transform(Well.Position = paste(
            substr(raw_data$Well.Position, 1, 1),
            sprintf("%02s", substr(
                raw_data$Well.Position, 2,
                nchar(raw_data$Well.Position)
            )),
            sep = ""
        ))

    # normalize fluorescence by min max
    if (is.na(fluo)) {
        min_f <- min(norm_data$Fluorescence)
        max_f <- max(norm_data$Fluorescence)
        norm_data <- norm_data %>%
            mutate(
                Normalized =
                    (norm_data$Fluorescence - min_f) / (max_f - min_f)
            ) %>%
            dplyr::select(all_of(selected))
    } else {
        norm_data <- norm_data %>%
            mutate(
                Normalized =
                    (norm_data[[fluo]] - min(norm_data[[fluo]])) /
                        (max(norm_data[[fluo]]) - min(norm_data[[fluo]]))
            ) %>%
            dplyr::select(all_of(selected))
    }
}


#' Numerical Tm by treating Tm as inflection point
#'
#' Estimates the inflection point (Tm) as the (x, y) coordinate where the
#' discrete second derivative changes sign (zero-crossing). Among all
#' zero-crossings, chooses the one with the largest absolute first derivative
#' (steepest slope). Uses linear interpolation between adjacent points to refine
#' the zero of the second derivative.
#'
#' @param data data.frame with numeric columns x and y (specified by names).
#'   Data will be sorted by x; duplicated x's are averaged.
#' @param x,y character; names of the predictor and response columns.
#' @return Named numeric vector \code{c(x = x_Tm, y = y_Tm)}; \code{NA}s on failure.
#' @examples
#' # tm_xy <- numerical_Tm(df, x = "Temperature", y = "Normalized")
numerical_Tm <- function(data, x, y) {
  stopifnot(x %in% names(data), y %in% names(data))
  
  # keep finite values only
  df <- data[is.finite(data[[x]]) & is.finite(data[[y]]), c(x, y)]
  names(df) <- c("x", "y")
  if (nrow(df) < 3L) return(c(x = NA_real_, y = NA_real_))
  
  # sort by x and average duplicates to avoid zero spacing
  df <- df[order(df$x), , drop = FALSE]
  if (anyDuplicated(df$x)) {
    # aggregate duplicates by mean y
    df <- aggregate(y ~ x, data = df, FUN = mean)
  }
  
  n <- nrow(df)
  if (n < 3L) return(c(x = NA_real_, y = NA_real_))
  
  # spacing and finite differences
  dx <- diff(df$x)
  if (any(dx <= 0)) return(c(x = NA_real_, y = NA_real_))  # safety (sorted unique should prevent)
  
  # central first derivative (dy/dx) at interior points i=2..n-1
  dy_c <- (df$y[3:n] - df$y[1:(n - 2)]) / (df$x[3:n] - df$x[1:(n - 2)])
  # central second derivative at interior i=2..n-1:
  # d2 ~ ( (y_{i+1}-y_i)/h2 - (y_i - y_{i-1})/h1 ) / ((h1 + h2)/2)
  h1 <- df$x[2:(n - 1)] - df$x[1:(n - 2)]
  h2 <- df$x[3:n]       - df$x[2:(n - 1)]
  d1 <- (df$y[2:(n - 1)] - df$y[1:(n - 2)]) / h1
  d2 <- (df$y[3:n]       - df$y[2:(n - 1)]) / h2
  d2_c <- (d2 - d1) / ((h1 + h2) / 2)
  
  # indices for interior points
  ii <- 2:(n - 1)
  
  # zero-crossings of d2_c between consecutive interior indices
  sgn <- sign(d2_c)
  zc_idx <- which(sgn[-length(sgn)] * sgn[-1L] < 0)  # sign change between (i,i+1) in interior space
  if (!length(zc_idx)) {
    # fallback: steepest slope among interior central differences
    k <- which.max(abs(dy_c))
    # Tm at the interior point corresponding to k (index ii[k])
    i_star <- ii[k]
    return(c(x = df$x[i_star], y = df$y[i_star]))
  }
  
  # For each zero-crossing between interior indices (ii[zc] and ii[zc+1]),
  # linearly interpolate root of d2_c and evaluate y on the segment between raw x's.
  cand_x <- numeric(length(zc_idx))
  cand_y <- numeric(length(zc_idx))
  cand_slope <- numeric(length(zc_idx))
  
  for (j in seq_along(zc_idx)) {
    k <- zc_idx[j]               # in 1..length(d2_c)-1
    iL <- ii[k]                  # left raw index (in 2..n-2)
    iR <- ii[k + 1L]             # right raw index
    
    # interpolate zero of d2_c between k and k+1 in "interior grid":
    # d2_c[k] at x ~ df$x[iL]; d2_c[k+1] at x ~ df$x[iR]
    yL2 <- d2_c[k]; yR2 <- d2_c[k + 1L]
    xL  <- df$x[iL]; xR <- df$x[iR]
    if (!is.finite(yL2) || !is.finite(yR2) || yL2 == yR2) {
      x_star <- (xL + xR) / 2
    } else {
      x_star <- xL - yL2 * (xR - xL) / (yR2 - yL2)
    }
    x_star <- max(min(x_star, xR), xL)
    
    # interpolate y along the raw segment that contains x_star
    # choose the immediate raw segment that brackets x_star
    # here, use the segment [iL, iR] (they are adjacent or nearly adjacent)
    yL <- df$y[iL]; yR <- df$y[iR]
    y_star <- yL + (yR - yL) * (x_star - xL) / (xR - xL)
    
    # slope proxy: central first derivative magnitude near k
    slope_k <- abs(dy_c[k])
    
    cand_x[j] <- x_star
    cand_y[j] <- y_star
    cand_slope[j] <- slope_k
  }
  
  j_best <- which.max(cand_slope)
  c(x = cand_x[j_best], y = cand_y[j_best])
}

#' Beta-knot natural cubic spline model (Tm-centered via numerical_Tm)
#'
#' Fits a natural cubic spline to \code{y} as a function of \code{x} using
#' interior knots at Beta(a,a) quantiles, centered at the numerically estimated
#' Tm from \code{\link{numerical_Tm}}.
#'
#' @inheritParams numerical_Tm
#' @param norm_data data.frame with numeric columns x and y (normalized).
#' @param beta_shape numeric > 0; Beta(a,a) shape parameter. Default 4.
#' @param beta_knots_frac numeric in (0,1); fraction of unique x used for
#'   interior knots if \code{beta_n_knots} is NULL. Default 0.008.
#' @param beta_n_knots integer or NULL; overrides fraction when provided.
#' @param use_natural logical; if TRUE (default) uses \code{splines::ns()},
#'   else \code{splines::bs(..., degree = 3)}.
#' @return \code{"lm"} fit with spline basis; attributes store knots and settings.
#' @export
model_beta <- function(norm_data, x, y,
                       beta_shape = 4,
                       beta_knots_frac = 0.008,
                       beta_n_knots = NULL,
                       use_natural = TRUE) {
  # --- resolve names if user passed bare symbols
  .resolve_name <- function(arg) {
    if (is.character(arg) && length(arg) == 1) return(arg)
    deparse(substitute(arg))
  }
  x <- .resolve_name(x); y <- .resolve_name(y)
  
  xvec <- norm_data[[x]]
  xmin <- min(xvec, na.rm = TRUE)
  xmax <- max(xvec, na.rm = TRUE)
  if (!is.finite(xmin) || !is.finite(xmax) || xmax <= xmin) {
    stop("Invalid x range for model_beta().")
  }
  
  # --- center at numerical Tm (no GAM)
  tm_xy <- numerical_Tm(norm_data, x = x, y = y)
  tm_val <- tm_xy[["x"]]
  if (!is.finite(tm_val)) tm_val <- (xmin + xmax) / 2
  c_frac <- (tm_val - xmin) / (xmax - xmin)
  c_frac <- max(0, min(1, c_frac))
  s <- 2 * min(c_frac, 1 - c_frac)
  
  # --- number of interior knots
  if (is.null(beta_n_knots)) {
    S <- length(unique(stats::na.omit(xvec)))
    m <- max(1L, floor(beta_knots_frac * S))
  } else {
    m <- as.integer(beta_n_knots)
  }
  
  # --- build spline formula
  if (!is.finite(m) || m <= 0) {
    frm <- stats::as.formula(paste0(
      y, " ~ splines::ns(", x,
      ", df = 3, Boundary.knots = c(", xmin, ",", xmax, "))"
    ))
  } else {
    u <- seq_len(m) / (m + 1)
    q_beta <- stats::qbeta(u, shape1 = beta_shape, shape2 = beta_shape)
    q_star <- c_frac + (q_beta - 0.5) * s
    knots_x <- xmin + q_star * (xmax - xmin)
    
    if (isTRUE(use_natural)) {
      frm <- stats::as.formula(paste0(
        y, " ~ splines::ns(", x,
        ", knots = knots_x, Boundary.knots = c(", xmin, ",", xmax, "), intercept = FALSE)"
      ))
    } else {
      frm <- stats::as.formula(paste0(
        y, " ~ splines::bs(", x,
        ", knots = knots_x, degree = 3, Boundary.knots = c(", xmin, ",", xmax, "), intercept = FALSE)"
      ))
    }
  }
  
  fit <- stats::lm(frm, data = norm_data, x = TRUE, y = TRUE)
  
  # ---- augment: fitted and derivative on original rows
  # keep original order, but compute derivative on x-sorted rows
  df_aug <- norm_data
  # predict at original x
  df_aug$fitted <- stats::predict(fit, newdata = df_aug)
  
  # derivative from fitted vs x (central differences on sorted-by-x)
  ord <- order(df_aug[[x]])
  xo <- as.numeric(df_aug[[x]][ord])
  yo <- as.numeric(df_aug$fitted[ord])
  
  n <- length(xo)
  deriv <- rep(NA_real_, n)
  if (n >= 3) {
    # central difference: (y_{i+1}-y_{i-1})/(x_{i+1}-x_{i-1})
    deriv[2:(n - 1)] <- (yo[3:n] - yo[1:(n - 2)]) / (xo[3:n] - xo[1:(n - 2)])
    # optional: one-sided at ends
    deriv[1] <- (yo[2] - yo[1]) / (xo[2] - xo[1])
    deriv[n] <- (yo[n] - yo[n - 1]) / (xo[n] - xo[n - 1])
  }
  # put derivative back to original row order
  df_aug$norm_deriv <- NA_real_
  df_aug$norm_deriv[ord] <- deriv
  
  # If your view expects the response column named "Normalized" specifically,
  # ensure the column exists (duplicate y if named differently)
  if (y != "Normalized" && "Normalized" %in% names(df_aug) == FALSE) {
    df_aug$Normalized <- df_aug[[y]]
  }
  # Ensure Temperature column exists similarly
  if (x != "Temperature" && "Temperature" %in% names(df_aug) == FALSE) {
    df_aug$Temperature <- df_aug[[x]]
  }
  
  # attach some metadata if useful
  attr(df_aug, "beta_model") <- fit
  attr(df_aug, "beta_knots") <- if (exists("knots_x")) knots_x else numeric()
  attr(df_aug, "beta_shape") <- beta_shape
  attr(df_aug, "beta_center_frac") <- c_frac
  attr(df_aug, "Boundary.knots") <- c(xmin, xmax)
  
  # return augmented data frame so view_model() can consume it
  df_aug
}




#' Generalized Addidtive Modeling on TSA data
#'
#' Function finds fitted fluorescence values by imposing generalized
#'     additive model on fluorescence data by temperature. Model assumes
#'     method = "GACV.Cp" and sets to \code{formula = y ~ s(x, bs = "ad")}.
#'     Function inherits function from gam package, \code{\link{gam}}().
#'
#'
#' @importFrom magrittr %>%
#' @importFrom mgcv gam
#' @importFrom dplyr mutate select
#'
#' @param norm_data data frame input of only one well's reading, preferably
#'   normalized using \code{\link{normalize}}.
#' @param x temperature column
#' @param y normalized fluorescence column
#' @return data frame containing gam model fitted values
#'
#' @family data_preprocess
#' @examples
#' data("qPCR_data1")
#' test <- subset(qPCR_data1, Well.Position == "A01")
#' test <- normalize(test, fluo = 5, selected = c(
#'     "Well.Position", "Temperature",
#'     "Fluorescence", "Normalized"
#' ))
#' model_gam(test, x = test$Temperature, y = test$Normalized)
#'
#' @export
model_gam <- function(norm_data, x, y) {
    #model data with gam
    mgcv::gam(
        formula = y ~ s(x, bs = "ad"),
        data = norm_data,
        method = "GACV.Cp"
    )
}

#' Boltzmann Modeling on TSA data
#'
#' Function finds fitted fluorescence values by imposing Boltzmann function.
#'
#'
#' @importFrom minpack.lm nlsLM
#'
#' @param norm_data data frame input, preferably normalized using
#'     \code{\link{normalize}}.
#' @return dtaa frame containing gam model fitted values
#'
#' @family data_preprocess
#' @examples
#' data("qPCR_data1")
#' A01 <- subset(qPCR_data1, Well.Position == "A01")
#' A01 <- normalize(A01)
#' model_boltzmann(A01)
#'
#' @export
model_boltzmann <- function(norm_data) {
    #check for pre-written min max
    if (is.null(norm_data$minB) && is.null(norm_data$maxB)) {
        maxfluo <- 0
        minfluo <- 100
        temp <- norm_data
        trimmed <- norm_data
        stock <- norm_data
        while (maxfluo <= minfluo) {
            maxfluo <- temp$Temperature[which.max(temp$Fluorescence)]
            temp <- subset(norm_data, Temperature <= maxfluo)
            minfluo <- temp$Temperature[which.min(temp$Fluorescence)]
            temp <- subset(trimmed, Temperature >=
                min(trimmed$Temperature) + 10 &
                Temperature <= max(trimmed$Temperature) - 10)
            trimmed <- temp
        }
        norm_data$minB <- minfluo
        norm_data$maxB <- maxfluo + 5
        norm_data <- subset(norm_data, Temperature >= minfluo &
            Temperature <= maxfluo + 5)
    } else {
        norm_data <- subset(norm_data, Temperature >= norm_data$minB[1] &
            Temperature <= norm_data$maxB[1] + 5)
    }
    #normalize
    norm_data <- TSAR::normalize(norm_data)
    #fit boltzmann equation
    f <- minpack.lm::nlsLM(y ~ 1 / (1 + exp(-k * (x - x2))),
        data = data.frame(
            x = norm_data$Temperature,
            y = norm_data$Normalized
        ),
        start = list(k = 100, x2 = maxfluo)
    )
    #pack fitted numbers and summaries into one variable
    fitted <- unlist(f$m$fitted())
    fitted <- append(
        rep(0, which(stock$Temperature ==
            min(norm_data$Temperature))),
        fitted
    )
    fitted <- append(fitted, rep(1, (length(stock$Temperature) -
        length(fitted))))
    f$fitted.values <- fitted
    return(f)
}

#' Run Boltzmann Modeling
#'
#' Function runs function \code{model_boltzmann()} and raises warning when
#'     modeling generates error or warnings.
#'
#' @param norm_data data frame input, preferably normalized using
#'     \code{\link{normalize}}.
#' @return data frame containing gam model fitted values
#'
#' @family data_preprocess
#' @examples
#' data("qPCR_data1")
#' A01 <- subset(qPCR_data1, Well.Position == "A01")
#' A01 <- normalize(A01)
#' run_boltzmann(A01)
#'
#' @export
run_boltzmann <- function(norm_data) {
    tryCatch(
        {
            model <- model_boltzmann(norm_data)
            return(model)
        },
        warning = function(w) {
            condition <- conditionMessage(w)
            warning(
                "Warning: Error caught while imposing boltzmann fit on well ",
                unique(norm_data$Well.Position), "Derivative Model is applied
            instead. To override, specify a min and max of modeling manually. ",
                condition, "\n"
            )
            model <- model_gam(norm_data,
                x = norm_data$Temperature,
                y = norm_data$Normalized
            )
            return(model)
        },
        error = function(e) {
            condition <- conditionMessage(e)
            warning(
                "Warning: Error caught while imposing boltzmann fit on well ",
                unique(norm_data$Well.Position), "Derivative Model is applied
            instead. To override, specify a min and max of modeling manually. ",
                condition, "\n"
            )
            model <- model_gam(norm_data,
                x = norm_data$Temperature,
                y = norm_data$Normalized
            )
            return(model)
        }
    )
}

#' Refit and calculate derivative function
#'
#' Model_fit calculates derivatives by refitting model onto data. Only runs
#'     on data of a single well.
#'
#' @importFrom magrittr %>%
#' @importFrom dplyr mutate select
#'
#' @param norm_data data frame; the raw data set input
#' @param model fitted model containing fitted values
#' @param smoothed inform whether data already contains a smoothed model; Input
#'     the column name of the smoothed data to override values of gam model
#'     fitting. For example, existing "Fluorescence" column contains data
#'     already smoothed, set \code{smoothed = "Flourescence"} to calculate
#'     derivative function upon the called smoothed data.
#' @return data frame; with calculated derivative columns
#'
#' @family data_preprocess
#'
#' @examples
#' data("qPCR_data1")
#' test <- subset(qPCR_data1, Well.Position == "A01")
#' test <- normalize(test, fluo = 5, selected = c(
#'     "Well.Position", "Temperature",
#'     "Fluorescence", "Normalized"
#' ))
#' gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
#' model_fit(test, model = gammodel)
#' # if data come smoothed, run ...
#' model_fit(test, smoothed = "Fluorescence")
#'
#' @export
#'
model_fit <- function(norm_data, model, smoothed) {
    if (missing(smoothed)) {
        norm_data <- norm_data %>%
            # pull in the fitted values into data frame
            mutate(fitted = model$fitted.values)
        # calculate derivative using fitted values
        norm_data <- mutate(norm_data,
            norm_deriv =
                c(diff(norm_data$fitted) /
                    diff(norm_data$Temperature), NA)
        )
    } else {
        norm_data <- mutate(norm_data,
            norm_deriv =
                c(diff(norm_data[, smoothed]) /
                    diff(norm_data$Temperature), NA)
        )
    }
}

mainpage <- function(raw_data, data_name) {
    fluidPage(
        useShinyjs(),
        tags$style(
            HTML(
                "
              .sticky-panel {
                position: sticky;
                top: 0;
                z-index: 100;
                background-color: white;
              }
              "
            )
        ),
        fluidRow(
            column(
                width = 4,
                shiny::checkboxInput("dialogueToggle",
                    "Hide All Hints and Messages",
                    value = FALSE
                )
            ),
            column(
                width = 4,
                shiny::actionButton(
                    "clear",
                    "Clear all Selected"
                )
            )
        ),
        fluidRow(
            column(width = 4, h3("Current Raw Dataset: ")),
            h3(data_name)
        ),
        div(
            class = "sticky-panel",
            plotly::plotlyOutput("distPlot")
        ),
        verbatimTextOutput("info"),
        fluidRow(
            column(width = 3, h4("Select by Grid")),
            column(
                width = 2,
                h5(" "),
                shinyWidgets::materialSwitch("toggle_button", "Show Grid",
                    value = TRUE, right = TRUE
                )
            )
        ),
        div(
            id = "grid-panel",
            style = "display: none;",
            fluidRow(
                column(
                    width = 12,
                    div(
                        id = "grid-container",
                        style = "display: grid; grid-template-columns:
                                repeat(12, 50px); grid-gap: 0px;",
                        lapply(seq_len(96), function(i) {
                            row <- ceiling(i / 12)
                            col <- i %% 12
                            if (col == 0) col <- 12
                            col_label <- sprintf("%02d", col)
                            div(
                                id = paste0("cell-", i),
                                class = "grid-cell",
                                style = "background-color: white;
                                       height: 30px; cursor: pointer;
                                       display: flex; align-items:
                                       center; justify-content: center;
                                       font-size: 12px;",
                                paste0(LETTERS[row], col_label)
                            )
                        })
                    )
                )
            )
        ),
        verbatimTextOutput("highlighted_list"),
        h4("Copy Edited"),
        fluidRow(
            column(
                width = 3,
                shiny::actionButton("myButton", "Copy Selected Wells")
            ),
            column(
                width = 4,
                shiny::actionButton(
                    "removeCall",
                    "Copy in remove_raw() Call"
                )
            ),
            column(
                width = 3,
                actionButton(
                    "savetoR",
                    "Save changes to R"
                )
            ),
        ),
        br(),
        fluidRow(
            column(
                width = 6,
                h4("Refresh Plot"),
                fluidRow(
                    column(
                        width = 4,
                        shiny::actionButton(
                            "viewRemovedButton",
                            "View Selected"
                        )
                    ),
                    column(
                        width = 2,
                        shiny::actionButton(
                            "refreshButton",
                            "Refresh Screening"
                        )
                    )
                )
            ),
            column(
                width = 5,
                h4("Edit Data"),
                fluidRow(
                    column(
                        width = 2,
                        shiny::actionButton(
                            "removeButton",
                            "Remove Selected"
                        )
                    )
                )
            )
        ),
        br(),
        actionButton("stopButton", "Close Window"),
        div(
            id = "hidden",
            shiny::checkboxGroupInput("extractedname",
                label = "name",
                choices = list(data_name),
                selected = data_name
            )
        )
    )
}

#' Average TSA Curves
#'
#' This function will take either Fluorescence or Normalized Fluorescence
#'     curves from the submitted data frame and find the average (mean) and
#'     standard deviation (sd) for each temperature measured in the TSA curve.
#'     Mean and sd are smoothened by default to generate cleaner curves.
#'     The function \code{\link[mgcv]{gam}} from the mgcv package is used for
#'     regression to smoothen lines. Smoothing can be turned off and the true
#'     average for each point can be given, however, plots will look messier.
#'     The qPCR machine may return temperatures with many decimal places,
#'     and TSAR only merges identical values, therefore rounding is necessary.
#'     Data is rounded to one decimal place to improve regression smoothing.
#'     \cr \cr
#'     \strong{Note:} All submitted data is averaged, regardless of condition or
#'     well ID.
#'     If you wish to average by condition, you will need to sort the data frame
#'     and run this function on subsets.
#'
#' @importFrom magrittr %>%
#' @importFrom dplyr summarize group_by n_distinct
#'
#' @param tsa_data a data frame that is merged and generated by
#'     TSAR::merge_TSA(). If \code{y = 'RFU'}, tsa_data must also be
#'     generated by TSAR::normalize_fluorescence. The Temperature column will be
#'     rounded and the average & sd of each rounded temperature is calculated.
#' @param y character string; c('Fluorescence', 'RFU').
#'     When \code{y = 'Fluorescence'}, the original Fluorescence data from
#'     TSAR::read_raw_data() is averaged. When \code{y = 'RFU'}, the average is
#'     calculated by the rescaled fluorescence.
#' @param digits an integer; \code{digits = 1} by default.
#'     The number of decimal places to round temperature to for averaging.
#' @param avg_smooth,sd_smooth logical; TRUE by default. Decides if the
#'    average (avg_smooth) or standard deviation (sd_smooth) will be smoothened
#'    by regression via mgcv::gam()
#' @param smoother character; one of \code{c("gam","beta","none")}.
#'     Passed to \code{\link{TSA_average}} to select the aggregate smoother:
#'     \code{"gam"} uses \pkg{mgcv}, \code{"beta"} uses a natural cubic spline
#'     with Beta(a,a) interior knots centered at Tm, and \code{"none"} uses the
#'     unsmoothed average. Default follows \code{TSA_average}.
#' @param beta_shape numeric; shape parameter \eqn{a} for the Beta(a,a) knot
#'     placement when \code{smoother = "beta"}. \code{beta_shape = 3} by default.
#' @param beta_n_knots integer or \code{NULL}; number of interior knots when
#'     \code{smoother = "beta"}. If \code{NULL}, uses \code{beta_knots_frac}.
#' @param beta_knots_frac numeric in (0,1); fraction of unique temperatures used
#'     as interior knots when \code{smoother = "beta"} and \code{beta_n_knots}
#'     is \code{NULL}. \code{beta_knots_frac = 0.008} by default.
#' @param use_natural logical; if TRUE (default) uses natural cubic spline
#'     basis for the beta method.
#'
#' @return a data frame of each temperature measured with the average, sd, and
#'    n(# of averaged values) calculated. Depending on avg_smooth and sd_smooth,
#'    the smoothened lines for the maximum and mimimum sd and the average will
#'    also be returned.
#' @family TSAR Formatting
#' @seealso \code{\link{merge_TSA}} and \code{\link{merge_TSA}} for
#'    preparing data.
#'
#' @examples
#' data("example_tsar_data")
#' TSA_average(example_tsar_data,
#'     y = "Fluorescence", digits = 1,
#'     avg_smooth = TRUE, sd_smooth = TRUE
#' )
#'
#' @export


TSA_average <- function(
    tsa_data,
    y = "Fluorescence",
    digits = 1,
    avg_smooth = TRUE,
    sd_smooth  = TRUE,
    smoother   = c("gam", "beta", "none"),
    # beta-spline options (used when smoother == "beta")
    beta_shape      = 4,        # 'a' in Beta(a, a); larger -> more central concentration
    beta_n_knots    = NULL,     # integer; if NULL use fraction below
    beta_knots_frac = 0.008,     # fraction of N (unique x) for interior knots
    use_natural     = TRUE      # TRUE: natural cubic (splines::ns); FALSE: splines::bs
) {
  smoother <- match.arg(smoother)
  y <- match.arg(y, choices = c("Fluorescence", "RFU"))
  
  tsa_data_new <- tsa_data
  tsa_data_new$Temperature <- round(tsa_data_new$Temperature, digits = digits)
  
  # summarize by temperature
  if (y == "Fluorescence") {
    tsa_data_new <- dplyr::as_tibble(tsa_data_new) %>%
      dplyr::group_by(Temperature) %>%
      dplyr::summarize(
        average = mean(Fluorescence, na.rm = TRUE),
        n       = dplyr::n(),
        sd      = stats::sd(Fluorescence, na.rm = TRUE),
        .groups = "drop"
      )
  }
  if (y == "RFU") {
    # use normalized fluorescence column 'RFU' created by normalize_fluorescence()
    if (!"RFU" %in% names(tsa_data_new)) {
      stop("Expected column 'RFU' not found. Run normalize_fluorescence() first or set y='Fluorescence'.")
    }
    tsa_data_new <- dplyr::as_tibble(tsa_data_new) %>%
      dplyr::group_by(Temperature) %>%
      dplyr::summarize(
        average = mean(RFU, na.rm = TRUE),
        n       = dplyr::n(),
        sd      = stats::sd(RFU, na.rm = TRUE),
        .groups = "drop"
      )
  }
  
  # handle NA sd (e.g., n==1 at a given temperature)
  tsa_data_new$sd[is.na(tsa_data_new$sd)] <- 0
  tsa_data_new$sd_max <- tsa_data_new$average + tsa_data_new$sd
  tsa_data_new$sd_min <- tsa_data_new$average - tsa_data_new$sd
  
  # ---- helper: beta-knot fit centered at Tm ----
  .beta_fit_predict_centered <- function(x, y,
                                         shape      = beta_shape,
                                         n_knots    = beta_n_knots,
                                         knots_frac = beta_knots_frac,
                                         natural    = use_natural,
                                         center_x   = NULL) {
    o <- order(x); x_sorted <- x[o]; y_sorted <- y[o]
    N <- length(x_sorted)
    if (N < 6L) return(y)  # not enough points; return as-is in original order
    
    xmin <- min(x_sorted); xmax <- max(x_sorted)
    span <- xmax - xmin
    if (!is.finite(span) || span <= 0) return(y)
    
    # number of interior knots
    m <- if (!is.null(n_knots)) as.integer(n_knots) else max(1L, floor(knots_frac * N))
    m <- max(1L, min(m, max(1L, N - 5L)))
    
    # choose center: from Tm column if available; otherwise midpoint
    cx <- if (is.null(center_x) || !is.finite(center_x)) (xmin + xmax) / 2 else center_x
    cx <- max(xmin, min(xmax, cx))      # clip to domain
    c_frac <- if (span > 0) (cx - xmin) / span else 0.5  # in [0,1]
    
    # Beta(a,a) interior quantiles on (0,1), then re-center around c_frac
    u        <- seq_len(m) / (m + 1)
    q_center <- stats::qbeta(u, shape1 = shape, shape2 = shape)  # around 0.5
    # shift & scale so center is at c_frac and knots stay inside [0,1]
    s <- 2 * min(c_frac, 1 - c_frac)        # ≤ 1
    if (!is.finite(s) || s <= 0) s <- 1e-6  # if center at boundary, avoid collapse
    q_shifted <- c_frac + (q_center - 0.5) * s
    q_shifted <- pmin(1, pmax(0, q_shifted))
    knots_x <- xmin + q_shifted * span
    
    B <- if (isTRUE(natural)) {
      splines::ns(x_sorted, knots = knots_x, Boundary.knots = c(xmin, xmax), intercept = TRUE)
    } else {
      splines::bs(x_sorted, degree = 3, knots = knots_x, Boundary.knots = c(xmin, xmax), intercept = TRUE)
    }
    fit <- stats::lm(y_sorted ~ 0 + B)
    yhat_sorted <- as.numeric(B %*% stats::coef(fit))
    
    yhat <- numeric(N); yhat[o] <- yhat_sorted
    yhat
  }
  
  # ---- smoother dispatcher ----
  .smooth_vec <- function(x, y, method, center_x_for_beta = NULL) {
    if (!isTRUE(all(is.finite(x)))) return(y)
    if (method == "none") {
      return(y)
    } else if (method == "gam") {
      df <- data.frame(x = x, y = y)
      m  <- mgcv::gam(y ~ s(x), data = df)
      return(as.numeric(stats::predict(m, newdata = data.frame(x = x))))
    } else if (method == "beta") {
      return(.beta_fit_predict_centered(x, y, center_x = center_x_for_beta))
    } else {
      return(y)
    }
  }
  
  x <- tsa_data_new$Temperature
  
  # Determine Tm center from original data (single value for this subset)
  tm_center <- NA_real_
  if ("Tm" %in% names(tsa_data)) {
    # use a robust single-center value
    tm_center <- suppressWarnings(stats::median(tsa_data$Tm, na.rm = TRUE))
    if (!is.finite(tm_center)) tm_center <- NA_real_
  }
  
  if (isTRUE(avg_smooth)) {
    tsa_data_new$avg_smooth <- .smooth_vec(x, tsa_data_new$average, smoother,
                                           center_x_for_beta = tm_center)
  }
  if (isTRUE(sd_smooth)) {
    tsa_data_new$sd_min_smooth <- .smooth_vec(x, tsa_data_new$sd_min, smoother,
                                              center_x_for_beta = tm_center)
    tsa_data_new$sd_max_smooth <- .smooth_vec(x, tsa_data_new$sd_max, smoother,
                                              center_x_for_beta = tm_center)
  }
  
  return(tsa_data_new)
}
#' Diagnose Smoother Accuracy Across Conditions
#'
#' Computes per-condition fitting error for \code{gam} and \code{beta} smoothers
#' (as implemented in \code{\link{TSA_average}}) against the aggregated mean
#' curve. For each \code{condition_ID}, the function compares the smoothed mean
#' (\code{avg_smooth}) to the real aggregated mean (\code{average}) at
#' each temperature and reports the error in corresponding metric.
#'
#' By default, it evaluates:
#' \itemize{
#'   \item \strong{GAM}: \code{smoother = "gam"}
#'   \item \strong{Beta (single spec)}: \code{beta_shape = 4}, \code{beta_knots_frac = 0.008}
#' }
#' If you supply vectors to \code{beta_shapes} and/or \code{beta_fracs}, the
#' function evaluates \emph{all} combinations for the beta method.
#'
#' @inheritParams TSA_average
#' @param metric character; one of \code{c("L2","L1","R2")} with default
#'   \code{"L2"}. Defines the accuracy metric between \code{average} (truth) and
#'   \code{avg_smooth} (fit):
#'   \itemize{
#'     \item \code{"L1"}: mean absolute error (MAE)
#'     \item \code{"L2"}: root mean squared error (RMSE)
#'     \item \code{"R2"}: coefficient of determination; higher is better
#'   }
#' @param beta_shapes numeric vector or \code{NULL}; shapes \eqn{a} for the
#'   Beta(a,a) knot placement when evaluating the \code{"beta"} smoother.
#'   \strong{Default behavior:} if \code{NULL} or length 0, uses a single value
#'   \code{4}.
#' @param beta_fracs numeric vector or \code{NULL}; fractions in \eqn{(0,1)}
#'   giving the proportion of unique temperatures used as interior knots for the
#'   \code{"beta"} smoother. \strong{Default behavior:} if \code{NULL} or length
#'   0, uses a single value \code{0.008}.
#' @param beta_n_knots integer or \code{NULL}; number of interior knots for the
#'   \code{"beta"} smoother. If \code{NULL} (default), the number of knots is
#'   determined by \code{beta_knots_frac}. Passed through to
#'   \code{\link{TSA_average}}.
#' @param digits integer; rounding applied to the Temperature grid before
#'   aggregation. Default \code{1}. Passed to \code{\link{TSA_average}}.
#' @param use_natural logical; if \code{TRUE} (default) uses natural cubic spline
#'   basis for the beta method. Passed to \code{\link{TSA_average}}.
#'
#' 
#' @return (Invisibly) a \code{data.frame} with columns:
#' \itemize{
#'   \item \code{condition_id}: condition label,
#'   \item \code{method}: \code{"gam"} or \code{beta(a=..., frac=...)} string,
#'   \item \code{error}: numeric value of the requested metric.
#' }
#' The function also prints a short textual summary to the console.
#'
#' @examples
#' \dontrun{
#' normalized_tsa_df
#'
#' result <- TSA_smoother_diagnostics(normalized_tsa_df, y = "RFU", metric = "L2")
#' as.data.frame(result)
#' }
#' @family Diagnostics
#' @seealso \code{\link{TSA_average}} (smoother implementation),
#'   \code{\link{TSA_wells_plot}} (visualization of per-well and averaged curves).

TSA_smoother_diagnostics <- function(
    tsa_data,
    y = c("Fluorescence", "RFU"),
    metric = c("L2", "L1", "R2"),
    # beta grids to test; if NULL or length 0, default to a=4 and frac=0.008
    beta_shapes = NULL,
    beta_fracs  = NULL,
    # pass-through options if you need them
    digits = 1,
    use_natural = TRUE,
    beta_n_knots = NULL
) {
  y <- match.arg(y)
  metric <- match.arg(metric)
  
  # --- safety checks
  if (!all(c("Temperature", "condition_ID") %in% names(tsa_data))) {
    stop("tsa_data must contain columns: Temperature and condition_ID (as in previous functions).")
  }
  
  # conditions helper (use package helper if available)
  get_conditions <- function(df) {
    if (exists("condition_IDs", mode = "function")) {
      condition_IDs(df)
    } else {
      unique(df$condition_ID)
    }
  }
  
  # defaults for beta grid
  if (is.null(beta_shapes) || length(beta_shapes) == 0) beta_shapes <- 4
  if (is.null(beta_fracs)  || length(beta_fracs)  == 0) beta_fracs  <- 0.008
  
  # internal: compute error between avg_smooth and average for a single condition
  .fit_and_error <- function(df_cond, smoother, beta_shape = NA_real_, beta_frac = NA_real_) {
    # run TSA_average to get smoothed mean; we smooth the average curve only
    av <- TSA_average(
      tsa_data    = df_cond,
      y           = y,
      digits      = digits,
      avg_smooth  = TRUE,
      sd_smooth   = FALSE,
      smoother    = smoother,
      beta_shape      = if (is.na(beta_shape)) 3 else beta_shape,
      beta_n_knots    = beta_n_knots,
      beta_knots_frac = if (is.na(beta_frac)) 0.12 else beta_frac,
      use_natural     = use_natural
    )
    
    # y_true = aggregated mean; y_hat = smoothed mean (or copy if smoother="none")
    y_true <- av$average
    y_hat  <- if ("avg_smooth" %in% names(av)) av$avg_smooth else av$average
    
    # remove any NA pairs to avoid trouble
    ok <- is.finite(y_true) & is.finite(y_hat)
    y_true <- y_true[ok]; y_hat <- y_hat[ok]
    
    if (length(y_true) < 2L) return(NA_real_)
    
    if (metric == "L1") {
      # Mean Absolute Error
      mean(abs(y_true - y_hat))
    } else if (metric == "L2") {
      # Root Mean Squared Error
      sqrt(mean((y_true - y_hat)^2))
    } else { # "R2"
      ss_res <- sum((y_true - y_hat)^2)
      ss_tot <- sum((y_true - mean(y_true))^2)
      if (ss_tot == 0) return(NA_real_)
      1 - ss_res / ss_tot
    }
  }
  
  conds <- get_conditions(tsa_data)
  res_list <- vector("list", length = 0L)
  
  # --- iterate conditions
  for (cond in conds) {
    df_cond <- tsa_data[tsa_data$condition_ID == cond, , drop = FALSE]
    
    # 1) GAM
    err_gam <- .fit_and_error(df_cond, smoother = "gam")
    res_list[[length(res_list) + 1L]] <- data.frame(
      condition_ID = cond,
      method = "gam",
      error = err_gam,
      stringsAsFactors = FALSE
    )
    
    # 2) BETA grid
    for (a in beta_shapes) {
      for (f in beta_fracs) {
        err_beta <- .fit_and_error(df_cond, smoother = "beta", beta_shape = a, beta_frac = f)
        res_list[[length(res_list) + 1L]] <- data.frame(
          condition_ID = cond,
          method = sprintf("beta(a=%s, frac=%s)", as.character(a), as.character(f)),
          error = err_beta,
          stringsAsFactors = FALSE
        )
      }
    }
  }
  
  results <- do.call(rbind, res_list)
  
  # --- summaries
  # mean by method label
  mean_by_method <- stats::aggregate(error ~ method, data = results, FUN = function(x) mean(x, na.rm = TRUE))
  # separate mean for GAM vs each beta spec
  mean_gam <- mean_by_method[mean_by_method$method == "gam", "error"]
  mean_beta_rows <- mean_by_method[grepl("^beta\\(", mean_by_method$method), , drop = FALSE]
  
  # print required summaries
  if (metric %in% c("L1", "L2")) {
    cat(sprintf("mean %s error of gam method: %s\n",
                metric, ifelse(length(mean_gam), signif(mean_gam, 4), "NA")))
    if (nrow(mean_beta_rows)) {
      for (i in seq_len(nrow(mean_beta_rows))) {
        cat(sprintf("mean %s error of %s: %s\n",
                    metric, mean_beta_rows$method[i], signif(mean_beta_rows$error[i], 4)))
      }
    }
    # pick best (lowest)
    best_idx <- which.min(mean_by_method$error)
    if (length(best_idx)) {
      cat("\n", mean_by_method$method[best_idx], " has lowest error and highest accuracy.\n", sep = "")
    }
  } else { # R2
    cat(sprintf("mean %s of gam method: %s\n",
                metric, ifelse(length(mean_gam), signif(mean_gam, 4), "NA")))
    if (nrow(mean_beta_rows)) {
      for (i in seq_len(nrow(mean_beta_rows))) {
        cat(sprintf("mean %s of %s: %s\n",
                    metric, mean_beta_rows$method[i], signif(mean_beta_rows$error[i], 4)))
      }
    }
    # pick best (highest R2)
    best_idx <- which.max(mean_by_method$error)
    if (length(best_idx)) {
      cat("\n", mean_by_method$method[best_idx], "has higher mean R^2 and thus higher accuracy.\n", sep = "")
    }
  }
  
  # return the full table (condition, method, error)
  # column names as requested: (1) condition id (2) method name and param (3) corresponding error
  names(results) <- c("condition_id", "method", "error")
  return(invisible(results))
}

#' Graph tsar_data
#'
#' Function allows users to graph out tsar_data, building boxplot, compare
#'     plots, and curves by condition. Input of data as parameter is optional.
#'     graph_tsar wraps together all graphing functions and relative helper
#'     functions.
#'
#' @import shiny
#' @import ggplot2
#' @importFrom plotly ggplotly subplot layout plotlyOutput renderPlotly
#' @importFrom shinyjs toggle hidden
#' @importFrom ggpubr ggarrange
#' @importFrom shinyWidgets actionBttn
#'
#' @export
#'
#' @param tsar_data tsar data outputted by merge_norm or merge_tsa.
#'     Parameter is optional. If no data is passed, access the merge panel
#'     to merge norm_data into tsar_data.
#'
#' @return prompts separate app window for user interaction,
#'     does not return specific value; generates boxplot and compare plots
#'     according to user input
#'
#' @family TSA Plots
#'
#' @seealso \code{\link{TSA_boxplot}}, \code{\link{TSA_compare_plot}},
#'     \code{\link{condition_IDs}}, \code{\link{well_IDs}},
#'     \code{\link{merge_norm}}, \code{\link{TSA_Tms}},
#'     \code{\link{Tm_difference}}
#'
#' @examples
#' if (interactive()) {
#'     data("example_tsar_data")
#'     shiny::runApp(graph_tsar(example_tsar_data))
#' }
#'
graph_tsar <- function(tsar_data = data.frame()) {
    ui <- graphpage(tsar_data)

    server <- function(input, output, session) {
        datelist <- reactiveVal(NULL)
        options(shiny.maxRequestSize = 30 * 1024^2)
        dated <- reactiveVal(FALSE)
        compare <- reactiveVal(NULL)
        graph_tsar_data <- reactiveVal(tsar_data)
        stock_tsar_data <- reactiveVal(tsar_data)

        dummy_plot(input, output)
        hide_p(input, output)
        toggle_p(input, output)
        initialize(input, output, graph_tsar_data)
        build_dates(input, output)
        save_dates(input, output, dated, datelist)
        saved_merged(input, output, graph_tsar_data)
        merge_update(input, output, dated, graph_tsar_data, datelist, session)
        build_boxplot(input, output, graph_tsar_data)
        build_compare(input, output, graph_tsar_data, compare)
        view_compare(input, output, compare)
        build_curves(input, output, graph_tsar_data)
        build_derivatives(input, output, graph_tsar_data)
        render_condition(input, output, graph_tsar_data)
        render_well(input, output, graph_tsar_data)
        render_tm(input, output, graph_tsar_data)
        render_message(input, output)
        remove_selected_graph(input, output, graph_tsar_data)
        restore_removal(input, output, graph_tsar_data, stock_tsar_data)
        closegraph(input, output)
    }

    app <- shinyApp(ui = ui, server = server)
}

#' Example tsar_data file
#'
#' Dataset Description:
#'     This is an example dataset of the tsar_data strucutre. The data
#'     frame contains well ID, conditions, and experimental details.
#'
#' @name example_tsar_data
#'
#' @format A data frame with the following columns:
#'     \describe{
#'       \item{Well}{Well position}
#'       \item{Temperature}{Temperature in degrees}
#'       \item{Fluorescence}{Fluorescence reading}
#'       \item{Normalized}{Normalized value}
#'       \item{norm_deriv}{Calculated first derivative}
#'       \item{Tm}{Tm value}
#'       \item{Protein}{Protein information}
#'       \item{Ligand}{Ligand information}
#'       \item{ExperimentFileName}{Experiment file name}
#'       \item{well_ID}{Well ID}
#'       \item{condition_ID}{Condition ID}
#'     }
#'
#' @usage data(example_tsar_data)
#' @return example tsar_data in data frame
#' @keywords dataset
#' @source experimentally obtained
#' @keywords dataset
data(example_tsar_data, envir = environment())

---
title: "TSAR Workflow by Command"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{TSAR Workflow by Command}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r, echo=FALSE, fig.width=4, out.width="400px"}
knitr::include_graphics("images/TSAR_logo.png")
```
Pipeline analysis from raw data reading to graphic visualization

## 1. Introduction
TSAR Package provides simple solution to qPCR data processing, computing 
thermal shift analysis given either raw fluorescent data or smoothed curves. 
The functions provide users with the protocol to conduct preliminary data checks
and also expansive analysis on large scale of data. Furthermore, it showcases 
simple graphic presentation of analysis, generating clear box plot and line 
graphs given input of desired designs. Overall, TSAR Package offers a workflow
easy to manage and visualize.

## 2. Installation
Load Package and other relevant packages. Usage of dplyr and ggplot2 
along with TSAR package is recommended for enhanced analysis.

```{r, include = FALSE}
knitr::opts_chunk$set(warning = FALSE, message = FALSE, comment = "#>")
```

Use commands below to install TSAR package:
library(BiocManager)
BiocManager::install("TSAR")

```{r setup, message=FALSE, warning=FALSE}
library(TSAR)
library(dplyr)
library(ggplot2)
```

## 3. Load Data

Read data in .txt or .csv format. Use read.delim function to input tab
delimited file; use read.csv to input comma separated files. Other
formats of input are welcomed as long as data is stored in data frame
structure as numeric type (calculation-required data) and characters 
(non-calculation-required data). Ensure excessive lines are removed (e.g.
skip = , nrows = ). Means to check these are View(), pre-opening data
file in excel, or manually removing all excessive data before input
reading. Package defaults variable names as "Well.Position",
"Temperature", "Fluorescence", "Normalized". Consider renaming data
frame before proceding to following step.

```{r}
data("qPCR_data1")
raw_data <- qPCR_data1
```

## 4. Data Pre-Processing

Select data of individual cell for pre-analysis screening. e.g. select
well A1

```{r}
test <- raw_data %>% filter(Well.Position == "A01")
```

Run example analysis on one well to screen potential errors and
enhancement of model.

```{r}
# normalize fluorescence reading into scale between 0 and 1
test <- normalize(test, fluo = 5, selected = c(
    "Well.Position", "Temperature",
    "Fluorescence", "Normalized"
))
head(test)
gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
gammaresult <- model_fit(test, model = gammodel)
```

Output analysis result using `view_model()` to view normalized data and fitted
model. Determine if any noise need to be removed (i.e. subsetting by
temperature range). Determine which model is the best (i.e. is currrent
data already smoothed, does fitted model suit well.) Determine if
Tm-estimation is proper. \*current model assumes derivative estimation
of Tm value.

```{r}
Tm_est(gammaresult)
view <- view_model(gammaresult)
view[[1]] + theme(aspect.ratio = 0.7, legend.position = "bottom")
view[[2]] + theme(aspect.ratio = 0.7, legend.position = "bottom")
```
Alternatively, user can also use `model_beta()` to fit a beta model to the data.

```{r}
betamodel <- model_beta(test, x ="Temperature", y = "Normalized")
view <- view_model(betamodel)
view[[1]] + theme(aspect.ratio = 0.7, legend.position = "bottom")
view[[2]] + theme(aspect.ratio = 0.7, legend.position = "bottom")
```



Screen all wells for curve shape on raw_data set and sift out corrupted
data. This step is not required but may help remove data modeling
errors.

```{r}
myApp <- weed_raw(raw_data, checkrange = c("A", "C", "1", "12"))
```

```{r, eval = FALSE}
shiny::runApp(myApp)
```

```{r}
raw_data <- remove_raw(raw_data, removerange = c("B", "H", "1", "12"))
screen(raw_data) + theme(
    aspect.ratio = 0.7,
    legend.position = "bottom",
    legend.text = element_text(size = 6),
    legend.key.size = unit(0.4, "cm"),
    legend.title = element_text(size = 8)
) +
    guides(color = guide_legend(nrow = 2, byrow = TRUE))
```

## 5. 96-well Analysis Application

TSAR package excels in mass analysis by propagating identical protocols
to all 96 wells. `smoothed = T` infers current data is smoothed and no
separate modeling is needed. If modeling is needed, input argument as
`smoothed = F`.

TSAR package performs derivative analysis using a generalized additive model 
through package `mgcv` or boltzmann analysis using nlsLM from package 
`minpack.lm`.

```{r, echo = FALSE}
x <- gam_analysis(raw_data,
    smoothed = TRUE, fluo_col = 5,
    selections = c(
        "Well.Position", "Temperature",
        "Fluorescence", "Normalized"
    )
)
x <- na.omit(x)
```

## 6. Intermediate Data Output

Read analysis using read_tsar() function and view head and tail to
ensure appropriate output was achieved. Data output can also be saved
locally into .csv or .txt format using function wrtie_tsar. However,
pipeline to downstream analysis does not require output to be locally
saved.

```{r, echo = FALSE}
# look at only Tm result by well
output <- read_tsar(x, output_content = 0)
head(output)
tail(output)
```

write output data file
`write_tsar(read_tsar(x, output_content = 2),`
    `name = "0923_tm_val", file = "csv")`

## 7. Complete Dataset with Ligand and Protein Information

For downstream analysis, data need to be mapped towards specific ligand
and compound. Use may input by default excel template included in the
package or input as .txt or .csv table, specifying Ligand and Compound
by Well ID. Data with coumpound and ligand labels can also be stored
locally using the same mean as previous step. All data are kept including
wells with blank condition information (specified as NA). 
In case removal is needed, call function na.omit().

```{r, echo = FALSE}
# join protein and ligand information
data("well_information")
norm_data <- join_well_info(
    file_path = NULL,
    file = well_information,
    read_tsar(x, output_content = 2), type = "by_template"
)
norm_data <- na.omit(norm_data)
head(norm_data)
tail(norm_data)
```

Write output into the working directory with write_tsar
`write_tsar(norm_data, name = "vitamin_tm_val_norm", file = "csv")`


## 8. Merge Data across Biological Replicates

Repeat step 2 through 6 on replicate data set. A five step function call
will complete all analysis. If additional screening is desired, a two
step call will run the interactive window to allow selection of

```{r}
data("qPCR_data2")
raw_data_rep <- qPCR_data2

raw_data_rep <- remove_raw(raw_data_rep, removerange = c("B", "H", "1", "12"))
```

```{r, eval = FALSE}
myApp <- weed_raw(raw_data_rep)
shiny::runApp(myApp)
```

```{r}
raw_data_rep <- remove_raw(raw_data_rep, removelist = "A12")
screen(raw_data_rep) + theme(
    aspect.ratio = 0.7,
    legend.position = "bottom",
    legend.text = element_text(size = 6),
    legend.key.size = unit(0.4, "cm"),
    legend.title = element_text(size = 8)
) +
    guides(color = guide_legend(nrow = 2, byrow = TRUE))

analysis_rep <- gam_analysis(raw_data_rep, smoothed = TRUE)
output_rep <- read_tsar(analysis_rep, output_content = 2)
norm_data_rep <- join_well_info(
    file_path = NULL,
    file = well_information,
    output_rep, type = "by_template"
)
norm_data_rep <- na.omit(norm_data_rep)
```

Merge data by content. All data are marked its source file name and
experiment date.

```{r}
norm_data <- na.omit(norm_data)
norm_data_rep <- na.omit(norm_data_rep)
tsar_data <- merge_norm(
    data = list(norm_data, norm_data_rep),
    name = c(
        "Vitamin_RawData_Thermal Shift_02_162.eds.csv",
        "Vitamin_RawData_Thermal Shift_02_168.eds.csv"
    ),
    date = c("20230203", "20230209")
)
```

## 9. Tm Estimation Shift Visualization

Use condition_IDs() and well_IDs() to select or remove condition to
visualize. Visualize Tm estimation by compound or ligand type in the
format of box graph.

```{r}
condition_IDs(tsar_data)
well_IDs(tsar_data)
conclusion <- tsar_data %>%
    filter(condition_ID != "NA_NA") %>%
    filter(condition_ID != "CA FL_Riboflavin")
TSA_boxplot(conclusion,
    color_by = "Protein", label_by = "Ligand",
    separate_legend = FALSE
)
```

## 10. TSA Curve Visualization

Specify Control condition by assigning condition_ID to control.
TSA_compare_plot generated multiple line graphs for comparison.

What's new:
User can now choose the smoother type ("gam", "beta", or "none") to be used in the plot.

```{r}
control_ID <- "CA FL_DMSO"

TSA_compare_plot(conclusion,
    y = "Fluorescence",
    control_condition = control_ID,
    smoother = "beta",
    beta_shape = 4, 
    beta_knots_frac = 0.008, 
    show_Tm = TRUE
)
```

Select by condition or well IDs to view curves and estimated Tm values.

What's new:
User can now choose the smoother type ("gam", "beta", or "none") to be used in the plot.
```{r}
error <- conclusion %>% filter(condition_ID == "CA FL_PyxINE HCl")
TSA_wells_plot(error, y='Fluorescence', separate_legend = TRUE, smoother = "beta", beta_shape = 4, beta_knots_frac = 0.008, show_Tm = TRUE)
```

To further visualize the comparison, graph first derivatives grouped by needs 
(i.e. well_ID, condition_ID, or other separately appended conditions).

Below is an example command. Due to size limit of vignette, graph will not be
displayed.
`view_deriv(conclusion, frame_by = "condition_ID")`

## 11. Session Info

### 11.1 Citation
```{r}
citation("TSAR")
citation()
citation("dplyr")
citation("ggplot2")
```

### 11.2 Session Info
```{r}
sessionInfo()
```

---
title: "TSAR Workflow by Shiny"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{TSAR Workflow by Shiny}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(warning = FALSE, message = FALSE, comment = "#>")
```
```{r, echo=FALSE, fig.width=4, out.width="400px"}
knitr::include_graphics("images/TSAR_logo.png")
```
## 1. Introduction
TSAR, short for Thermal Shift Analysis in R, provides simple solution to qPCR 
data processing, computing thermal shift analysis given either raw fluorescent 
data or smoothed curves. 
The functions provide users with the protocol to conduct preliminary data checks
and also expansive analysis on large scale of data. Furthermore, it showcases 
simple graphic presentation of analysis, generating clear box plot and line 
graphs given input of desired designs. Overall, TSAR Package offers a workflow
easy to manage and visualize.
TSAR Package is wrapped within a three separate shiny application regarding, 
data pre-processing, data analysis, and data visualization. All application 
can be opened in both interactive window or browsing engine by copy pasting 
server address into web browser.

## 2. Installation

## 3. Load TSAR Package
Use commands below to install TSAR package:
library(BiocManager)
BiocManager::install("TSAR")

```{r setup}
library(TSAR)
library(shiny)
```



## 4. Data Pre-Processing

Load data and remove blank wells by specifying a range of wells with 
`removerange = c()` or individual wells with `removelist = c()`. User may 
also use function `weed_raw()` to open Shiny application to screen all raw 
curves and remove by curve selection. To propagate change to data to local 
change, close window properly by clicking `Close Window`. The updated data 
set will be stored in the global environment as `new_raw_data`.

```{r}
raw <- data("qPCR_data1")
qPCR_data1 <- remove_raw(qPCR_data1, removerange = c("B", "H", "1", "12"))
```
```{r, eval = FALSE}
runApp(weed_raw(qPCR_data1))
```

In the plot panel, user can interact with graph and select and un-select 
data by clicking on curves. User may also `View Selected` wells only and 
`Remove Selected`. To return to default page and current all change to 
data set, click `Refresh Screening`. To propagate the same changes to 
local data set, simple click`Close Window`. Else, click `Copy Well IDs` 
or `Copy Selected in Full Function Call` to copy a `remove_raw()` call 
containing all selected wells into clipboard. Paste the call back into 
console or script and run to propagate changes locally. 
p.s. It is recommended to remove large areas of blank wells before calling 
`weed_raw()` to boost speed of application.

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/weed_raw.png")
```

Selection by grid is also possible; click on grid to highlight wells of 
selection. Click again to un-highlight. Click too fast may lead to error due to 
speed of reaction, use `Clear All Selected` button at the top of page to reset 
selection. Note that clearing selections alters to-be-copied selections, but 
will not restore curves already removed in the graph. However, changes are not 
made permanently. If erroneously removed, simply close and reopen window to 
run again.

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/weed_raw1.png")
```

Use `View Selected` and `Remove Selected` to view and remove selection. To 
return to viewing rest of the data, click `Refresh Screening`. To propagate the 
same changes to local data set, click `Copy Selected in Full Function Call` to 
copy a `remove_raw()` call containing all selected wells into clipboard. Paste 
the call back into console or script and run to propagate changes locally.
```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/weed_raw2.png")
```

## 5. Data Analysis

Analyze data by calling function `analyze_norm()` and follow the workflow from 
top to bottom. Preview data table for changes occurring at each step and refer 
to graph to view fit of model on each well. Always refer to the message for 
hints and error references. A success message will also be prompted after each 
successful run of step.

Remember to save analysis output locally by clicking `Save File`. Always 
preview data before saving to ensure data contains all necessary information.

```{r, eval = FALSE}
runApp(analyze_norm(qPCR_data1))
```
new feature:
user is able to choose either GAM or beta model for test fit.
```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/beta_model.png")
```

The top left panel output a preview of current data set. The right panel allows 
user to view the fit of model and Tm estimation by individual wells of 
selection. Once confirming correct data input and modeling effects,

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/analyze_norm.png")
```

Click `Analyze all Wells` to propagate model and analysis onto the rest of data.
A preview of analyzed data will also be modeling and analyzing all 96 Wells will
take between 30 to 50 seconds. If no modeling are needed, given data are 
smooth enough, analysis of all 96 wells should be completed under 5 seconds. 
p.s. actual time length many subject to change under different conditions

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/analyze_norm1.png")
knitr::include_graphics("images/analyze_norm12.png")
```

Upload well information by excel template and preview to confirm if information 
is correct. Use `Preview` to preview uploaded information and directly edit 
inside the window. User may also choose `Manual Input` to put in all condition 
information by Well. Make sure to hit `Save Changes` after editing and click 
`Set Conditions` to link all data to the conditions. A success message should 
be prompted.
p.s. manually inputted information will override file upload. If mistakenly 
saved, please close window and re-run command.

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/analyze_norm2.png")
knitr::include_graphics("images/analyze_norm3.png")
```

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/analyze_norm4.png")
```

Lastly, to save all analysis locally, click `Save File` after previewing output.
p.s. make to select `both` under `Choose dataset` if intending to use 
`graph_tsar()` or other graphing tools within the package.

## 6. Data Visualization

Use function `graph_tsar()` to start a Shiny application for graphing options. 
Run `na.omit()` on data if error occurs. Four graphing options are allowed, 
boxplot of Tm, compare plots, and conditions plot.

Function takes optional data parameter. If analysis file is already imported 
in the environment, call function as `graph_tsar(tsar_data)`. Refer to vignette 
"TSAR Workflow by Command" for instructions on how to merge using function 
`merge_norm()`. Else, user may use the merge data panel to upload and merge 
data of all test trials. Simply call `graph_tsar()` and click 
`Upload and Merge Data` button to reveal the panel to merge data.

```{r, eval = FALSE}


data("example_normalized_data")


runApp(graph_tsar(example_normalized_data))
```

Upload is limited to 30MB in size, but not count. After upload, user will 
be prompted corresponding numbers of input boxes to specify date of each 
experiment. Click `Save Dates`, then `Merge and Save Data`. A short preview 
of tsar_data will be prompted. For full lists and filters of well_ID and 
conditions_ID, refer to the helper buttons at the bottom of page.

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/graph_tsar_01.png")
knitr::include_graphics("images/graph_tsar_02.png")
knitr::include_graphics("images/graph_tsar_03.png")
knitr::include_graphics("images/graph_tsar_04.png")
```

Top panel outputs all plots, selected desired graphing features below and click 
generate to output graphs. p.s. Graphing compare plots and selected curves are 
takes longer than boxplot, please give it few seconds to load.

Boxplot has the option to be loaded interactive. Note interactive graphs have 
to be produced with legend combined.

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/graph_tsar.png")
knitr::include_graphics("images/graph_tsar1.png")
```

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/graph_tsar2.png")
```

Generating compare plots will output all plausible comparisons by control. To 
any specific one, a drop list `View Only:` will be prompted below the button 
`Generate Compare Plots`. Select by condition_ID to zoom in on graphs.

```{r echo=FALSE, out.width="400px"}
knitr::include_graphics("images/graph_tsar_32.png")
```

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/graph_tsar3.png")
```

Compare plots for first derivative graphs are interactive graphs grouped by 
specified variable.
```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/graph_tsar6.png")
```

Helper functions include these following:

```{r, out.width = "400px", echo = FALSE}
knitr::include_graphics("images/graph_tsar5.png")
knitr::include_graphics("images/graph_tsar4.png")
```

## 7. Session Info

### 7.1 Citation
```{r}
citation("TSAR")
citation()
citation("dplyr")
citation("ggplot2")
citation("shiny")
citation("utils")
```

### 7.2 Session Info
```{r}
sessionInfo()
```

---
title: "Frequently Asked Questions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Frequently Asked Questions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(warning = FALSE, message = FALSE, comment = "#>")
```
```{r, echo=FALSE, fig.width=4, out.width="400px"}
knitr::include_graphics("images/TSAR_logo.png")
```

```{r setup}
library(TSAR)
```

## FAQ

-   What are required dependencies?

    Load `Shiny` separately if running shiny applications. Load `ggplot2` and 
    `plotly` if intending to modify automated graphs.

-   How are Tm values estimated?

    TSAR package estimates uses derivative method by locating maximum first 
    derivative. Errors may occur during the process, edit estimation with 
    caution if errors were perceived.

-   Are smoothing applied to data, what kind?

    TSAR package utilizes `gam` function from package `mgcv`. Model assumes 
    `method = "GACV.Cp"` and sets to `formula = y ~ s(x, bs = "ad")`. Smoothing 
    may be toggled off by specifying parameters. Refer to function 
    documentations for instructions.

-   Got new suggestions or unmentioned-problems?

    Start a new issue in our github repository: `"CGAO123/TSAR"`

## Verify Data Input

TSAR package processes large amounts of data of similar properties, it is easy 
to confuse one with another and fail analysis. Verify your data input and out 
here by check for variable names. Consider renaming your data frame to theese 
following if issues are encountered.

-   raw_data

    -   required:

        -   Temperature (double)

        -   Fluorescence (double)

        -   Well.Position (character)

-   norm_data

    -   required:

        -   Temperature (double)

        -   Fluorescence (double)

        -   Well.Position (character)

    -   generated by tsar:

        -   norm_deriv (double)

        -   tm (double)

-   tsar_data

    -   required:

        -   Temperature (double)

        -   Fluorescence (double)

        -   Well.Position/Well (character)

        -   norm_deriv (double)

        -   Tm (double)

        -   Protein (character)

        -   Ligand (character)
        
    -   generated by tsar:

        -   ExperimentalFileName (character)

        -   well_ID (character)

        -   condition_ID (character)

For more detailed examples, refers to vignettes, `TSAR_Package_Structure` 
and `TSAR_Workflow_by_Command`.

## Locating Appropriate Function Calls

Given the two available workflows, here is a list of corresponding shiny 
functions and relative command line functions.

#### Under Data Pre-Processing, weed_raw():

-   View Selected; `TSAR::screen()`

-   Remove Selected; `remove_raw()`

#### Under Data Analysis, analyze_norm():

-   View Model Fit; `view_model()`

-   Analyze all Wells; `gam_analysis()`

    -   function wraps together, `normalize()`, `model_gam()`, 
    `model_fit()`, `Tm_est()`

-   Upload Well Information & Set Conditions; `join_well_info()`

-   Manual Input; load template using `data("well_information")`

    -   edit empty template by writing file into local end, `write.csv()`

-   Save File; `write_tsar()`, `read_tsar()`

#### Under Data Visualization, graph_tsar():

-   Merge Replicate Trials; `merge_norm()`

-   Generate Boxplot; `TSA_boxplot()`

-   Generate Compare Plots; `TSA_compare_plot()`

-   Graph Selected Curves; `TSA_wells_plot()`

-   Compare Derivatives; `view_deriv()`

-   List Condition IDs; `condition_IDs()`

-   List Well IDs; `well_IDs()`

-   List Tms; `TSA_Tms()`

-   List Delta Tms; `Tm_difference()`

## Session Info

```{r}
sessionInfo()
```

---
title: "R Notebook"
output: html_notebook
---

This is an [R Markdown](http://rmarkdown.rstudio.com) Notebook. When you execute code within the notebook, the results appear beneath the code.

Try executing this chunk by clicking the *Run* button within the chunk or by placing your cursor inside it and pressing *Cmd+Shift+Enter*.

```{r}
# IMPORTANT


# uncomment this if you clone newest version of TSAR from GitHub
# rm(list = c("merge_TSA", "TSA_average", "TSA_boxplot", "TSA_compare_plot", "TSA_wells_plot"))
# library("devtools")
# load_all('../../TSARnew') # make sure this should be the directory of TSAR source code
# devtools::document()
# devtools::install()


library(TSAR)


control_ID <- "CA121 WT_dH2O"
data("example_normalized_data")



# library('svglite')
library(ggplot2)
library(ggpubr)
library(readxl)
library(stringr)
library(tidyr)
library(dplyr)
library(readr)
```



```{r}



library('svglite')
# names(normalized_tsa_df)[names(normalized_tsa_df) == "RFU"] <- "Normalized"
# TSA_compare_plotx(example_normalized_data,y = "RFU",control_condition = control_ID,show_Tm = T)
TSA_compare_plot(example_normalized_data, control_condition = control_ID,
                           y = "RFU", smoother = "beta",
                           beta_shape = 4, beta_knots_frac = 0.008, show_Tm = TRUE)

# uncomment the following if you want to save the plots


# for (nm in names(plots)) {
#   # If your list contains NULLs (e.g. for unused slots), skip those:
#   if (is.null(plots[[nm]])) next
# 
#   # Construct filename: e.g., "plot_ConditionA.svg"
#   fname <- paste0("plot_", gsub("[^A-Za-z0-9_-]", "_", nm), ".svg")
#   ggsave(filename = fname, plot = plots[[nm]], width = 6, height = 4, units = "in", device = "svg")
# }
```

```{r}

# source("../R/tsa_curves.R")
for(i in 1:condition_IDs(example_normalized_data, n = TRUE)) {
    cond_i <- condition_IDs(example_normalized_data)[i]
    subset_i <- example_normalized_data$condition_ID == cond_i
    subset_i <- example_normalized_data[subset_i, ]
    # plot_i <- TSA_wells_plot(subset_i)
    plot_i <-  TSA_wells_plot(subset_i, y = "RFU", smooth = TRUE,
               smoother = "beta", beta_shape = 4, beta_knots_frac = 0.008)
    
    print(plot_i)
    
    # uncomment the following if you want to save the plots
    # Create a filename for each condition
    # svg_file <- paste0("tsa_plot_", gsub("[^A-Za-z0-9_-]", "_", cond_i), ".svg")
  
    # Save the plot as SVG
    # ggplot2::ggsave(filename = svg_file, plot = plot_i[[1]], device = "svg", width = 6, height = 4, units = "in")
}
```
```{r}
# 1) Compare L2 (RMSE) across GAM and a small Beta grid
# source("../R/averageTSA.R")
res_L2 <- TSA_smoother_diagnostics(
  example_normalized_data,
  y = "RFU",
  metric = "L2",
  # beta_shapes = c(2, 4),
  # beta_fracs  = c(0.008, 0.02)
)
res_L2_df <- as.data.frame(res_L2)
res_L2_df

```
```{r}

```

Add a new chunk by clicking the *Insert Chunk* button on the toolbar or by pressing *Cmd+Option+I*.

When you save the notebook, an HTML file containing the code and output will be saved alongside it (click the *Preview* button or press *Cmd+Shift+K* to preview the HTML file).

The preview shows you a rendered HTML copy of the contents of the editor. Consequently, unlike *Knit*, *Preview* does not run any R code chunks. Instead, the output of the chunk when it was last run in the editor is displayed.

---
title: "TSAR Package Structure"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{TSAR Package Structure}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(warning = FALSE, message = FALSE, comment = "#>")
```

# 1. Introduction
TSAR Package provides simple solution to qPCR data processing, computing 
thermal shift analysis given either raw fluorescent data or smoothed curves. 
The functions provide users with the protocol to conduct preliminary data checks
and also expansive analysis on large scale of data. Furthermore, it showcases 
simple graphic presentation of analysis, generating clear box plot and line 
graphs given input of desired designs. Overall, TSAR Package offers a workflow
easy to manage and visualize.

# 2. Installation
Use commands below to install TSAR package:
library(BiocManager)
BiocManager::install("TSAR")

```{r setup, message=FALSE, warning=FALSE}
library(TSAR)
library(dplyr)
library(ggplot2)
library(shiny)
library(utils)
```

```{r, echo=FALSE, fig.width=4, out.width="400px"}
knitr::include_graphics("images/TSAR_logo.png")
```

# 3. Data Structure

TSAR segregates data structure into three tiers:

-   `raw_data`, raw-readings of qPCR data
-   `norm_data`, pre-processed and normalized data
-   `tsar_data`, analyzed with ready for graphs

Users may initiate the TSAR workflow from either `raw_data` or `norm_data` as
long as the data achieves approriate qualities. Functions corresponding each 
tier are wrapped in shiny application. All analysis and visualization can be 
achieved within user-interactive window, also open-able in local browser.

- `weed_raw()`; input: raw; output: data without blank and corrupted curves
- `analyze_norm()`; input: raw or output of weed_raw; output: analyzed data 
with conditions specified
- `graph_tsar()`; input: tsar_data or output of analyze_norm; output: graphs

# 4. From `raw_data` to `norm_data`

To access the workflow of `TSAR`, user can input raw fluorescent readings 
through built in functions from utils: `read.delim()` or `read.csv()`. Built 
in data imports from RStudio UI is also appropriate, given `raw_data` only needs
to be saved as `dataframe` structure. Wrapping input in `data.frame()` will 
also ensure correct data type. Use `head()` and `tail()` to ensure excessive 
information is removed, i.e. blank lines, duplicate titles, etc.

```{r, results = 'hide'}
data("qPCR_data1")
raw_data <- qPCR_data1
head(raw_data)
tail(raw_data)
```

## 4.1 Data Preprocessing

From raw data to ready-for-analysis data, there are few functions to assist the 
selection and normalization of data.

Functions `screen()` and `remove_raw()` helps screen and remove data of 
selection. This save time and space by remove unwanted data such as blank wells 
and corrupted curves. Aside from `data` param input, both functions share 
similar parameters of `checkrange`, `checklist` or `removerange`, `removelist`.

-   `checkrange`, `removerange` : a range of wells, e.g. wells from A01 to 
B08 is `c("A","B","1","8")`

-   `checklist`, `removelist`: a list of wells, e.g. `c("A01","C03")`

If no wells are specified, `screen()` will default to screening every well 
and `reomve_raw()` will default to not removing an well.

```{r, out.width = "400px"}
screen(raw_data) + theme(
    aspect.ratio = 0.7,
    legend.position = "bottom",
    legend.text = element_text(size = 4),
    legend.key.size = unit(0.1, "cm"),
    legend.title = element_text(size = 6)
) +
    guides(color = guide_legend(nrow = 4, byrow = TRUE))
raw_data <- remove_raw(raw_data, removerange = c("C", "H", "1", "12"))
```

Both functions above are wrapped inside an interactive window through function
`weed_raw()`. It is implemented through R Shiny application where users can 
select curves using cursor and remove selected curves easily. Refer to separate 
vignette, `"TSAR Workflow by Shiny"`, and README.md file for more documentation.

```{r eval = FALSE}
runApp(weed_raw(raw_data))
```

Running the window, we can spot that the curve at A12 has an abnormally high 
initial fluorescence and should be removed for data accuracy. We can make sure 
of correct selection through the `View Selected` button and remove it using 
the `Remove Data` button. Note that all data edits are made inside the 
interactive window. To translate the change globally for downstream analysis, 
simply click `Save to R`and close window to store data into the global 
environment. Alternatively, click `Copy Selected` and paste information to 
`remove_raw()`. To avoid error, close window proper through `Close Window` 
instead of the cross mark on top left corner.

```{r}
raw_data <- remove_raw(raw_data,
    removelist =
        c(
            "B04", "B11", "B09", "B05", "B10", "B03",
            "B02", "B01", "B08", "B12", "B07", "B06"
        )
)
```

## 4.2 Data Analysis

Normalizing data is prompted through `normalize()`. Although individual calls 
are not necessary as they are wrapped together in `gam_analysis()`, if viewing 
the validity of model is desired, one can prompt analysis of one well.

TSAR package performs derivative analysis using a generalized additive model 
through package `mgcv` or boltzmann analysis using nlsLM from package 
`minpack.lm`.

### 4.2.1 Individual Well Application

For analysis of an idividual well, refer to these following functions:

-   `normalize()`

-   `model_gam()`

-   `model_fit()`

-   `view_model()`

-   `Tm_est()`

```{r, out.width = "400px"}
test <- filter(raw_data, raw_data$Well.Position == "A01")
test <- normalize(test)
gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
test <- model_fit(test, model = gammodel)
view <- view_model(test)
view[[1]] + theme(aspect.ratio = 0.7, legend.position = "bottom")
view[[2]] + theme(aspect.ratio = 0.7, legend.position = "bottom")
Tm_est(test)
```
View model generates a list of two graphs, showing fit of modeling on 
fluorescence data and the derivative calculation of such data.


### 4.2.2 96-Well Plate Application

All analysis necessary are formatted in function `gam_analysis()`. Parameters 
are inherited from functions noted in section "Individual Well Application". 
Hence, if any errors are prompted, check through individual well application 
for correct parameter input and other potential errors.

-   `smoothed` inherited from model_fit()
-   `fluo` and `selections` inherited from normalize()

```{r}
x <- gam_analysis(raw_data,
    smoothed = TRUE,
    fluo_col = 5,
    selections = c(
        "Well.Position", "Temperature",
        "Fluorescence", "Normalized"
    )
)
```

## 4.3 Data Summary

Data summary offers an exit point from the workflow if no further graphic 
outputs are required. Output is allowed in three formats: 
-   `output_content = 0`, only Tm values by well 
-   `output_content = 1`, all data analysis by each temperature reading. 
If previously called `smoothed = T`, analysis will not run gam modeling, thus 
will not have `fitted` data. 
-   `output_content = 2`, combination of the above two data set.

To associate ligand and protein conditions with each individual well, call 
the function `join_well_info()`. One may specify using the template excel or 
separate csv file containing a table of three variables, "Well", "Protein", 
and "Ligand".

```{r message=FALSE}
data("well_information")
output <- join_well_info(
    file_path = NULL, file = well_information,
    read_tsar(x, output_content = 0), type = "by_template"
)
```

Write output using command write_tsar
`write_tsar(output, name = "vitamin_analysis", file = "csv")`

To streamline to the following graphic analysis, make sure `output_content = 2` 
to maintain all necessary data.

```{r message=FALSE}
norm_data <- join_well_info(
    file_path = NULL, file = well_information,
    read_tsar(x, output_content = 2), type = "by_template"
)
```

All of the function above in section 2 are wrapped together in a shiny 
application named `analyze_norm()`. Refer to separate vignette, 
`"TSAR Workflow by Shiny"`, and README.md file for more documentation.

```{r eval = FALSE}
runApp(analyze_norm(raw_data))
```

# 5. From `norm_data` to `tsar_data`

`norm_data` contains normalized fluorescent data on a scale of 0 to 1 based 
on the maximum and minimum fluorescence reading. `norm_data` also contains a 
first derivative column. `tsar_data` is the final format of project data 
encapsulating all replication. Therefore, it contains all condition data 
including experiment date and analysis file source.

## 5.1 Merge Replicates

Use `merge_norm()` to merge all norm_data and specify original data file name 
and experiment date for latter tracking purposes.

```{r message=FALSE}
# analyze replicate data
data("qPCR_data2")
raw_data_rep <- qPCR_data2
raw_data_rep <- remove_raw(raw_data_rep,
    removerange = c("B", "H", "1", "12"),
    removelist = c("A12")
)
analysis_rep <- gam_analysis(raw_data_rep, smoothed = TRUE)
norm_data_rep <- join_well_info(
    file_path = NULL, file = well_information,
    read_tsar(analysis_rep, output_content = 2),
    type = "by_template"
)

# merge data
tsar_data <- merge_norm(
    data = list(norm_data, norm_data_rep),
    name = c(
        "Vitamin_RawData_Thermal Shift_02_162.eds.csv",
        "Vitamin_RawData_Thermal Shift_02_168.eds.csv"
    ),
    date = c("20230203", "20230209")
)
```

### 5.1.1 Jumpstart to Graph

If outputted data from qPCR already contains analysis and data necessary, 
enter TSAR workflow from here, using functions `merge_TSA()`, `read_raw_data()`,
`read_analysis()`.

```{r}
#analysis_file <- read_analysis(analysis_file_path)
#raw_data <- read_raw_data(raw_data_path)
#merge_TSA(analysis_file, raw_data)
```

After merging, use assisting functions to check and trace data. Use these two 
functions to guide graphics analysis for error identification, selective 
graphing and graph comparisons.

-   `condition_IDs()` list all conditions in data
-   `well_IDs()` list all IDs of individual well
-   `TSA_proteins()` list all distinct proteins
-   `TSA_ligands()` list all distinct ligands
-   `TSA_Tms()` list all Tm estimations by condition
-   `Tm_difference()` list all delta Tm estimations by control condition

```{r}
condition_IDs(tsar_data)
well_IDs(tsar_data)
TSA_proteins(tsar_data)
TSA_ligands(tsar_data)

conclusion <- tsar_data %>%
    filter(condition_ID != "NA_NA") %>%
    filter(condition_ID != "CA FL_Riboflavin")
```

## 5.2 Graphic Analysis

### 5.2.1 Tm Boxplot

Use `TSA_boxplot()` to generate comparison boxplot graphs. Stylistics choices 
include coloring by protein or ligand, and legend separation. Function returns 
ggplot object, thus further stylistic changes are allowed.

```{r, out.width = "400px"}
TSA_boxplot(conclusion,
    color_by = "Protein",
    label_by = "Ligand", separate_legend = TRUE
)
```

### 5.2.2 TSA Curve Visualization

`TSA_compare_plot()` generates multiple line graphs for comparison. Specify 
Control condition by assigning condition_ID to control. Functions allows 
graphing by both: - raw fluorescent readings `y = 'Fluorescence'` - normalized 
readings `y = 'RFU'`.

```{r}
control_ID <- "CA FL_DMSO"

TSA_compare_plot(conclusion,
    y = "Fluorescence",
    control_condition = control_ID
)
```

### 5.2.3 Curves by Condition

Users may also graph by condition IDs or well IDs using function 
`TSA_wells_plot()`.

```{r}
ABA_Cond <- conclusion %>% filter(condition_ID == "CA FL_4-ABA")
TSA_wells_plot(ABA_Cond, separate_legend = TRUE, y = "Fluorescence")
```

### 5.2.4 First Derivative Comparison
To further visualization comparison, graph first derivatives grouped by needs. 
Note if modeling was set to boltzman fit, frist derivatives will be excessively 
smooth and contains no information beyond specified minimum and maximum.
Below is an example command. Due to size limit of vignette, graph will not be
displayed.
`view_deriv(conclusion, frame_by = "condition_ID")`

All of the above functions are also wrapped in an interactive window call
through `graph_tsar()`. Simply call function on merged tsar_data and access all 
graphing features in one window. Refer to separate vignette, 
`"TSAR Workflow by Shiny"`, and readme.md file for more documentation.

```{r eval = FALSE}
runApp(graph_tsar(tsar_data))
```

# 6. Session Info

## 6.1 Citation
```{r}
citation("TSAR")
citation()
citation("dplyr")
citation("ggplot2")
citation("shiny")
citation("utils")
```

## 6.2 Session Info
```{r}
sessionInfo()
```

# A more smoothed fitted curve method: Cubic Spline with beta-knots for Thermal Shift Analysis in R (TSAR)

*In the following text, "beta" or "beta method" is referred to Cubic Spline with beta-knots for Thermal Shift Analysis in R (TSAR).*

## Summary

This document introduces a new smoothing method, beta method, overwriting for the `TSA_average()` function that utilizes a beta-knots natural cubic spline centered at numerical $T_m$ point to fit a smoother curve to the melting point data. The new method exhibits **solves the problem of oscillating fitted curves at the boundaries resulted by General Additive Model (GAM).**

The new method is a overwritten method to existing method `TSA_average()` with two children methods calling the new method: `TSA_compare_plot()` and `TSA_wells_plot()` such that

-   **`TSA_average()`**
    -   New argument `smoother = c("gam", "beta", "none")`.
    -   **Beta-knot natural cubic spline** option with interior knots drawn from **Beta(a, a)** and **centered at the condition’s** $T_m$.
-   **`TSA_compare_plot()`**
    -   Accepts and forwards `smoother` to `TSA_average()`.
-   **`TSA_wells_plot()`**
    -   Accepts and forwards `smoother` to `TSA_average()`.

## Motivation

### Limitations of GAM

1.  **Oscillation at Boundaries**: GAM often induces Runge's phenomenon, where fitted curves show oscillation at the boundaries, reducing its capacity to fit the data accurately and leading to misleading its interpretations.
2.  **Lack of Domain Awareness**: GAM does not concentrate flexibility near the central melting-point region, resulting a different $T_m$ from numerical $T_m$.

### Resolution

The **Beta Method** uses a natural cubic spline with interior knots drawn from a Beta distribution centered interest's numerical $T_m$ calculated prior. This method stabilizes the fitted curve in boundary condition and in the same time provides a more accurate fit to the melting point data.

The idea of using Beta Method is drawn from the facts that (1) cubic spline is a well-known numerical method for smoothing data requiring few computations as well as providing a good numerical accuracy; (2) a beta distribution is a centered hill-like distribution that fulfills the demand of concentrated knot placements near the melting point region and sparse knots at the boundaries.

## What’s Changed

### 1) `TSA_average()`

``` r
TSA_average(
  tsa_data,
  y = c("Fluorescence", "RFU"),
  digits = 1,
  avg_smooth = TRUE,
  sd_smooth  = TRUE,
  smoother   = c("gam", "beta", "none"),
  beta_shape      = 3,      # a in Beta(a, a)
  beta_n_knots    = NULL,   # if NULL, use beta_knots_frac
  beta_knots_frac = 0.008,   # fraction of unique temperatures used as interior knots
  use_natural     = TRUE    # natural cubic (recommended)
)
```

> Note:`smoother="gam"` reproduces previous output.

### 2) `TSA_compare_plot()`

-   New argument: `smoother = c("gam", "beta", "none")`

### 3) `TSA_wells_plot()`

-   New argument: `smoother = c("gam", "beta", "none")`

## What’s New

### 1) `TSA_smoother_diagnostics()`

-   A diagnostic method to tests the fitted curves with different smoothing methods and calculates errors. It intends to help users to find the best smoothing method/parameters with lowest error.
-   It will return a data frame with the following columns:
    -   `condition_id`: the condition ID
    -   `method`: candidate smoothing method used
    -   `error`: the error of the fitted curve to the original data
-   It will also print a report of the best method/parameters with the lowest error.

``` r
TSA_smoother_diagnostics <- function(
    tsa_data,
    y = c("Fluorescence", "RFU"),
    metric = c("L2", "L1", "R2"),
    # beta grids to test; if NULL or length 0, default to a=4 and frac=0.008
    beta_shapes = NULL,
    beta_fracs  = NULL,
    # pass-through options if you need them
    digits = 1,
    use_natural = TRUE,
    beta_n_knots = NULL
)
```

### 2) `model_beta()`

-   The cloned method from `TSA_average()` that uses exactly the same beta method to fit the curve but in data pre-processing step.
-   Numerical $T_m$ is obtained via numerical differentiation: detect zero-crossings of the second finite difference and choose the steepest slope crossing.
-   Returns an augmented data frame ready for plotting/diagnostics (compatible with `view_model()`), including:
    -   original columns (e.g., `Temperature`, `Normalized`)
    -   fitted: spline-fitted values at the original `x`
    -   norm_deriv: numerical derivative of the fitted curve (The underlying `lm` fit and knot metadata are stored as attributes.)

``` r
model_beta <- function(
  norm_data,
  x,
  y,
  beta_shape      = 4,      # Beta(a,a) shape
  beta_knots_frac = 0.008,  # fraction of unique x used as interior knots
  beta_n_knots    = NULL,   # override: set number of interior knots directly
  use_natural     = TRUE    # use splines::ns() (natural cubic); else splines::bs()
)
```

### 3) `numerical_Tm()`

-   This method computes the **melting temperature** $(Tm)$ directly from raw TSA data.
-   The $T_m$ is fetched by assuming $T_m$ is the inflection point (local extremum point of first derivative) of the TSA curve via numerical method.
-   It returns a named numeric vector: `c(x = x_Tm, y = y_Tm)`. If the curve is too short or ill-posed, returns `NA`s.

``` r

numerical_Tm <- function(
  data, # data.frame containing at least the columns `x` (temperature) and `y` (fluorescence/normalized RFU).
  x, # `x`, `y`: column names (characters)
  y
) 
```

### 4) Updated Shiny GUI integrated with Beta Method

-   Beta method now has also incorporated into the Shiny app where user can choose either GAM or Beta method for analysis.

![GUI beta curve](/images/GUI_beta.png)

## Performance

> Dotted curves represent the fitted curve; colored curves represent the data.

**Example 1** Beta method shows smoother fitted curves than GAM

|                            |                             |
|----------------------------|-----------------------------|
| GAM                        | Beta Method                 |
| ![](/images/gam_CuCl2.png) | ![](/images/beta_CuCl2.png) |

**Example 2** Beta method shows less oscillating boundary condition than GAM

|                           |                            |
|---------------------------|----------------------------|
| GAM                       | Beta Method                |
| ![](/images/gam_PF74.png) | ![](/images/beta_PF74.png) |

**Example 3** With 13 samples in one experiment of TSA, Beta method shows more accurate fitted curves (\~30.4% improvement) than GAM. The data comes from

``` r
data("example_normalized_data")
```

| condition_id                 | method                | error      |
|------------------------------|-----------------------|------------|
| CA121 WT_dH2O                | gam                   | 0.03883613 |
| CA121 WT_dH2O                | beta(a=4, frac=0.008) | 0.01457599 |
| CA121 WT_PF74                | gam                   | 0.02145768 |
| CA121 WT_PF74                | beta(a=4, frac=0.008) | 0.01469731 |
| CA121 WT_Gallic acid         | gam                   | 0.02233872 |
| CA121 WT_Gallic acid         | beta(a=4, frac=0.008) | 0.02583934 |
| CA121 WT_FeCl3 + Gallic acid | gam                   | 0.02017039 |
| CA121 WT_FeCl3 + Gallic acid | beta(a=4, frac=0.008) | 0.01009895 |
| CA121 WT_CuCl2 + Gallic acid | gam                   | 0.01720631 |
| CA121 WT_CuCl2 + Gallic acid | beta(a=4, frac=0.008) | 0.01629382 |
| CA121 WT_CaCl2 + Gallic acid | gam                   | 0.01285055 |
| CA121 WT_CaCl2 + Gallic acid | beta(a=4, frac=0.008) | 0.03252719 |
| CA121 WT_MgCl2 + Gallic acid | gam                   | 0.02231318 |
| CA121 WT_MgCl2 + Gallic acid | beta(a=4, frac=0.008) | 0.02593790 |
| CA121 WT_NaCl + Gallic acid  | gam                   | 0.02344820 |
| CA121 WT_NaCl + Gallic acid  | beta(a=4, frac=0.008) | 0.02589357 |
| CA121 WT_FeCl3               | gam                   | 0.03972613 |
| CA121 WT_FeCl3               | beta(a=4, frac=0.008) | 0.01665110 |
| CA121 WT_CuCl2               | gam                   | 0.03298392 |
| CA121 WT_CuCl2               | beta(a=4, frac=0.008) | 0.01408443 |
| CA121 WT_CaCl2               | gam                   | 0.04074613 |
| CA121 WT_CaCl2               | beta(a=4, frac=0.008) | 0.02054448 |
| CA121 WT_MgCl2               | gam                   | 0.03869195 |
| CA121 WT_MgCl2               | beta(a=4, frac=0.008) | 0.01902785 |
| CA121 WT_NaCl                | gam                   | 0.03990452 |
| CA121 WT_NaCl                | beta(a=4, frac=0.008) | 0.02183548 |

mean L2 error of gam method: 0.02851 mean L2 error of beta(a=4, frac=0.008): 0.01985

beta(a=4, frac=0.008) has lowest error and highest accuracy.

> The table and report above was generated by `TSA_smoother_diagnostics()`

## Usage

see [`TSAR_Workflow_Beta_Method.Rmd`](/TSAR_Workflow_Beta_Method.Rmd)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Well_Information.R
\name{well_information}
\alias{well_information}
\title{example well information Data}
\format{
A data frame with the following columns:
    \describe{
      \item{...1}{n/a}
      \item{Protein...2}{Protein in Well 1}
      \item{Ligand...3}{Ligand in Well 1}
      \item{Protein...4}{Protein in Well 2}
      \item{Ligand...5}{Ligand in Well 2}
      \item{Protein...6}{Protein in Well 3}
      \item{Ligand...7}{Ligand in Well 3}
      \item{Protein...8}{Protein in Well 4}
      \item{Ligand...9}{Ligand in Well 4}
      \item{Protein...10}{Protein in Well 5}
      \item{Ligand...11}{Ligand in Well 5}
      \item{Protein...12}{Protein in Well 6}
      \item{Ligand...13}{Ligand in Well 6}
      \item{Protein...14}{Protein in Well 7}
      \item{Ligand...15}{Ligand in Well 7}
      \item{Protein...16}{Protein in Well 8}
      \item{Ligand...17}{Ligand in Well 8}
      \item{Protein...18}{Protein in Well 9}
      \item{Ligand...19}{Ligand in Well 9}
      \item{Protein...20}{Protein in Well 10}
      \item{Ligand...21}{Ligand in Well 10}
      \item{Protein...22}{Protein in Well 11}
      \item{Ligand...23}{Ligand in Well 11}
      \item{Protein...24}{Protein in Well 12}
      \item{Ligand...25}{Ligand in Well 12}
    }
}
\usage{
data(well_information)
}
\value{
well information data frame
}
\description{
Dataset Description:
    This file is a readin using well_information_template. File contains the
    conditions of well, specifying protein and ligand content in well.
    All experimental setup and relevant information are determined and
    manually put in by the author of this package.
}
\keyword{dataset}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_normalized_data.R
\docType{data}
\name{example_normalized_data}
\alias{example_normalized_data}
\title{Example normalized TSA dataset}
\format{
A data frame with 63,544 rows and 14 variables:
\describe{
  \item{well_ID}{Unique well identifier (character).}
  \item{Well}{Well label (character), e.g., "B01".}
  \item{Protein}{Protein name/label (character).}
  \item{Ligand}{Ligand name/label (character).}
  \item{Tm}{Estimated melting temperature for the well/condition (numeric).}
  \item{dTm D}{Delta Tm field as stored in the dataset (numeric; may be NA).}
  \item{Analysis Group}{Analysis group label (character).}
  \item{Flags}{QC/flag field (character or numeric depending on preprocessing).}
  \item{Experiment File Name}{Source experiment/file name (character).}
  \item{condition_ID}{Condition identifier used throughout TSAR (character).}
  \item{Well.Position}{Well position label (character).}
  \item{Temperature}{Temperature in degrees Celsius (numeric).}
  \item{Fluorescence}{Raw fluorescence (numeric).}
  \item{RFU}{Normalized fluorescence / relative fluorescence units (numeric).}
}
}
\source{
Included with the package for demonstration and testing.
}
\description{
A demonstration dataset of TSA measurements that have been normalized and
merged into the TSAR/TSAR-new long format (one row per observation).
Suitable for examples in \code{\link{TSA_wells_plot}} and
\code{\link{TSA_compare_plot}}.
}
\examples{
data("example_normalized_data")
head(example_normalized_data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsa_curves.R
\name{TSA_wells_plot}
\alias{TSA_wells_plot}
\title{TSA Well Curves Plot}
\usage{
TSA_wells_plot(
  tsa_data,
  y = "RFU",
  show_Tm = TRUE,
  Tm_label_nudge = 7.5,
  show_average = TRUE,
  plot_title = NA,
  plot_subtitle = NA,
  smooth = TRUE,
  separate_legend = TRUE,
  smoother = c("gam", "beta", "none"),
  beta_shape = 3,
  beta_n_knots = NULL,
  beta_knots_frac = 0.12,
  use_natural = TRUE
)
}
\arguments{
\item{tsa_data}{a data frame that is merged and generated by
TSAR::merge_TSA(). If \code{y = 'RFU'}, tsa_data must also be
generated by TSAR::normalize_fluorescence. The Temperature column will be
rounded and the average & sd of each rounded temperature is calculated.}

\item{y}{character string; c('Fluorescence', 'RFU').
When \code{y = 'Fluorescence'}, the original Fluorescence data from
TSAR::read_raw_data() is averaged. When \code{y = 'RFU'}, the average is
calculated by the rescaled fluorescence.}

\item{show_Tm}{logical; \code{show_Tm = TRUE} by default. When TRUE, the Tm
is displayed on the plot. When FALSE, the Tm is not added to the plot.}

\item{Tm_label_nudge}{numeric; \code{Tm_label_nudge = 7.5} the direction
in the x direction to move the Tm label. This is used prevent the label
from covering data. Ignored if \code{show_Tm = FALSE}.}

\item{show_average}{logical; \code{show_average = TRUE} by default.
When TRUE, the average is and sd is plotted as
generated by \code{\link{merge_TSA}}.}

\item{plot_title, plot_subtitle}{characer string, NA by default.
User-specified plots to overright automatic naming.}

\item{smooth}{logical; \code{smooth = TRUE} by default. When TRUE, linear
regression by \code{\link[mgcv]{gam}} is used to make clean lines on
the plot. See \code{\link{TSA_average}} for more details. When FALSE,
individual points are plotted (slows down rendering).}

\item{separate_legend}{logical; \code{separate_legend = TRUE} by default.
When TRUE, the ggplot2 legend is separated from the TSA curve.
This is to help with readability. One ggplot is returned when FALSE.}

\item{smoother}{character; one of \code{c("gam","beta","none")}.
Passed to \code{\link{TSA_average}} to select the aggregate smoother:
\code{"gam"} uses \pkg{mgcv}, \code{"beta"} uses a natural cubic spline
with Beta(a,a) interior knots centered at Tm, and \code{"none"} uses the
unsmoothed average. Default follows \code{TSA_average}.}

\item{beta_shape}{numeric; shape parameter \eqn{a} for the Beta(a,a) knot
placement when \code{smoother = "beta"}. \code{beta_shape = 3} by default. Passed to \code{TSA_average}.}

\item{beta_n_knots}{integer or \code{NULL}; number of interior knots when
\code{smoother = "beta"}. If \code{NULL}, uses \code{beta_knots_frac}.
Passed to \code{TSA_average}.}

\item{beta_knots_frac}{numeric in (0,1); fraction of unique temperatures used
as interior knots when \code{smoother = "beta"} and \code{beta_n_knots}
is \code{NULL}. \code{beta_knots_frac = 0.008} by default. Passed to \code{TSA_average}.}

\item{use_natural}{logical; if TRUE (default) uses natural cubic spline
basis for the beta method. Passed to \code{TSA_average}.}
}
\value{
By default, two ggplots are returned: one TSA curve and one legend
    key (when \code{separate_legend = TRUE}). When
    \code{separate_legend = FALSE}, a single ggplot is returned.
}
\description{
Generates the individual curves for each well in the merged tsa data input.
    Options to create an average and standard deviation sd of the plot
    in addition to the individual curves. The average and sd will be
    smoothened by linear regression; see \code{\link{TSA_average}} for
    details.
}
\examples{
data("example_tsar_data")
check <- subset(example_tsar_data, condition_ID == "CA FL_PyxINE HCl")
TSA_wells_plot(check, y = "Fluorescence", smooth = TRUE, separate_legend = TRUE,
               smoother = "beta", beta_shape = 4, beta_knots_frac = 0.008)

}
\seealso{
\code{\link{merge_TSA}} and \code{\link{normalize_fluorescence}}
    for preparing data. See \code{\link{TSA_average}} and
    \code{\link{get_legend}} for details on function parameters.

Other TSA Plots: 
\code{\link{TSA_boxplot}()},
\code{\link{TSA_compare_plot}()},
\code{\link{get_legend}()},
\code{\link{graph_tsar}()},
\code{\link{view_deriv}()}
}
\concept{TSA Plots}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{model_beta}
\alias{model_beta}
\title{Beta-knot natural cubic spline model (Tm-centered via numerical_Tm)}
\usage{
model_beta(
  norm_data,
  x,
  y,
  beta_shape = 4,
  beta_knots_frac = 0.008,
  beta_n_knots = NULL,
  use_natural = TRUE
)
}
\arguments{
\item{norm_data}{data.frame with numeric columns x and y (normalized).}

\item{x, y}{character; names of the predictor and response columns.}

\item{beta_shape}{numeric > 0; Beta(a,a) shape parameter. Default 4.}

\item{beta_knots_frac}{numeric in (0,1); fraction of unique x used for
interior knots if \code{beta_n_knots} is NULL. Default 0.008.}

\item{beta_n_knots}{integer or NULL; overrides fraction when provided.}

\item{use_natural}{logical; if TRUE (default) uses \code{splines::ns()},
else \code{splines::bs(..., degree = 3)}.}
}
\value{
\code{"lm"} fit with spline basis; attributes store knots and settings.
}
\description{
Fits a natural cubic spline to \code{y} as a function of \code{x} using
interior knots at Beta(a,a) quantiles, centered at the numerically estimated
Tm from \code{\link{numerical_Tm}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getLegend.R
\name{get_legend}
\alias{get_legend}
\title{Extract ggplot2 legend}
\usage{
get_legend(input_plot)
}
\arguments{
\item{input_plot}{a ggplot2 object}
}
\value{
two ggplots, one containing the legend and another containing
  all else.
}
\description{
Function enables separation of legends from plots within the TSAR package.
}
\examples{
data("example_tsar_data")
boxplot <- TSA_boxplot(example_tsar_data,
    color_by = "Protein",
    label_by = "Ligand", separate_legend = FALSE
)
get_legend(boxplot)

}
\seealso{
Other TSA Plots: 
\code{\link{TSA_boxplot}()},
\code{\link{TSA_compare_plot}()},
\code{\link{TSA_wells_plot}()},
\code{\link{graph_tsar}()},
\code{\link{view_deriv}()}
}
\concept{TSA Plots}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeTSA.R
\name{merge_TSA}
\alias{merge_TSA}
\title{Merge TSA Raw Data and Analysis Files}
\usage{
merge_TSA(analysis_file_path, raw_data_path, protein = NA, ligand = NA)
}
\arguments{
\item{raw_data_path, analysis_file_path}{a character string or vector of
    character strings; the path or the name of the file which the
    'RawData' or "AnalysisData'  are to be read from. Raw data and Analysis
    Data are to be loaded as a pair of results from the same experiment.
    When loading multiple files, the index/position of the pairs are
    merged where the first file specified by raw_data_path is to be merged
    with the first file specified by analysis_file_path. The same is done
    for the second, third, .. etc. \cr
    Either a .txt or .csv file; file type can vary between file pairs.

    raw_data_path path must contain the term \emph{RawData} and
    analysis_file_path must contain the term  \emph{AnalysisResults} as the
    TSA software automatically assigns this when exporting data. Data is
    loaded from the \code{\link{read_raw_data}} and
    \code{\link{read_analysis}} functions within this merge_TSA function.}

\item{protein}{can be used to select for an individual or multiple protein(s)
as a character string matching protein names assigned in the TSA
software. NA by default.}

\item{ligand}{can be used to select for an individual or multiple ligand(s)
as a character string matching ligand names assigned in the TSA software.
NA by default.}
}
\value{
A data frame of merged TSA data.
}
\description{
This function is used to load both the Raw Data and the Analysis Results
    which are returned by the TSA software. Both output files have unique
    information regarding the experiment, and these need reunited for
    downstream analysis. Automatically generated Well IDs are use to merge
    similar data within the same experiment from the different files. Both
    Raw Data and the Analysis Results files must be specified. The returned,
    merged results from this function are required for downstream analysis
    as the format is set up for the automated workflow.
}
\section{IDs}{

    The TSAR package relies on matching conditions and file names for each
    well and for each set of conditions between multiple files output by
    the TSA software. Conditions are assigned to individual wells within
    the TSA software; these assigned values are detected by
    \code{\link{read_analysis}} and
    \code{\link{read_raw_data}} then are converted into IDs.
    Ensure your labeling of values within the TSA software is consistent
    so that similar values can be merged - typos or varying terms will be
    treated as distinct values within TSAR unless the values
    are manually specified by the user.
    Automatically generated well IDs within a TSA file can be found using the
    \code{\link{well_IDs}} function; condition IDs can be found using the
    \code{\link{condition_IDs}} function. \cr\cr


    \strong{Condition IDs} are generated only in the
    \code{\link{read_analysis}}, see that function's documentation for
    more details. Condition IDs are assigned to raw data in the
    \code{\link{merge_TSA}} function.
     \cr\cr

    \strong{Well IDs} are similar to Condition IDs, as they are
    generated from columns in TSA output. Well IDs are used to match
    the analysis and raw data files for the same experiment, as both files
    contain unique, useful information for each well.
    The well ID includes the .eds file name saved from the
    PCR machine to match equivalent wells between files of the same
    experiment. Each well on all plates should have a unique well ID.
    If you wish to change or specify the file name used for the well ID,
    a new name can be manually assigned with the "manual_file" argument.
    \cr\cr
}

\examples{
# note: example does not contain example data to run
# merge_TSA(analysis_file_path, raw_data_path)

}
\seealso{
\code{\link{read_raw_data}} and \code{\link{read_analysis}}
    for loading data.

Other TSAR Formatting: 
\code{\link{TSA_Tms}()},
\code{\link{TSA_average}()},
\code{\link{Tm_difference}()},
\code{\link{merge_norm}()},
\code{\link{normalize_fluorescence}()},
\code{\link{rescale}()}
}
\concept{TSAR Formatting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{model_gam}
\alias{model_gam}
\title{Generalized Addidtive Modeling on TSA data}
\usage{
model_gam(norm_data, x, y)
}
\arguments{
\item{norm_data}{data frame input of only one well's reading, preferably
normalized using \code{\link{normalize}}.}

\item{x}{temperature column}

\item{y}{normalized fluorescence column}
}
\value{
data frame containing gam model fitted values
}
\description{
Function finds fitted fluorescence values by imposing generalized
    additive model on fluorescence data by temperature. Model assumes
    method = "GACV.Cp" and sets to \code{formula = y ~ s(x, bs = "ad")}.
    Function inherits function from gam package, \code{\link{gam}}().
}
\examples{
data("qPCR_data1")
test <- subset(qPCR_data1, Well.Position == "A01")
test <- normalize(test, fluo = 5, selected = c(
    "Well.Position", "Temperature",
    "Fluorescence", "Normalized"
))
model_gam(test, x = test$Temperature, y = test$Normalized)

}
\seealso{
Other data_preprocess: 
\code{\link{model_boltzmann}()},
\code{\link{model_fit}()},
\code{\link{normalize}()},
\code{\link{remove_raw}()},
\code{\link{run_boltzmann}()},
\code{\link{screen}()},
\code{\link{view_model}()},
\code{\link{weed_raw}()}
}
\concept{data_preprocess}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rescaleTSA.R
\name{rescale}
\alias{rescale}
\title{Rescale values between minimum and maximum.}
\usage{
rescale(x)
}
\arguments{
\item{x}{a numeric vector to be rescaled}
}
\value{
A numeric vector of rescaled values.
}
\description{
For a vector of numeric values, the minimum and maximum values are determined
    and each value of the vector is rescaled between 0 and 1.
    Values near 0 are close to the minimum, values near 1 are close to the
    max. This function is utilized by other TSAR functions.
}
\examples{
x <- c(0, 1, 3)
rescale(x)

}
\seealso{
Other TSAR Formatting: 
\code{\link{TSA_Tms}()},
\code{\link{TSA_average}()},
\code{\link{Tm_difference}()},
\code{\link{merge_TSA}()},
\code{\link{merge_norm}()},
\code{\link{normalize_fluorescence}()}
}
\concept{TSAR Formatting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qPCR_data2.R
\name{qPCR_data2}
\alias{qPCR_data2}
\title{qPCR_data2 Dataset}
\format{
A data frame with the following columns:
    \describe{
      \item{Well}{Well Count, not required for user}
      \item{Well.Position}{Well Label, i.e. A01; required input}
      \item{Reading}{reading count in time series, not required for user}
      \item{Temperature}{temperature reading, required input}
      \item{Fluorescence}{fluorescence reading, required input}
    }
}
\usage{
data(qPCR_data2)
}
\value{
qPCR_data2 data frame
}
\description{
Dataset Description:
    This dataset contains qPCR data for the CA121 protein and common
    vitamins. It provides fluorescence measurements obtained using
    QuantStudio3. A different experiemnt trial containing data of similar
    property as data, qPCR_data1.
    Dataset is experimentally obtained by author of this package.
}
\keyword{dataset}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsa_curves.R
\name{TSA_boxplot}
\alias{TSA_boxplot}
\title{TSA Box Plot}
\usage{
TSA_boxplot(
  tsa_data,
  control_condition = NA,
  color_by = "Protein",
  label_by = "Ligand",
  separate_legend = TRUE
)
}
\arguments{
\item{tsa_data}{a data frame that is merged and generated by
TSAR::merge_TSA(). If \code{y = 'RFU'}, tsa_data must also be
generated by TSAR::normalize_fluorescence. The Temperature column will be
rounded and the average & sd of each rounded temperature is calculated.}

\item{control_condition}{Either a condition_ID or NA; NA by default.
When a valid Condition ID is provided, a vertical line appears at the
average Tm for the specified condition. When NA, this is skipped.}

\item{color_by}{character string, either c("Ligand", "Protein").
The condition category to color the boxes within the box
plot for comparison. This is represented in the legend.
Set to NA to skip.}

\item{label_by}{character string, either c("Ligand", "Protein").
The condition category to group the boxes within the box
plot. This is represented in the axis. Set to NA to skip.}

\item{separate_legend}{logical; \code{separate_legend = TRUE} by default.
When TRUE, the ggplot2 legend is separated from the TSA curve.
This is to help with readability. One ggplot is returned when FALSE.}
}
\value{
by default, two ggplots are returned: one TSA curve and one key.
    When \code{separate_legend = FALSE} one ggplot is returned.
}
\description{
Generates a box and whiskers plot for each condition specified. This
    is used to compare Tm values between the data set.
    See \code{\link{Tm_difference}} for details.
}
\examples{
data("example_tsar_data")
TSA_boxplot(example_tsar_data,
    color_by = "Protein",
    label_by = "Ligand", separate_legend = FALSE
)
}
\seealso{
\code{\link{merge_TSA}}
    for preparing data. See \code{\link{Tm_difference}} and
    \code{\link{get_legend}} for details on function parameters.

Other TSA Plots: 
\code{\link{TSA_compare_plot}()},
\code{\link{TSA_wells_plot}()},
\code{\link{get_legend}()},
\code{\link{graph_tsar}()},
\code{\link{view_deriv}()}
}
\concept{TSA Plots}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_tsar_data.R
\name{example_tsar_data}
\alias{example_tsar_data}
\title{Example tsar_data file}
\format{
A data frame with the following columns:
    \describe{
      \item{Well}{Well position}
      \item{Temperature}{Temperature in degrees}
      \item{Fluorescence}{Fluorescence reading}
      \item{Normalized}{Normalized value}
      \item{norm_deriv}{Calculated first derivative}
      \item{Tm}{Tm value}
      \item{Protein}{Protein information}
      \item{Ligand}{Ligand information}
      \item{ExperimentFileName}{Experiment file name}
      \item{well_ID}{Well ID}
      \item{condition_ID}{Condition ID}
    }
}
\source{
experimentally obtained
}
\usage{
data(example_tsar_data)
}
\value{
example tsar_data in data frame
}
\description{
Dataset Description:
    This is an example dataset of the tsar_data strucutre. The data
    frame contains well ID, conditions, and experimental details.
}
\keyword{dataset}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screen.R
\name{screen}
\alias{screen}
\title{Screen raw curves}
\usage{
screen(raw_data, checkrange = NULL, checklist = NULL)
}
\arguments{
\item{raw_data}{input raw_data}

\item{checkrange}{list type input identifying range of wells to select.
For example, if viewing first 8 wells from row A to C is needed, one can
specify the row letters and column numbers like this:
\code{checkrange = c("A", "C", "1", "8")}}

\item{checklist}{use this parameter to view selected Wells with full
Well names. For example, \code{checklist = c('A01', 'D11')}}
}
\value{
returns a ggplot graph colors by well IDs
}
\description{
screens multiple wells of data and prepares to assist identification of
    corrupted wells and odd out behaviors
}
\examples{
data("qPCR_data1")
screen(qPCR_data1, checkrange = c("A", "C", "1", "12"))

}
\seealso{
Other data_preprocess: 
\code{\link{model_boltzmann}()},
\code{\link{model_fit}()},
\code{\link{model_gam}()},
\code{\link{normalize}()},
\code{\link{remove_raw}()},
\code{\link{run_boltzmann}()},
\code{\link{view_model}()},
\code{\link{weed_raw}()}
}
\concept{data_preprocess}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screen.R
\name{remove_raw}
\alias{remove_raw}
\title{Remove selected raw curves}
\usage{
remove_raw(raw_data, removerange = NULL, removelist = NULL)
}
\arguments{
\item{raw_data}{dataframe; to be processed data}

\item{removerange}{list type input identifying range of wells to select.
For example, if removing all 12 wells from row D to H is needed, one can
specify the row letters and column numbers like this:
\code{removerange = c("D", "H", "1", "12")}}

\item{removelist}{use this parameter to remove selected Wells with full
Well names. For example, \code{removelist = c('A01', 'D11')}}
}
\value{
dataframe; data frame with specified well removed
}
\description{
Removes selected curves with specified wells and range.
}
\examples{
data("qPCR_data1")
remove_raw(qPCR_data1, removelist = c("A01", "D11"))

}
\seealso{
Other data_preprocess: 
\code{\link{model_boltzmann}()},
\code{\link{model_fit}()},
\code{\link{model_gam}()},
\code{\link{normalize}()},
\code{\link{run_boltzmann}()},
\code{\link{screen}()},
\code{\link{view_model}()},
\code{\link{weed_raw}()}
}
\concept{data_preprocess}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TSA_Tms.R
\name{TSA_Tms}
\alias{TSA_Tms}
\title{Reformat TSA data into TSA Tms}
\usage{
TSA_Tms(analysis_data, condition_average = TRUE)
}
\arguments{
\item{analysis_data}{a data frame that is unmerged and generated by
TSAR::read_analysis() or a merged TSA data frame generated by
TSAR::merge_TSA(). Data frames require a column named 'condition_ID' for
averaging.}

\item{condition_average}{logical value; \code{n = TRUE} by default.
When TRUE, the average Tm is calculated by matched condition IDs within
the data frame. When FALSE, each well is reported as a unique value with
the corresponding Tm.}
}
\value{
A data frame of Tm values.
}
\description{
This function is used to output calculated Tm data from TSA analysis. The
    input data frame will be transformed into
    a new format that is helpful for user reading and automated analysis.
    The Tm values can be listed as a data frame of individual wells or the
    Tms from identical conditions can be averaged. When condition_average
    is TRUE (the default), samples with identical condition IDs will be
    aggregated and the average / standard deviation will be calculated
    where appropriate. To analyze multiple TSA experiments, use merge_TSA()
    to make a single data frame for analysis.
}
\examples{
data("example_tsar_data")
TSA_Tms(example_tsar_data)
}
\seealso{
\code{\link{merge_TSA}}, \code{\link{read_raw_data}},
    and \code{\link{read_analysis}} for preparing input.

Other TSAR Formatting: 
\code{\link{TSA_average}()},
\code{\link{Tm_difference}()},
\code{\link{merge_TSA}()},
\code{\link{merge_norm}()},
\code{\link{normalize_fluorescence}()},
\code{\link{rescale}()}
}
\concept{TSAR Formatting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screen.R
\name{view_deriv}
\alias{view_deriv}
\title{View Derivative Curves}
\usage{
view_deriv(tsar_data, frame_by = "Well")
}
\arguments{
\item{tsar_data}{dataset input, analyzed must have norm_deriv as a variable;
dataset qualifying norm_data or tsar_data both fulfills this parameter,
although tsar_data is more recommended given more data options.}

\item{frame_by}{builds plotly by specified frame variable. To graph by a
concentration gradient, well position, or other specified variable,
simple specify \code{frame_by = "condition_ID"}. To view all derivative
curves without frames, set to \code{frame_by = "NA"}, else it is
defaulted to frame by well labels.}
}
\value{
plotly object of derivative curves
}
\description{
Function reviews data by well and output a graph of the all derivatives
    wanted. Function called within graph_tsar function but also runnable
    outside.
}
\examples{
data("example_tsar_data")
view_deriv(example_tsar_data, frame_by = "condition_ID")

}
\seealso{
Other TSA Plots: 
\code{\link{TSA_boxplot}()},
\code{\link{TSA_compare_plot}()},
\code{\link{TSA_wells_plot}()},
\code{\link{get_legend}()},
\code{\link{graph_tsar}()}
}
\concept{TSA Plots}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsa_analysis.R
\name{gam_analysis}
\alias{gam_analysis}
\title{Analysis of all 96 wells through gam modeling}
\usage{
gam_analysis(
  raw_data,
  keep = TRUE,
  fit = FALSE,
  smoothed = FALSE,
  boltzmann = FALSE,
  fluo_col = NA,
  selections = c("Well.Position", "Temperature", "Fluorescence", "Normalized")
)
}
\arguments{
\item{raw_data}{data frame; raw data frame}

\item{keep}{Boolean; set to \code{keep = TRUE}
by default to return normalized data and fitted data}

\item{fit}{Boolean; set to \code{fit = FALSE} by default, \code{fit = TRUE}
returns access to information of each model fit. Not accessible in shiny.}

\item{smoothed}{Boolean; set to \code{smoothed = FALSE} by default,
if data is already smoothed, set smoothed to true}

\item{boltzmann}{Boolean; set to \code{boltzmann = FALSE} by default. Set
to \code{boltzmann = TRUE} if a botlzmann fit is preferred.}

\item{fluo_col}{integer; the Fluorescence variable column id
(e.g. fluo = 5 when 5th column of data frame is the Fluorescence value)
if fluorescence variable is named exactly as "Fluorescence", fluo does
not need to be specified.}

\item{selections}{list of characters; the variables in raw data user intends
to keep. It is set, by default, to \code{c("Well.Position",
"Temperature", "Fluorescence", "Normalized")}.}
}
\value{
List of data frames, list of three data frame outputs,
    Tm estimation by well, data set, fit of model by well.
}
\description{
Function pipeline that combines separated functions and iterate through
    each well to estimate the Tm.
}
\examples{
data("qPCR_data1")
gam_analysis(qPCR_data1,
    smoothed = TRUE, boltzmann = FALSE, fluo_col = 5,
    selections = c(
        "Well.Position", "Temperature", "Fluorescence",
        "Normalized"
    )
)
model <- gam_analysis(qPCR_data1, smoothed = FALSE, boltzmann = TRUE)

}
\seealso{
Other tsa_analysis: 
\code{\link{Tm_est}()}
}
\concept{tsa_analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAnalysis.R
\name{read_analysis}
\alias{read_analysis}
\title{Read TSA Analysis Data}
\usage{
read_analysis(
  path,
  type = "derivative",
  conditions = c("Protein", "Ligand"),
  manual_conditions = NA,
  manual_wells = NA,
  skip_flags = FALSE,
  manual_file = NA
)
}
\arguments{
\item{path}{a character string; the path or the name of the file which the
'AnalysisResults' data are to be read from. Either a .txt or .csv file.
 The path must contain the term \emph{AnalysisResults} as the TSA software
 automatically assigns this when exporting data.}

\item{type}{either c("boltzmann", "derivative"); \code{type = "derivative")}
by default. Determines what model of
Tm estimation to load from the TSA software. Loads Tms as 'Tm B'  when
\code{type = "boltzmann")}; loads Tms as 'Tm D' when
\code{type = "derivative")}.}

\item{conditions}{A character vector of condition types assigned within the
TSA software to load.
\code{conditions = c("Protein", "Ligand")} by default.
These conditions are used to generate the IDs discussed.}

\item{manual_conditions, manual_wells}{NA by default,
enabling automated analysis.
A character vector of Condition IDs and Well IDs to manually assign
each row of the read data.}

\item{skip_flags}{logical value; \code{type = FALSE} by default.
When \code{type = TRUE}, wells that have flags reported by TSA software
are removed.}

\item{manual_file}{NA by default. User can specify .eds for merging if needed
for Well IDs if needed with a character string.}
}
\value{
A data frame of TSA analysis data.
}
\description{
Open TSA Analysis files. This function is used to load data output from
    the thermal shift software
    analysis tab. Can be either .txt or .csv file with a path / file name
    including the string "AnalysisResults" due to its automatic naming from
    the software. The values assigned to wells within the TSA software are
    automatically extracted from the loaded file; values must be assigned
    within the TSA software for the automated workflow (See IDs Section
    Below).
    \strong{Note:} Wells that do not have an Analysis Group assigned are
    removed.
    The TSA software automatically assigns all wells to Analysis Group 1 by
    default, and can be changed but not removed by the software.
}
\section{IDs}{

    The TSAR package relies on matching conditions and file names for each
    well and for each set of conditions between multiple files output by
    the TSA software. Conditions are
    assigned to individual wells within the TSA software; these assigned
    values are detected by
    \code{\link{read_analysis}} and
    \code{\link{read_raw_data}} then are converted into IDs.
    Ensure your labeling of values within the TSA software is consistent
    so that similar values can be merged - typos or varying terms will be
    treated as distinct values within TSAR unless the values
    are manually specified by the user.
    Automatically generated well IDs within a TSA file can be found using the
    \code{\link{well_IDs}} function; condition IDs can be found using the
    \code{\link{condition_IDs}} function. \cr\cr


    \strong{Condition IDs} are generated from columns in TSA output
    specified by the 'conditions' argument.
    Protein and Ligand values, the default conditions within the TSA
    software, are the values used to create these IDs. You can manually
    specify the
    condition categories from the TSA software, including user-made
    conditions. Condition IDs are used to match equivalent observations
    between technical and biological replicates. Wells with identical
    condition IDs, specified by the 'conditions' argument, will be
    aggregated in down-stream analysis; user-specified conditions must
    remain consistent in use and order to create compatible IDs between TSA
    files from the same experiment and between replicates. \cr\cr

    \strong{Well IDs} are similar to Condition IDs, as they are
    generated from columns in TSA output that are
    specified by the 'conditions' argument. Well IDs are used to match
    the analysis and raw data files for the same experiment, as both files
    contain unique, useful information for each well. In addition to the
    condition ID, the well ID includes the .eds file name saved from the
    PCR machine to match equivalent wells between files of the same
    experiment. Each well on all plates should have a unique well ID.
    If you wish to change or specify the file name used for the well ID,
    a new name can be manually assigned with the "manual_file" argument.
    \cr\cr

    The user may manually assign condition IDs using the
    'manual_conditions' argument rather than using the automatically
    generated IDs. The same is true for well IDs, which can be manually
    assigned with 'manual_wells'. This is not suggested, as there may be
    issues with matching if well/conditions are not properly matching. This
    gives the potential for errors in downstream applications as well.
}

\examples{
path <- "~/Desktop/analysis_data"
# note: example does not contain example data to run
# read_analysis(path)
}
\seealso{
\code{\link{read_raw_data}} for loading accompanying data.
    \code{\link{merge_TSA}} for joining Analysis Results and Raw Data files
    from the TSA software.

Other Read TSA Data: 
\code{\link{read_raw_data}()}
}
\concept{Read TSA Data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{numerical_Tm}
\alias{numerical_Tm}
\title{Numerical Tm by treating Tm as inflection point}
\usage{
numerical_Tm(data, x, y)
}
\arguments{
\item{data}{data.frame with numeric columns x and y (specified by names).
Data will be sorted by x; duplicated x's are averaged.}

\item{x, y}{character; names of the predictor and response columns.}
}
\value{
Named numeric vector \code{c(x = x_Tm, y = y_Tm)}; \code{NA}s on failure.
}
\description{
Estimates the inflection point (Tm) as the (x, y) coordinate where the
discrete second derivative changes sign (zero-crossing). Among all
zero-crossings, chooses the one with the largest absolute first derivative
(steepest slope). Uses linear interpolation between adjacent points to refine
the zero of the second derivative.
}
\examples{
# tm_xy <- numerical_Tm(df, x = "Temperature", y = "Normalized")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weed_raw.R
\name{weed_raw}
\alias{weed_raw}
\title{Weed raw data for corrupt curves}
\usage{
weed_raw(raw_data, checkrange = NULL, checklist = NULL)
}
\arguments{
\item{raw_data}{The raw data for screening.}

\item{checkrange}{list type input identifying range of wells to select.
For example, if viewing first 8 wells from row A to C is needed, one can
specify the row letters and column numbers like this:
\code{checkrange = c("A", "C", "1", "8")}}

\item{checklist}{use this parameter to view selected Wells with full
Well names. For example, \code{checklist = c('A01', 'D11')}}
}
\value{
prompts separate app window for user interaction,
    does not return specific value
}
\description{
The weed_raw function allows users to interact with a screening graph
    and select curves to weed out before entering analysis. Function wraps
    together \code{\link[TSAR]{screen}} and \code{\link{remove_raw}}.
}
\examples{
data("qPCR_data1")
if (interactive()) {
    runApp(weed_raw(qPCR_data1, checkrange = c("A", "B", "1", "12")))
}

}
\seealso{
\code{\link[TSAR]{screen}} and \code{\link{remove_raw}}

Other data_preprocess: 
\code{\link{model_boltzmann}()},
\code{\link{model_fit}()},
\code{\link{model_gam}()},
\code{\link{normalize}()},
\code{\link{remove_raw}()},
\code{\link{run_boltzmann}()},
\code{\link{screen}()},
\code{\link{view_model}()}
}
\concept{data_preprocess}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsa_curves.R
\name{TSA_compare_plot}
\alias{TSA_compare_plot}
\title{Compare TSA curves to control}
\usage{
TSA_compare_plot(
  tsa_data,
  control_condition,
  y = "Fluorescence",
  show_Tm = TRUE,
  title_by = "both",
  digits = 1,
  smoother = c("gam", "beta", "none"),
  smooth_conditions = TRUE,
  beta_shape = 3,
  beta_n_knots = NULL,
  beta_knots_frac = 0.12,
  use_natural = TRUE
)
}
\arguments{
\item{tsa_data}{a data frame that is merged and generated by
TSAR::merge_TSA(). If \code{y = 'RFU'}, tsa_data must also be
generated by TSAR::normalize_fluorescence. The Temperature column will be
rounded and the average & sd of each rounded temperature is calculated.}

\item{control_condition}{character string matching a Condition ID. Must be
equal to a value within tsa_data$condition_ID. See unique condition IDs
with \code{\link{condition_IDs}}.}

\item{y}{character string; c('Fluorescence', 'RFU').
When \code{y = 'Fluorescence'}, the original Fluorescence data from
TSAR::read_raw_data() is averaged. When \code{y = 'RFU'}, the average is
calculated by the rescaled fluorescence.}

\item{show_Tm}{logical; \code{show_Tm = TRUE} by default. When TRUE, the Tm
is displayed on the plot. When FALSE, the Tm is not added to the plot.}

\item{title_by}{character string; c("ligand", "protein", "both").
Automatically names the plots by the specified condition category.}

\item{digits}{integer; the number of decimal places to round for change in Tm
calculations displayed in the subtitle of each plot.}

\item{smoother}{character; one of \code{c("gam","beta","none")}.
Passed to \code{\link{TSA_average}} to select the aggregate smoother:
\code{"gam"} uses \pkg{mgcv}, \code{"beta"} uses a natural cubic spline
with Beta(a,a) interior knots centered at Tm, and \code{"none"} uses the
unsmoothed average. Default follows \code{TSA_average}.}

\item{smooth_conditions}{logical; \code{smooth_conditions = TRUE} by default.
Controls whether each non-control condition curve is drawn using smoothed
aggregates from \code{\link{TSA_average}}. When \code{TRUE}, the condition
overlay uses \code{avg_smooth} and the smoothed standard deviation bounds
(\code{sd_min_smooth}, \code{sd_max_smooth}) when available. When \code{FALSE},
the overlay uses the unsmoothed mean (\code{average}) and raw bounds
(\code{sd_min}, \code{sd_max}). The control curve remains smoothed.
(Passed as \code{avg_smooth} and \code{sd_smooth} to \code{TSA_average}.)}

\item{beta_shape}{numeric; shape parameter \eqn{a} for the Beta(a,a) knot
placement when \code{smoother = "beta"}. \code{beta_shape = 3} by default. Passed to \code{TSA_average}.}

\item{beta_n_knots}{integer or \code{NULL}; number of interior knots when
\code{smoother = "beta"}. If \code{NULL}, uses \code{beta_knots_frac}.
Passed to \code{TSA_average}.}

\item{beta_knots_frac}{numeric in (0,1); fraction of unique temperatures used
as interior knots when \code{smoother = "beta"} and \code{beta_n_knots}
is \code{NULL}. \code{beta_knots_frac = 0.008} by default. Passed to \code{TSA_average}.}

\item{use_natural}{logical; if TRUE (default) uses natural cubic spline
basis for the beta method. Passed to \code{TSA_average}.}
}
\value{
A \emph{named list} of ggplot objects. One plot per non-control
    condition is included, each overlaid against the control. A final plot for
    the control alone is appended and named \code{"Control: <control_condition>"}.
}
\description{
Generate a number of plots based on the input data to compare the average
and standard deviation (sd) of each unique condition to a specified
control condition. To see all conditions use \code{condition_IDs(tsa_data)}.
}
\examples{
data("example_tsar_data")
TSA_compare_plot(example_tsar_data,
    y = "Fluorescence",
    control_condition = "CA FL_DMSO",
    smoother = "beta",
    beta_shape = 4, 
    beta_knots_frac = 0.008
)
}
\seealso{
\code{\link{merge_TSA}} and \code{\link{normalize_fluorescence}}
    for preparing data. See \code{\link{TSA_average}} and
    \code{\link{get_legend}} for details on function parameters.
    See \code{\link{TSA_wells_plot}} for individual curves of the averaged
    conditions shown.

Other TSA Plots: 
\code{\link{TSA_boxplot}()},
\code{\link{TSA_wells_plot}()},
\code{\link{get_legend}()},
\code{\link{graph_tsar}()},
\code{\link{view_deriv}()}
}
\concept{TSA Plots}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_write_analysis.R
\name{write_tsar}
\alias{write_tsar}
\title{write output files}
\usage{
write_tsar(data, name, file = "txt")
}
\arguments{
\item{data}{input data frame}

\item{name}{string, name file to be saved as. Final name will be appended
"tsar_output"}

\item{file}{\code{file = "txt"} writes txt output files;
\code{file = "csv"} writes csv output files;
default set to \code{file = "txt"}}
}
\value{
file output on the working directory where data was read in
}
\description{
writes output into csv or txt files
}
\examples{
data("qPCR_data1")
result <- gam_analysis(qPCR_data1,
    smoothed = TRUE, fluo_col = 5,
    selections = c(
        "Well.Position", "Temperature", "Fluorescence", "Normalized"
    )
)
output_data <- read_tsar(result, output_content = 2)
# example does not run, will build excessive file in package
# write_tsar(output_data, name = "2022_03_18_test", file = "txt")

}
\seealso{
Other read_write_analysis: 
\code{\link{join_well_info}()},
\code{\link{read_tsar}()}
}
\concept{read_write_analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_write_analysis.R
\name{read_tsar}
\alias{read_tsar}
\title{Read analysis result}
\usage{
read_tsar(gam_result, output_content)
}
\arguments{
\item{gam_result}{list; input uses resulting output of
\code{\link{gam_analysis}}() function}

\item{output_content}{integer;
\code{output_content = 0} returns only the tm value by wells
\code{output_content = 1} returns data table with fitted values
\code{output_content = 2} returns the combination of 0 and 1}
}
\value{
output files with select dataset
}
\description{
reads previous pipeline output lists from \code{\link{gam_analysis}}()
    and organizes them into separate data frames.
}
\examples{
data("qPCR_data1")
result <- gam_analysis(qPCR_data1,
    smoothed = TRUE, fluo_col = 5,
    selections = c(
        "Well.Position", "Temperature", "Fluorescence", "Normalized"
    )
)
read_tsar(result, output_content = 0)
output_data <- read_tsar(result, output_content = 2)

}
\seealso{
Other read_write_analysis: 
\code{\link{join_well_info}()},
\code{\link{write_tsar}()}
}
\concept{read_write_analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{normalize}
\alias{normalize}
\title{Normalize Fluorescence}
\usage{
normalize(
  raw_data,
  fluo = NA,
  selected = c("Well.Position", "Temperature", "Fluorescence", "Normalized")
)
}
\arguments{
\item{raw_data}{data frame; raw dataset input, should be of only one well.
If multiple wells need to be normalized, use \code{\link{gam_analysis}}()
for 96 well application. If only preliminary screening is needed, use
\code{\link[TSAR]{screen}}().}

\item{fluo}{integer; the Fluorescence variable column id (e.g. fluo = 5
when 5th column of the data frame is the Fluorescence value)
if fluorescence variable is named exactly as "Fluorescence", fluo does
not need to be specified. i.e. fluo is set to NA by default,
suggesting the variable is named "Fluorescence".}

\item{selected}{list of character strings;
variables from the original data set users intend to keep.
Variable default set to c("Well.Position", "Temperature", "Fluorescence",
"Normalized") if not otherwise specified. If data frame variables are
named differently, user needs to specify what column variables to keep.}
}
\value{
cleaned up data framed with selected columns
}
\description{
normalize() reads in raw_data. This function normalizes data by standardizing
    them according to maximum and minimum fluorescence per well, with maximum
    set to 1 and minimum set to 0. It also reformats data types by checking
    for potential error. i.e. a string specifying 100,000 will be read in
    as number, 100000, without issue.
    Function is applicable only to data of a single well, do not call on
    an entire data frame of all 96 well data. It is intended for single well
    screening purposes.
}
\examples{
data("qPCR_data1")
test <- subset(qPCR_data1, Well.Position == "A01")
normalize(test)

}
\seealso{
Other data_preprocess: 
\code{\link{model_boltzmann}()},
\code{\link{model_fit}()},
\code{\link{model_gam}()},
\code{\link{remove_raw}()},
\code{\link{run_boltzmann}()},
\code{\link{screen}()},
\code{\link{view_model}()},
\code{\link{weed_raw}()}
}
\concept{data_preprocess}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryTSA.R
\name{condition_IDs}
\alias{condition_IDs}
\title{TSAR Condition IDs}
\usage{
condition_IDs(analysis_data, n = FALSE)
}
\arguments{
\item{analysis_data}{a data frame that is unmerged and generated by
TSAR::read_analysis() or a merged TSA data frame generated by
TSAR::merge_TSA(). Data frames require a column named 'condition_ID'.}

\item{n}{logical value; \code{n = FALSE} by default. When TRUE, a numeric
value of unique IDs is returned. When FALSE, a character vector of unique
IDs are returned.}
}
\value{
Either a character vector of condition_IDs or a numeric value.
}
\description{
This function is used to extract information of the condition IDs from a
    loaded TSA Analysis Data file. Condition IDs are automatically generated
    by the read_analysis function in the automated workflow. This returns
    either a character vector of unique IDs present or a numeric value of the
    number of unique IDs.
}
\examples{
data("example_tsar_data")
condition_IDs(example_tsar_data)

}
\seealso{
\code{\link{merge_TSA}} and \code{\link{read_analysis}}
for preparing input.

Other TSA Summary Functions: 
\code{\link{TSA_ligands}()},
\code{\link{TSA_proteins}()},
\code{\link{well_IDs}()}
}
\concept{TSA Summary Functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze_norm.R
\name{analyze_norm}
\alias{analyze_norm}
\title{Analyze to Normalize}
\usage{
analyze_norm(raw_data)
}
\arguments{
\item{raw_data}{The raw data for analysis.}
}
\value{
shiny application
}
\description{
The analyze_norm function allows users to process analysis through an UI
    interface. Function wraps together all functions with in TSA_analysis
    family and read_write_analysis family.
}
\examples{
if (interactive()) {
    data("qPCR_data1")
    shiny::runApp(analyze_norm(qPCR_data1))
}

}
\seealso{
\code{\link[TSAR]{gam_analysis}}, \code{\link{read_tsar}},
  \code{\link{write_tsar}}, \code{\link{join_well_info}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryTSA.R
\name{well_IDs}
\alias{well_IDs}
\title{TSAR Well IDs}
\usage{
well_IDs(tsa_data, n = FALSE)
}
\arguments{
\item{tsa_data}{a data frame that is merged and generated by
TSAR::merge_TSA(), or an unmerged data frame read by
TSAR::read_analysis() or TSAR::read_raw_data().
Data frames require a column named 'well_ID'.}

\item{n}{logical value; \code{n = FALSE} by default. When TRUE, a numeric
value of unique IDs is returned. When FALSE, a character vector of unique
IDs are returned.}
}
\value{
Either a character vector of unique well IDs or a numeric value.
}
\description{
This function is used to extract information of the well IDs from a
    merged TSA data frame. Well IDs are automatically generated
    by the read_analysis and read_raw_data functions in the automated
    workflow. This function returns
    either a character vector of unique IDs present or a numeric value of the
    number of unique IDs.
}
\examples{
data("example_tsar_data")
well_IDs(example_tsar_data)

}
\seealso{
\code{\link{merge_TSA}}, \code{\link{read_raw_data}},
and \code{\link{read_analysis}} for preparing input.

Other TSA Summary Functions: 
\code{\link{TSA_ligands}()},
\code{\link{TSA_proteins}()},
\code{\link{condition_IDs}()}
}
\concept{TSA Summary Functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rescaleTSA.R
\name{normalize_fluorescence}
\alias{normalize_fluorescence}
\title{Normalize Fluorescence Curve}
\usage{
normalize_fluorescence(tsa_data = tsa_data, by = "rescale", control_vect = NA)
}
\arguments{
\item{tsa_data}{a data frame that is unmerged and generated by
 TSAR::read_raw_data() or a merged TSA data frame generated by
 TSAR::merge_TSA(). Data frames require a column named "Fluorescence"
containing numeric values for normalizing.}

\item{by}{character string; either c("rescale", "min", "max", "control").
\code{by = "rescale"} by default, scaling Fluorescence values in-between
the minimum and maximum observation. Each well can be
normalized to either the minimum or maximum value with
\code{by = "min"} or \code{by = "max"}, respectively. To normalize
all values to a numeric value or vector, set \code{by = "control"}.}

\item{control_vect}{numeric vector to normalize the column "Fluorescence"
to. An individual number will normalize all measurements to be normalized
to it. The vector will need to align with tsa_data$Fluorescence. Ensure
\code{by = "control"}, else the supplied vector will be ignored.}
}
\value{
a data frame identical to the tsa_data input with a new column
    named "RFU" containing the normalized values
}
\description{
This function will take the TSA data and normalize the arbitrary fluorescence
    measurements based on the specified method. Each well, determined by a
    unique well ID, is normalized independently.
    All measurements can be normalized to the minimum or maximum value.
    Alternatively, setting  \code{by = "rescale"} (the default) will cause
    all values to be normalized between the minimum and
    maximum values, with the maximum  = 1 and the minimum = 0 and all other
    values normalized in-between. Finally, the user can supply a single
    value or vector of values to normalize the data to. The returned data
    frame will be the input tsa data frame with a new column named "RFU"
    containing the normalized TSA data.
}
\examples{
# examples not ran without example dataset
# raw_data <- read_raw_data(raw_data_path)
# normalize_fluorescence(raw_data, by == "control)

}
\seealso{
\code{\link{read_raw_data}} and  \code{\link{merge_TSA}}
    for loading data.

Other TSAR Formatting: 
\code{\link{TSA_Tms}()},
\code{\link{TSA_average}()},
\code{\link{Tm_difference}()},
\code{\link{merge_TSA}()},
\code{\link{merge_norm}()},
\code{\link{rescale}()}
}
\concept{TSAR Formatting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/averageTSA.R
\name{TSA_smoother_diagnostics}
\alias{TSA_smoother_diagnostics}
\title{Diagnose Smoother Accuracy Across Conditions}
\usage{
TSA_smoother_diagnostics(
  tsa_data,
  y = c("Fluorescence", "RFU"),
  metric = c("L2", "L1", "R2"),
  beta_shapes = NULL,
  beta_fracs = NULL,
  digits = 1,
  use_natural = TRUE,
  beta_n_knots = NULL
)
}
\arguments{
\item{tsa_data}{a data frame that is merged and generated by
TSAR::merge_TSA(). If \code{y = 'RFU'}, tsa_data must also be
generated by TSAR::normalize_fluorescence. The Temperature column will be
rounded and the average & sd of each rounded temperature is calculated.}

\item{y}{character string; c('Fluorescence', 'RFU').
When \code{y = 'Fluorescence'}, the original Fluorescence data from
TSAR::read_raw_data() is averaged. When \code{y = 'RFU'}, the average is
calculated by the rescaled fluorescence.}

\item{metric}{character; one of \code{c("L2","L1","R2")} with default
\code{"L2"}. Defines the accuracy metric between \code{average} (truth) and
\code{avg_smooth} (fit):
\itemize{
  \item \code{"L1"}: mean absolute error (MAE)
  \item \code{"L2"}: root mean squared error (RMSE)
  \item \code{"R2"}: coefficient of determination; higher is better
}}

\item{beta_shapes}{numeric vector or \code{NULL}; shapes \eqn{a} for the
Beta(a,a) knot placement when evaluating the \code{"beta"} smoother.
\strong{Default behavior:} if \code{NULL} or length 0, uses a single value
\code{4}.}

\item{beta_fracs}{numeric vector or \code{NULL}; fractions in \eqn{(0,1)}
giving the proportion of unique temperatures used as interior knots for the
\code{"beta"} smoother. \strong{Default behavior:} if \code{NULL} or length
0, uses a single value \code{0.008}.}

\item{digits}{integer; rounding applied to the Temperature grid before
aggregation. Default \code{1}. Passed to \code{\link{TSA_average}}.}

\item{use_natural}{logical; if \code{TRUE} (default) uses natural cubic spline
basis for the beta method. Passed to \code{\link{TSA_average}}.}

\item{beta_n_knots}{integer or \code{NULL}; number of interior knots for the
\code{"beta"} smoother. If \code{NULL} (default), the number of knots is
determined by \code{beta_knots_frac}. Passed through to
\code{\link{TSA_average}}.}
}
\value{
(Invisibly) a \code{data.frame} with columns:
\itemize{
  \item \code{condition_id}: condition label,
  \item \code{method}: \code{"gam"} or \code{beta(a=..., frac=...)} string,
  \item \code{error}: numeric value of the requested metric.
}
The function also prints a short textual summary to the console.
}
\description{
Computes per-condition fitting error for \code{gam} and \code{beta} smoothers
(as implemented in \code{\link{TSA_average}}) against the aggregated mean
curve. For each \code{condition_ID}, the function compares the smoothed mean
(\code{avg_smooth}) to the real aggregated mean (\code{average}) at
each temperature and reports the error in corresponding metric.
}
\details{
By default, it evaluates:
\itemize{
  \item \strong{GAM}: \code{smoother = "gam"}
  \item \strong{Beta (single spec)}: \code{beta_shape = 4}, \code{beta_knots_frac = 0.008}
}
If you supply vectors to \code{beta_shapes} and/or \code{beta_fracs}, the
function evaluates \emph{all} combinations for the beta method.
}
\examples{
\dontrun{
normalized_tsa_df

result <- TSA_smoother_diagnostics(normalized_tsa_df, y = "RFU", metric = "L2")
as.data.frame(result)
}
}
\seealso{
\code{\link{TSA_average}} (smoother implementation),
  \code{\link{TSA_wells_plot}} (visualization of per-well and averaged curves).
}
\concept{Diagnostics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/screen.R
\name{view_model}
\alias{view_model}
\title{View Model}
\usage{
view_model(raw_data)
}
\arguments{
\item{raw_data}{dataset input, not processing needed}
}
\value{
list of two ggplot graphs
}
\description{
Function reviews data by well and output a graph of the fit and a graph of
    derivative. Function called within analyze_norm function.
}
\examples{
data("qPCR_data1")
test <- subset(qPCR_data1, Well.Position == "A01")
test <- normalize(test)
gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
test <- model_fit(test, model = gammodel)
view_model(test)

}
\seealso{
Other data_preprocess: 
\code{\link{model_boltzmann}()},
\code{\link{model_fit}()},
\code{\link{model_gam}()},
\code{\link{normalize}()},
\code{\link{remove_raw}()},
\code{\link{run_boltzmann}()},
\code{\link{screen}()},
\code{\link{weed_raw}()}
}
\concept{data_preprocess}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{run_boltzmann}
\alias{run_boltzmann}
\title{Run Boltzmann Modeling}
\usage{
run_boltzmann(norm_data)
}
\arguments{
\item{norm_data}{data frame input, preferably normalized using
\code{\link{normalize}}.}
}
\value{
data frame containing gam model fitted values
}
\description{
Function runs function \code{model_boltzmann()} and raises warning when
    modeling generates error or warnings.
}
\examples{
data("qPCR_data1")
A01 <- subset(qPCR_data1, Well.Position == "A01")
A01 <- normalize(A01)
run_boltzmann(A01)

}
\seealso{
Other data_preprocess: 
\code{\link{model_boltzmann}()},
\code{\link{model_fit}()},
\code{\link{model_gam}()},
\code{\link{normalize}()},
\code{\link{remove_raw}()},
\code{\link{screen}()},
\code{\link{view_model}()},
\code{\link{weed_raw}()}
}
\concept{data_preprocess}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/norm_to_tsar.R
\name{merge_norm}
\alias{merge_norm}
\title{Merge and format norm_data into tsar_data}
\usage{
merge_norm(data, name, date)
}
\arguments{
\item{data}{list, a character vector specifying the file paths of the data
files or data frame objects of analysis data set. For example, given data
frames named "data1" and "data2", specify parameter as
\code{data = list(data1, data2)}.}

\item{name}{list, character vector specifying the experiment names.}

\item{date}{list, character vector specifying the dates. Does not require
any date format restrictions.}
}
\value{
data frame in the format of tsar_data
}
\description{
This function merges data of experiment replicates across different dates.
    It merges and produces information variables used to group wells of same
    set up.
}
\details{
This function merges and normalizes test data from multiple files.
    The lengths of the \code{data}, \code{name}, and \code{date} vectors
    must match, otherwise an error is thrown.
}
\examples{
data("qPCR_data1")
result <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
data("well_information")
norm_data <- join_well_info(
    file_path = NULL, file = well_information,
    read_tsar(result, output_content = 2), type = "by_template"
)
norm_data <- na.omit(norm_data)
data("qPCR_data2")
result2 <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
norm_data2 <- join_well_info(
    file_path = NULL, file = well_information,
    read_tsar(result2, output_content = 2), type = "by_template"
)
norm_data2 <- na.omit(norm_data2)
tsar_data <- merge_norm(
    data = list(norm_data, norm_data2),
    name = c("Thermal Shift_162.eds.csv", "Thermal Shift_168.eds.csv"),
    date = c("20230203", "20230209")
)

}
\seealso{
Other TSAR Formatting: 
\code{\link{TSA_Tms}()},
\code{\link{TSA_average}()},
\code{\link{Tm_difference}()},
\code{\link{merge_TSA}()},
\code{\link{normalize_fluorescence}()},
\code{\link{rescale}()}
}
\concept{TSAR Formatting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/averageTSA.R
\name{TSA_average}
\alias{TSA_average}
\title{Average TSA Curves}
\usage{
TSA_average(
  tsa_data,
  y = "Fluorescence",
  digits = 1,
  avg_smooth = TRUE,
  sd_smooth = TRUE,
  smoother = c("gam", "beta", "none"),
  beta_shape = 4,
  beta_n_knots = NULL,
  beta_knots_frac = 0.008,
  use_natural = TRUE
)
}
\arguments{
\item{tsa_data}{a data frame that is merged and generated by
TSAR::merge_TSA(). If \code{y = 'RFU'}, tsa_data must also be
generated by TSAR::normalize_fluorescence. The Temperature column will be
rounded and the average & sd of each rounded temperature is calculated.}

\item{y}{character string; c('Fluorescence', 'RFU').
When \code{y = 'Fluorescence'}, the original Fluorescence data from
TSAR::read_raw_data() is averaged. When \code{y = 'RFU'}, the average is
calculated by the rescaled fluorescence.}

\item{digits}{an integer; \code{digits = 1} by default.
The number of decimal places to round temperature to for averaging.}

\item{avg_smooth, sd_smooth}{logical; TRUE by default. Decides if the
average (avg_smooth) or standard deviation (sd_smooth) will be smoothened
by regression via mgcv::gam()}

\item{smoother}{character; one of \code{c("gam","beta","none")}.
Passed to \code{\link{TSA_average}} to select the aggregate smoother:
\code{"gam"} uses \pkg{mgcv}, \code{"beta"} uses a natural cubic spline
with Beta(a,a) interior knots centered at Tm, and \code{"none"} uses the
unsmoothed average. Default follows \code{TSA_average}.}

\item{beta_shape}{numeric; shape parameter \eqn{a} for the Beta(a,a) knot
placement when \code{smoother = "beta"}. \code{beta_shape = 3} by default.}

\item{beta_n_knots}{integer or \code{NULL}; number of interior knots when
\code{smoother = "beta"}. If \code{NULL}, uses \code{beta_knots_frac}.}

\item{beta_knots_frac}{numeric in (0,1); fraction of unique temperatures used
as interior knots when \code{smoother = "beta"} and \code{beta_n_knots}
is \code{NULL}. \code{beta_knots_frac = 0.008} by default.}

\item{use_natural}{logical; if TRUE (default) uses natural cubic spline
basis for the beta method.}
}
\value{
a data frame of each temperature measured with the average, sd, and
   n(# of averaged values) calculated. Depending on avg_smooth and sd_smooth,
   the smoothened lines for the maximum and mimimum sd and the average will
   also be returned.
}
\description{
This function will take either Fluorescence or Normalized Fluorescence
    curves from the submitted data frame and find the average (mean) and
    standard deviation (sd) for each temperature measured in the TSA curve.
    Mean and sd are smoothened by default to generate cleaner curves.
    The function \code{\link[mgcv]{gam}} from the mgcv package is used for
    regression to smoothen lines. Smoothing can be turned off and the true
    average for each point can be given, however, plots will look messier.
    The qPCR machine may return temperatures with many decimal places,
    and TSAR only merges identical values, therefore rounding is necessary.
    Data is rounded to one decimal place to improve regression smoothing.
    \cr \cr
    \strong{Note:} All submitted data is averaged, regardless of condition or
    well ID.
    If you wish to average by condition, you will need to sort the data frame
    and run this function on subsets.
}
\examples{
data("example_tsar_data")
TSA_average(example_tsar_data,
    y = "Fluorescence", digits = 1,
    avg_smooth = TRUE, sd_smooth = TRUE
)

}
\seealso{
\code{\link{merge_TSA}} and \code{\link{merge_TSA}} for
   preparing data.

Other TSAR Formatting: 
\code{\link{TSA_Tms}()},
\code{\link{Tm_difference}()},
\code{\link{merge_TSA}()},
\code{\link{merge_norm}()},
\code{\link{normalize_fluorescence}()},
\code{\link{rescale}()}
}
\concept{TSAR Formatting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_well_info.R
\name{join_well_info}
\alias{join_well_info}
\title{Well information input function}
\usage{
join_well_info(
  file_path,
  file = NULL,
  analysis_file,
  skips = 0,
  nrows = 96,
  type
)
}
\arguments{
\item{file_path}{string; file path to read in the file}

\item{file}{object; use file to override the need of file_path if
information is already read in}

\item{analysis_file}{data frame; data frame containing smoothed fluorescence
data and tm values}

\item{skips}{integer; number indicating the number of headers present in
input file, default set to 0 when file input is "by_well"
If the input follows the excel template, this parameter does not apply.}

\item{nrows}{integer; number indicating the number of rows the data is.
Default set to 96 assuming analysis on 96 well plate. Parameter is only
applicable when file input is "by_well". If inputting by excel
template, this parameter does not apply, please ignore.}

\item{type}{string; variable specifies the type of input read in.
type = "by_well" requires input of csv or txt files of three variables:
Well, Protein, Ligand.
type = "by_template" requires input of excel file following the template
format provided}
}
\value{
outputs data frame joining data information with well information
}
\description{
Reads in the ligand and protein information and joins them accordingly
    to the big data frame for graphing purposes.
}
\examples{
data("qPCR_data1")
result <- gam_analysis(qPCR_data1, smoothed = TRUE, fluo = 5)
data("well_information")
join_well_info(
    file_path = NULL, file = well_information,
    read_tsar(result, output_content = 2), type = "by_template"
)

}
\seealso{
Other read_write_analysis: 
\code{\link{read_tsar}()},
\code{\link{write_tsar}()}
}
\concept{read_write_analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qPCR_data1.R
\name{qPCR_data1}
\alias{qPCR_data1}
\title{qPCR_data1 Dataset}
\format{
A data frame with the following columns:
    \describe{
      \item{Well}{Well Count, not required for user}
      \item{Well.Position}{Well Label, i.e. A01; required input}
      \item{Reading}{reading count in time series, not required for user}
      \item{Temperature}{temperature reading, required input}
      \item{Fluorescence}{fluorescence reading, required input}
    }
}
\source{
experimentally obtained
}
\usage{
data(qPCR_data1)
}
\value{
qPCR_data1 data frame
}
\description{
Dataset Description:
    This dataset contains qPCR data for the CA121 protein and common
    vitamins. It provides fluorescence measurements obtained
    using QuantStudio3.
    Dataset is experimentally obtained by author of this package.
}
\keyword{dataset}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{model_fit}
\alias{model_fit}
\title{Refit and calculate derivative function}
\usage{
model_fit(norm_data, model, smoothed)
}
\arguments{
\item{norm_data}{data frame; the raw data set input}

\item{model}{fitted model containing fitted values}

\item{smoothed}{inform whether data already contains a smoothed model; Input
the column name of the smoothed data to override values of gam model
fitting. For example, existing "Fluorescence" column contains data
already smoothed, set \code{smoothed = "Flourescence"} to calculate
derivative function upon the called smoothed data.}
}
\value{
data frame; with calculated derivative columns
}
\description{
Model_fit calculates derivatives by refitting model onto data. Only runs
    on data of a single well.
}
\examples{
data("qPCR_data1")
test <- subset(qPCR_data1, Well.Position == "A01")
test <- normalize(test, fluo = 5, selected = c(
    "Well.Position", "Temperature",
    "Fluorescence", "Normalized"
))
gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
model_fit(test, model = gammodel)
# if data come smoothed, run ...
model_fit(test, smoothed = "Fluorescence")

}
\seealso{
Other data_preprocess: 
\code{\link{model_boltzmann}()},
\code{\link{model_gam}()},
\code{\link{normalize}()},
\code{\link{remove_raw}()},
\code{\link{run_boltzmann}()},
\code{\link{screen}()},
\code{\link{view_model}()},
\code{\link{weed_raw}()}
}
\concept{data_preprocess}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_tsar.R
\name{graph_tsar}
\alias{graph_tsar}
\title{Graph tsar_data}
\usage{
graph_tsar(tsar_data = data.frame())
}
\arguments{
\item{tsar_data}{tsar data outputted by merge_norm or merge_tsa.
Parameter is optional. If no data is passed, access the merge panel
to merge norm_data into tsar_data.}
}
\value{
prompts separate app window for user interaction,
    does not return specific value; generates boxplot and compare plots
    according to user input
}
\description{
Function allows users to graph out tsar_data, building boxplot, compare
    plots, and curves by condition. Input of data as parameter is optional.
    graph_tsar wraps together all graphing functions and relative helper
    functions.
}
\examples{
if (interactive()) {
    data("example_tsar_data")
    shiny::runApp(graph_tsar(example_tsar_data))
}

}
\seealso{
\code{\link{TSA_boxplot}}, \code{\link{TSA_compare_plot}},
    \code{\link{condition_IDs}}, \code{\link{well_IDs}},
    \code{\link{merge_norm}}, \code{\link{TSA_Tms}},
    \code{\link{Tm_difference}}

Other TSA Plots: 
\code{\link{TSA_boxplot}()},
\code{\link{TSA_compare_plot}()},
\code{\link{TSA_wells_plot}()},
\code{\link{get_legend}()},
\code{\link{view_deriv}()}
}
\concept{TSA Plots}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Tm_Diff.R
\name{Tm_difference}
\alias{Tm_difference}
\title{Calculate Tm difference for all conditions}
\usage{
Tm_difference(tsa_data, control_condition)
}
\arguments{
\item{tsa_data}{a data frame that is merged and generated by
TSAR::merge_TSA(). If \code{y = 'RFU'}, tsa_data must also be
generated by TSAR::normalize_fluorescence. The Temperature column will be
rounded and the average & sd of each rounded temperature is calculated.}

\item{control_condition}{character string matching a Condition ID. Must be
equal to a value within tsa_data$condition_ID. See unique condition IDs
with \code{\link{condition_IDs}}.}
}
\value{
a data frame of reformatted data with the \code{\link{TSA_average}}
    data and the Tm.
}
\description{
From a specified control condition, the change in Tm is calculated for
    each condition in the tsa_data. Specifically,
    \eqn{Tm = condition - control}. Individual Tm values are averaged by
    condition, see \code{\link{TSA_average}} for details.
    To see all conditions use \code{condition_IDs(tsa_data)}.
}
\examples{
data("example_tsar_data")
control <- condition_IDs(example_tsar_data)[1]
Tm_difference(example_tsar_data, control_condition = control)

}
\seealso{
\code{\link{merge_TSA}} for preparing data.
    \code{\link{TSA_average}} for more information on the output data.
    \code{\link{condition_IDs}} to get unique Condition IDs within the
    input.
    \code{\link{TSA_boxplot}} for application.

Other TSAR Formatting: 
\code{\link{TSA_Tms}()},
\code{\link{TSA_average}()},
\code{\link{merge_TSA}()},
\code{\link{merge_norm}()},
\code{\link{normalize_fluorescence}()},
\code{\link{rescale}()}
}
\concept{TSAR Formatting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsa_analysis.R
\name{Tm_est}
\alias{Tm_est}
\title{Find inflection point function}
\usage{
Tm_est(norm_data, min, max)
}
\arguments{
\item{norm_data}{data frame; data frame input containing derivative values
can only be data frames for one well; finding inflections points across
multiple wells require iteration through individual wells}

\item{min}{restricts finding to be above the given minimum temperature}

\item{max}{restricts finding to be below the given maximum temperature
parameter min and max can be used to remove messy or undesired data
for better accuracy in tm estimation; removing data is  before fitting
the model is more recommended than removing here}
}
\value{
integer; tm estimation
}
\description{
Looks for Tm temperature values by finding the inflection point in the
    fluorescence data. The inflection point is approximated by
    locating the maximum first derivative stored in "norm_deriv" column.
}
\examples{
data("qPCR_data1")
test <- subset(qPCR_data1, Well.Position == "A01")
test <- normalize(test, fluo = 5, selected = c(
    "Well.Position", "Temperature",
    "Fluorescence", "Normalized"
))
gammodel <- model_gam(test, x = test$Temperature, y = test$Normalized)
fit <- model_fit(test, model = gammodel)
Tm_est(fit)

}
\seealso{
Other tsa_analysis: 
\code{\link{gam_analysis}()}
}
\concept{tsa_analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaryTSA.R
\name{TSA_proteins}
\alias{TSA_proteins}
\title{TSA Proteins}
\usage{
TSA_proteins(tsa_data, n = FALSE)
}
\arguments{
\item{tsa_data}{a data frame that is merged and generated by
TSAR::merge_TSA(), or an unmerged data frame read by
TSAR::read_analysis() or TSAR::read_raw_data().
The data frame must have a column named 'Protein'.}

\item{n}{logical value; \code{n = FALSE} by default. When TRUE, a numeric
value describing the number of unique protein names is returned.
When FALSE, a character vector of unique IDs are returned.}
}
\value{
Either a character vector of unique well_IDs or a numeric value.
}
\description{
This function is used to extract information from a data frame of TSA data.
    The Protein values should be assigned in the TSA software.
}
\examples{
data("example_tsar_data")
TSA_proteins(example_tsar_data)

}
\seealso{
\code{\link{merge_TSA}}, \code{\link{read_raw_data}},
    and \code{\link{read_analysis}} for preparing input.

Other TSA Summary Functions: 
\code{\link{TSA_ligands}()},
\code{\link{condition_IDs}()},
\code{\link{well_IDs}()}
}
\concept{TSA Summary Functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{model_boltzmann}
\alias{model_boltzmann}
\title{Boltzmann Modeling on TSA data}
\usage{
model_boltzmann(norm_data)
}
\arguments{
\item{norm_data}{data frame input, preferably normalized using
\code{\link{normalize}}.}
}
\value{
dtaa frame containing gam model fitted values
}
\description{
Function finds fitted fluorescence values by imposing Boltzmann function.
}
\examples{
data("qPCR_data1")
A01 <- subset(qPCR_data1, Well.Position == "A01")
A01 <- normalize(A01)
model_boltzmann(A01)

}
\seealso{
Other data_preprocess: 
\code{\link{model_fit}()},
\code{\link{model_gam}()},
\code{\link{normalize}()},
\code{\link{remove_raw}()},
\code{\link{run_boltzmann}()},
\code{\link{screen}()},
\code{\link{view_model}()},
\code{\link{weed_raw}()}
}
\concept{data_preprocess}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readRawData.R
\name{read_raw_data}
\alias{read_raw_data}
\title{Read TSA Raw Data}
\usage{
read_raw_data(path, manual_file = NA, type = "fluorescence")
}
\arguments{
\item{path}{a character string; the path or the name of the file which the
'RawData' data are to be read from. Either a .txt or .csv file.
The path must contain the term \emph{RawData} as the TSA software
automatically assigns this when exporting data.}

\item{manual_file}{NA by default. User can specify .eds for merging if needed
for Well IDs with a character string.}

\item{type}{either c("boltzmann", "derivative", "fluorescence");
\code{type = "fluorescence")} by default. Determines what data track to
load. When \code{type = "fluorescence")}, the arbitrary fluorescence
of the TSA dye is  loaded; this is the primary data. Alternately,
derivatives van be loaded:  Loads data as boltzman estimated tracks when
\code{type = "boltzmann")}; loads the 2nd derivative of emissions when
\code{type = "derivative")}.}
}
\value{
A data frame of TSA raw data.
}
\description{
Open TSA Raw Data files. This function is used to load data output from
    the thermal shift software Raw Data tab.
    Can be either .txt or .csv file with a path / file name
    including the string "RawData" due to its automatic naming from
    the software.
    The values assigned to wells within the TSA software are
    automatically extracted from the loaded file; values must be assigned
    within the TSA software for the automated workflow
    (See IDs Section Below).
}
\section{IDs}{

    The TSAR package relies on matching conditions and file names for each
    well and for each set of conditions between multiple files output by
    the TSA software. Conditions are
    assigned to individual wells within the TSA software; these assigned
    values are detected by
    \code{\link{read_analysis}} and
    \code{\link{read_raw_data}} then are converted into IDs.
    Ensure your labeling of values within the TSA software is consistent
    so that similar values can be merged - typos or varying terms will be
    treated as distinct values within TSAR unless the values
    are manually specified by the user.
    Automatically generated well IDs within a TSA file can be found using the
    \code{\link{well_IDs}} function; condition IDs can be found using the
    \code{\link{condition_IDs}} function. \cr\cr


    \strong{Condition IDs} are generated only in the
    \code{\link{read_analysis}}, see that function's documentation for
    more details. Condition IDs are assigned to raw data in the
    \code{\link{merge_TSA}} function.
    \cr\cr

    \strong{Well IDs} are similar to Condition IDs, as they are
    generated from columns in TSA output. Well IDs are used to match
    the analysis and raw data files for the same experiment, as both files
    contain unique, useful information for each well.
    The well ID includes the .eds file name saved from the
    PCR machine to match equivalent wells between files of the same
    experiment. Each well on all plates should have a unique well ID.
    If you wish to change or specify the file name used for the well ID,
    a new name can be manually assigned with the "manual_file" argument.
    \cr\cr
}

\examples{
path <- "~/Desktop/raw_data"
# note: example does not contain example data to run
# read_raw_data(path)

}
\seealso{
\code{\link{read_analysis}} for loading accompanying data.
    \code{\link{merge_TSA}} for joining Analysis Results and Raw Data files
    from the TSA software.

Other Read TSA Data: 
\code{\link{read_analysis}()}
}
\concept{Read TSA Data}
